#
# -*- coding: utf-8 -*-

'''General things all querytool scripts need.
It also adjusts the sys path via biocase.adjustpath.py .
This module should be executed via execfile('ABS PATH TO HERE'). 
'''

############################################################################################################
#
#   IMPORTS
#
#===========================================================================================================

import cgitb; cgitb.enable()
import cgi, os, sys, string, md5, time, random
from stat import ST_MTIME
from types import *

import logging
import logging.handlers

import biocase.configuration
from biocase.wrapper.errorclasses import *
from biocase.configtool        import __version__
from biocase.datasources       import Datasource
from biocase.tools.htmltools   import ankerClass, escapeHtml, getDropDownHtml, getDropDownOptionHtml
from biocase.tools.templating  import PageMacro, genOptionList
from biocase.tools.various_functions import getEncodingList, getDBMSList



############################################################################################################
#
#   GENERAL FUNCTIONS
#
#===========================================================================================================
def printOverHTTP(templ):
    print cfg.http_header
    print str(templ)

# ------------------------------------------------------------------------------------
def authenticate(MD5passwd, clearPasswd, dsa=None):
    '''Check MD5 encrypted password with temporary DSA .password file.
    Removes file if older than 30 minutes.'''
    global cfg
    passwdFilename = '.password'
    # check file
    if dsa is not None:
        passwdFile = os.path.join(cfg.datasourcesLocator, dsa, passwdFilename)
    else:
        # system admin part
        passwdFile = os.path.join(cfg.datasourcesLocator, passwdFilename)
    # check if file is not older than 30 minutes
    if MD5passwd == '' or MD5passwd is None or not os.path.isfile(passwdFile) or os.stat(passwdFile)[ST_MTIME] + 1800 < int(time.time()):
        return authenticateInitially(clearPasswd, passwdFile)
    log.debug("MD5 password file exists at %s" % passwdFile)        
    # compare md5 passwords from file and parameter
    content = file(passwdFile).read()
    if content == MD5passwd:
        # update filedate to now
        file(passwdFile, 'w').write(content)
        return MD5passwd
    return authenticateInitially(clearPasswd, passwdFile)
    
# ------------------------------------------------------------------------------------
def authenticateInitially(clearPasswd, passwdFile):
    '''Compare clear password with DSA or system config files.
    systemAdmin password allows to edit anything.
    datasource specific passwords are stored in the configtool prefs of the datasource
    and allow to edit that dsa only.
    If system wide authentication is required, leave the dsa parameter None.'''
    global cfg
    if clearPasswd == '' or clearPasswd is None:
        return False
    elif clearPasswd == cfg.configtool.adminpassword:
        log.info("Initial login with the system password")
        return createAuthenPasswordFile(clearPasswd, passwdFile)
    else:
        log.debug("System password login failed. Current system password in file %s is %s" % (cfg.configtool.__filename__, cfg.configtool.adminpassword))
        # check alternatively for dsa specific password
        if dsa is not None:
            dsaObj = Datasource(name=dsa)
            psfObj = dsaObj.getPSFObj()
            log.debug("Using also PSF %s for cfgtool password" % psfObj.PSF)
            if clearPasswd == psfObj.adminpassword:
                log.info("Initial login with the dsa '%s' password" % dsa)
                return createAuthenPasswordFile(clearPasswd, passwdFile)
            else:
                log.info("Failed login for dsa %s" % dsa)
                return False
        else:
            return False

# ------------------------------------------------------------------------------------
def createAuthenPasswordFile(clearPasswd, passwdFile):
    #MD5Passwd = md5.new("%s-%f" %(clearPasswd, time.time()) ).digest()
    MD5Passwd = str( random.randint(100000000000000,999999999999999) )
    log.debug("MD5 password file created: %s"%MD5Passwd)
    file(passwdFile, 'w').write(MD5Passwd)
    return MD5Passwd

# ------------------------------------------------------------------------------------
def authenticationForm(script='main.py', dsa=None):
    global templateDir
    tmpl = PageMacro('Content', PageMacro.DELMODE)
    tmpl.load('Content', os.path.join(templateDir, '_password.html'))
    if dsa is not None:
        tmpl['dsa'] = dsa
    tmpl['script'] = script
    # print HTML !
    printOverHTTP( tmpl )    
    # stop script
    sys.exit()
    
# ------------------------------------------------------------------------------------
def readTmpCmFile(cmfObj, cmFile, psfObj):
    global cfg
    cmFilePickle = cmFile + cfg.configtool.tmpCmfPickleSuffix
    if os.path.isfile( cmFilePickle ):
        log.debug("Read existing pickled tmp file %s"%(cmFilePickle))
        isGood = cmfObj.__unpickle__(filename=cmFilePickle)
        if not isGood:
            log.debug("Failed to read pickled file. Read original CMF xml-file.")
            readOriginalCmFile(cmfObj, cmFile=cmFile, psfObj=psfObj)
    else:
        readOriginalCmFile(cmfObj, cmFile=cmFile, psfObj=psfObj)
    log.debug("altRootTableAliasSPICE = %s" % unicode(cmfObj.altRootTableAliasSPICE))
       

# ------------------------------------------------------------------------------------
def readOriginalCmFile(cmfObj, psfObj, cmFile=None):
    global cfg
    log.debug("Read original CMF xml-file %s"%(cmFile))
    try:
        cmfObj.loadCMFdata( filename=cmFile, psfObj=psfObj, pickle=1 )
    except TableConfigError:
        log.debug("CMF file %s and the PSF dont match. Table config errors."%(cmFile))
    
    
    
############################################################################################################
#
#   MAIN
#
#===========================================================================================================


# get global project configurations
cfg = biocase.configuration.Cfg('configtool')
# LOGGING
cfg._updateLogger('cfgtool')
log = logging.getLogger("webapp.cfgtool")
log.info("--- NEW CONFIGTOOL REQUEST ---")
# dirs used with the configtool
configDir   = os.path.join(cfg.configurationLocator, 'configtool') # configtool config dir
templateDir = os.path.join(cfg.wwwLocator, 'configtool') 
# onscreen debugging
diagnostics = None


#
# ANALYZE CGI ENVIRONMENT
#
form = cgi.FieldStorage()
log.debug("CGI FORM VALUES: %s"%str(form.keys()))

# passwords
clearPasswd = form.getfirst('passwd', None)
log.debug("Passwd clear=%s"%str(clearPasswd))
MD5Passwd = form.getfirst('id', None)
log.debug("Passwd MD5=%s"%str(MD5Passwd))

# dsa
dsa = form.getfirst('dsa',None)
if dsa is not None:
    dsaObj = Datasource(name=dsa, cfg=cfg)
else:
    dsaObj = None
log.debug("DSA=%s"%str(dsa))
    
# schema
schema = form.getfirst('schema', None)
log.debug("Schema=%s"%str(schema))
    
# action
action = form.getfirst('action', form.getfirst('action_override', None))
if action is not None:
    action = action.lower()
log.debug("Action=%s"%str(action))
