#
# -*- coding: utf-8 -*-



############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
u                                           = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
neo                                         = u.neo


############################################################################################################
#
#
#
#===========================================================================================================
class AssessmentsMatrix( neo.Matrix ):


    #-------------------------------------------------------------------------------------------------------
    def __init__( self, dsGrove, topicGauge, focusGauge, _termrefByOntonymCache = None ):
        self.topicGauge         = topicGauge
        self.focusGauge         = focusGauge
        self.topicTermnrminmax  = 0, dsGrove.getGraphlength( topicGauge )
        self.focusTermnrminmax  = 0, dsGrove.getGraphlength( focusGauge )
        self.extremum           = None # only set by ~.normalize()
        self.traces             = {}
        super( AssessmentsMatrix, self ).__init__(
            iminmax     = self.topicTermnrminmax,
            jminmax     = self.focusTermnrminmax,
            vdefault    = 0,
            vminmax     = None,
            )
        if _termrefByOntonymCache is None:
            _termrefByOntonymCache = self._termrefByOntonymCache = {}
            for fact in dsGrove.factoid( 'node' ):
                _termrefByOntonymCache[ fact.ontonym ] = fact.gauge, fact.termnr


    #-------------------------------------------------------------------------------------------------------
    def _topicOntonymForLocator( self, topicLocator ):
        return ':%s:%s' % ( self.topicGauge, topicLocator )

    #-------------------------------------------------------------------------------------------------------
    def _focusOntonymForLocator( self, focusLocator ):
        return ':%s:%s' % ( self.focusGauge, focusLocator )

    #-------------------------------------------------------------------------------------------------------
    def _termrefForOntonym( self, dsGrove, ontonym ):
        return dsGrove._termrefForOntonymOrNonesFromCache( ontonym )
#           gauge, locator, termnr = dsGrove.tripletForOntonymOrNones( ontonym )
#           try:
#               assert gauge    is not None
#               assert termnr   is not None
#               return gauge, termnr
#           except AssertionError:
#               raise KeyError( ''
#                   +   "Unable to resolve ontonym: %r" % ( ontonym, )
#                   )

    #-------------------------------------------------------------------------------------------------------
    def _termrefForGaugeAndLocator( self, dsGrove, gauge, locator ):
        ontonym = ':%s:%s' % ( gauge, locator )
        return self._termrefForOntonym( dsGrove, ontonym )

    #-------------------------------------------------------------------------------------------------------
    def _topicTermrefForLocator( self, dsGrove, topicLocator ):
        topicOntonym = self._topicOntonymForLocator( topicLocator )
        return self._termrefForOntonym( dsGrove, topicOntonym )

    #-------------------------------------------------------------------------------------------------------
    def _focusTermrefForLocator( self, dsGrove, focusLocator ):
        focusOntonym = self._focusOntonymForLocator( focusLocator )
        return self._focusTermrefForOntonym( dsGrove, focusOntonym )

    #-------------------------------------------------------------------------------------------------------
    def addAssessmentForLocators( self, dsGrove, topicLocator, focusLocator, assessment ):
        topicOntonym = self._topicOntonymForLocator( topicLocator )
        focusOntonym = self._focusOntonymForLocator( focusLocator )
        #print '+++++++++++', topicOntonym, focusOntonym
        return self.addAssessmentForOntonyms( dsGrove, topicOntonym, focusOntonym, assessment )

    #-------------------------------------------------------------------------------------------------------
    def addAssessmentForOntonyms( self, dsGrove, topicOntonym, focusOntonym, assessment ):
        topicTermref = self._termrefForOntonym( dsGrove, topicOntonym )
        focusTermref = self._termrefForOntonym( dsGrove, focusOntonym )
        #print '~~~~~~~~~~~~~', topicTermref, focusTermref
        return self.addAssessmentForTermrefs( dsGrove, topicTermref, focusTermref, assessment )

    #-------------------------------------------------------------------------------------------------------
    def addAssessmentForTermrefs( self, dsGrove, topicTermref, focusTermref, assessment ):
        #---------------------------------------------------------------------------------------------------
        def iterSubsequents( termref, includeSelf ):
            ontonym = self._ontonymForTermref( termref )
            try:
                return iter( dsGrove.iterSubsequentTermrefs( ontonym, includeSelf = includeSelf ) )
            except KeyError:
                return iter( [] )
        #---------------------------------------------------------------------------------------------------
        topicGraphname, topicTermnr = topicTermref
        topicTermnr                 = int( topicTermnr )
        topicGauge                  = ':%s:' % ( topicGraphname, )
        topicOntonym                = '%s%d' % ( topicGauge, topicTermnr )
        topicTermref                = topicGraphname, topicTermnr
        #---------------------------------------------------------------------------------------------------
        focusGraphname, focusTermnr = focusTermref
        focusTermnr                 = int( focusTermnr )
        focusGauge                  = ':%s:' % ( focusGraphname, )
        focusOntonym                = '%s%d' % ( focusGauge, focusTermnr )
        focusTermref                = focusGraphname, focusTermnr
        #---------------------------------------------------------------------------------------------------
        #print '$$$$$$', list( iterSubsequents( topicTermref, True ) )
        #print '$$$$$$', list( iterSubsequents( focusTermref, True ) )
        #---------------------------------------------------------------------------------------------------
        focusSubTermrefs            = list( iterSubsequents( focusTermref, True ) )
        #---------------------------------------------------------------------------------------------------
        for topicSubTermref in iterSubsequents( topicTermref, True ):
            #-----------------------------------------------------------------------------------------------
            topicSubGraphname, topicSubTermnr   = topicSubTermref
            topicSubTermnr                  = int( topicSubTermnr )
            topicSubTermref                 = topicSubGraphname, topicSubTermnr
            #-----------------------------------------------------------------------------------------------
            for focusSubTermref in focusSubTermrefs:
                #-------------------------------------------------------------------------------------------
                focusSubGraphname, focusSubTermnr   = focusSubTermref
                focusSubTermnr                  = int( focusSubTermnr )
                focusSubTermref                 = focusSubGraphname, focusSubTermnr
                #-------------------------------------------------------------------------------------------
                ij              = ( topicSubTermnr, focusSubTermnr )
                self[ ij ]      += assessment
                #print '~~~~~~~~~~$', ij, assessment
                #-------------------------------------------------------------------------------------------
                traceskey       = ( topicSubTermref, focusSubTermref              )
                tracesvalue     = (    topicOntonym,    focusOntonym, assessment, )
                trascestarget   = self.traces.setdefault( traceskey, [] )
                trascestarget.append( tracesvalue )

    #-------------------------------------------------------------------------------------------------------
    def _nodefactForTermref( self, dsGrove, termref ):
        #return Elohim.getNode( termref )
        return dsGrove.nodefactForOntonym( self._ontonymForTermref( termref ) )
    #_nodefactForTermref = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, termref ):
        graphname, termnr = termref
        gauge = ':%s:' % ( graphname, )
        return ( gauge, termnr )
    _ontonymForTermref = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def iterSparseNodesAndAssessments( self ):
        for ij, assessment in self.iterSparseItems():
            #print ij, assessment
            topicTermnr, focusTermnr = ij
            topicTermref    = ( self.topicGauge, topicTermnr )
            focusTermref    = ( self.focusGauge, focusTermnr )
            topicNode       = self._nodefactForTermref( topicTermref )
            focusNode       = self._nodefactForTermref( focusTermref )
            yield topicNode, focusNode, assessment
        #topicTermnr, focusTermnr in assessments.iter

    #-------------------------------------------------------------------------------------------------------
    def vectorForTopicTermnr( self, topicTermnr ):
        return self.getColumnVector( int( topicTermnr ) )

    #-------------------------------------------------------------------------------------------------------
    def vectorForFocusTermnr( self, focusTermnr ):
        return self.getRowVector( int( focusTermnr ) )

    #-------------------------------------------------------------------------------------------------------
    def vectorForTopicLocator( self, topicLocator ):
        topicGauge, topicTermnr = self._topicTermrefForLocator( topicLocator )
        return self.vectorForTopicTermnr( topicTermnr )

    #-------------------------------------------------------------------------------------------------------
    def vectorForFocusLocator( self, focusLocator ):
        focusGauge, focusTermnr = self._focusTermrefForLocator( focusLocator )
        return self.vectorForFocusTermnr( focusTermnr )

    #-------------------------------------------------------------------------------------------------------
    def ratingsVectorWithTermnrsForTopicLocator( self, topicLocator ):
        topicGauge, topicTermnr = self._topicTermrefForLocator( topicLocator )
        return self.ratingsVectorWithTermnrsForTopicTermnr( topicTermnr )

    #-------------------------------------------------------------------------------------------------------
    def ratingsVectorWithTermnrsForTopicTermnr( self, topicTermnr ):
        vector = self.vectorForTopicTermnr( topicTermnr )
        return vector
#   #           print vector; YYYYYYYYYY
#   #           R = [
#   #               ( assessment, termnr )
#   #                   for termnr, assessment in vector.iterAllItems()
#   #               ]
#   #           R.sort()
#   #           R.reverse()
#   #           return R

    #-------------------------------------------------------------------------------------------------------
    def iterRatingsVectorWithNodesForTopicLocator( self, topicLocator ):
        gauge   = self.focusGauge
        nftr    = self._nodefactForTermref
        R       = self.ratingsVectorWithTermnrsForTopicLocator( topicLocator )
        #print '$$$$$$$$$$$$$$$$$', topicLocator, R
        for termnr, assessment in R.iteritems():
            yield assessment, nftr( ( gauge, termnr ) )

    #-------------------------------------------------------------------------------------------------------
    def f( this, gauge, ratingsVector, maxcount = None, minrating = None ):
        """This method allows consumers that have retrieved a ratings vector from persistence to iterate it
        without instantiating an ``AssessmentsMatrix`` object."""
        nftr    = this._nodefactForTermref
        for termnr, assessment in ratingsVector.iteritems():
            yield assessment, nftr( ( gauge, termnr ) )
    iterRatingsVectorWithNodesForGaugeAndRatingsVectorWithTermnrs = classmethod( f ); del f

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit(50)
    sep0 = '#' * 108
    sep1 = '=' * 108
    sep2 = '-' * 108


    topicGauge = 'bion'
    focusGauge = 'abcd12'

    assessments = AssessmentsMatrix( topicGauge, focusGauge )


    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection/organism-group/animals'
    focusLocator = '/DataSets/DataSet/Units/Unit/Identifications/Identification/TaxonIdentified/ScientificNameAtomized/Zoological'
    assessment = 1
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection/collection-type/botanical-garden'
    focusLocator = '/DataSets/DataSet/Units/Unit/UnitCollectionDomain/BotanicalGardenUnit'
    assessment = +2
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection/organism-group/plants'
    focusLocator = '/DataSets/DataSet/Units/Unit/Identifications/Identification/TaxonIdentified/ScientificNameAtomized/Zoological'
    assessment = -2
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection/organism-group/plants'
    focusLocator = '/DataSets/DataSet/Units/Unit/Identifications/Identification/TaxonIdentified/ScientificNameAtomized/Botanical'
    assessment = +2
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection/organism-group/plants'
    focusLocator = '/DataSets/DataSet/Units/Unit/Gathering/GatheringSite/Synecology/AssociatedTaxa/TaxonIdentified/ScientificNameAtomized/Botanical'
    assessment = +1
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection/organism-group'
    focusLocator = '/DataSets/DataSet/Units/Unit/Gathering/GatheringSite/Synecology'
    assessment = -1
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

    #-------------------------------------------------------------------------------------------------------
    topicLocator = '/bion/collection'
    focusLocator = '/DataSets/DataSet/Units/Unit/Gathering/GatheringSite/Synecology'
    assessment = -1
    assessments.addAssessmentForLocators( topicLocator, focusLocator, assessment )

#       for topicNode, focusNode, assessment in assessments.iterSparseNodesAndAssessments():
#           print
#           print topicNode.ontonym
#           print focusNode.ontonym
#           print assessment

    #topicLocator = '/bion/collection/organism-group/plants'
    #topicLocator = '/bion/collection/organism-group/plants'

    topicLocators = [ x.strip() for x in """
            /bion/collection/collection-type/botanical-garden
            /bion/collection/collection-type/field-samples
#               /bion/collection/collection-type/microbial-or-tissue-culture
#               /bion/collection/collection-type/natural-history-collection
#               /bion/collection/collection-type/observations
#               /bion/collection/organism-group/algae
#               /bion/collection/organism-group/animals
#               /bion/collection/organism-group/bacteria
#               /bion/collection/organism-group/fungi
#               /bion/collection/organism-group/lichens
#               /bion/collection/organism-group/plants
#               /bion/collection/organism-group/protozoa
#               /bion/collection/organism-group/viruses
#               /bion/collection/state/conserved
#               /bion/collection/state/derivate-preparation
#               /bion/collection/state/digital
#               /bion/collection/state/dormant
#               /bion/collection/state/fossil
#               /bion/collection/state/living
#               /bion/collection/state/physical-image
#               /bion/unit-data/administrative-data
        """.splitlines() if not '#' in x ]

    for topicLocator in topicLocators:
        if not topicLocator: continue
        print sep0
        print topicLocator
        ratings = assessments.iterRatingsVectorWithNodesForTopicLocator( topicLocator )
        for assessment, node in ratings:
            if assessment == 0: continue
            print '\t%7.2f %s' % ( assessment, node.ontonym, )

#       topicVector = assessments.vectorForTopicLocator( topicLocator )
#       ratings = [
#           ( assessment, assessments._nodefactForTermref( ( 'abcd12', termnr ) ) )
#               for termnr, assessment in topicVector.iterAllItems()
#           ]
#       ratings.sort()
#       ratings.reverse()
#       for assessment, node in ratings:
#           if assessment == 0: continue
#           print assessment, node.ontonym

#       for ij, assessment in assessments.iterSparseItems():
#           #print ij, assessment
#           topicTermnr, focusTermnr = ij
#           topicTermref = ( topicGauge, topicTermnr )
#           focusTermref = ( focusGauge, focusTermnr )
#           topicNode = Elohim.getNode( topicTermref )
#           focusNode = Elohim.getNode( focusTermref )
#           print
#           print topicNode.ontonym
#           print focusNode.ontonym
#           print assessment
#       #topicTermnr, focusTermnr in assessments.iter

#       RatingsVectorsByTopicTermnrDb = g.configtool.utilities.persistence.storedefs.RatingsVectorsByTopicTermnrDb
#       ratingsVectorsByTopicTermnrDb = RatingsVectorsByTopicTermnrDb( focusGauge, topicGauge )
#       ratingsVectorsByTopicTermnrDb.openForWriting()
#       for topicTermnr in assessments.iterAllI():
#           ratings = assessments.ratingsVectorWithTermnrsForTopicTermnr( topicTermnr )
#           print topicTermnr
#           #break
#           ratingsVectorsByTopicTermnrDb[ topicTermnr ] = ratings








