#
# -*- coding: utf-8 -*-

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
#   from __future__ import division
#-----------------------------------------------------------------------------------------------------------
from sets import                        Set
#-----------------------------------------------------------------------------------------------------------
ReprMx                                  = g.configtool.utilities.IOU.ReprMx

#===========================================================================================================
def getFget( hiddenname ):
    def fget( self ):
        return getattr( self, hiddenname )
    return fget

#===========================================================================================================
def getFset( hiddenname ):
    def fset( self, value ):
        setattr( self, hiddenname, value )
    return fset

#===========================================================================================================
def getHiddenname( name ):
    return '_%s' % ( name, )

#===========================================================================================================
allowableSlotValues = Set( ( 0, 1, True, False, ) )

#===========================================================================================================
def getInit( obligatorynames ):
    #-------------------------------------------------------------------------------------------------------
    def __init__( self, **Q ):
        #---------------------------------------------------------------------------------------------------
        qnames                  = Set( Q.keys() )
        missingobligatorynames  = obligatorynames - qnames
        #---------------------------------------------------------------------------------------------------
        if missingobligatorynames:
            raise TypeError( ""
                + "Unable to instantiate class %s because the following " % ( self.__class__.__name__, )
                + "arguments are missing: %s." % ', '.join( map( repr, missingobligatorynames ) )
                )
        #---------------------------------------------------------------------------------------------------
        for name, value in Q.iteritems():
            setattr( self, name, value )
    #-------------------------------------------------------------------------------------------------------
    return __init__

#===========================================================================================================
def getUnicode( classname ):
    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        return '<%s %s>' % (
            classname,
            ', '.join( [
                '%s = %r' % ( name, getattr( self, name ), )
                    for name in self.__allnamessorted__ ]
                ),
            )
    #-------------------------------------------------------------------------------------------------------
    return __unicode__

#===========================================================================================================
def getPrintoverview():
    #-------------------------------------------------------------------------------------------------------
    def __printoverview__( self ):
        for name in self.__allnamessorted__:
            print '\t%-20s:%s' % ( name, getattr( self, name ), )
    #-------------------------------------------------------------------------------------------------------
    return __printoverview__

#===========================================================================================================
#   def getPreinit():
#       def __preinit__( this ):
#           for name, value in this.__defaults__.iteritems():
#               setattr( this, name, value )
#       return __preinit__

############################################################################################################
#
#
#
#===========================================================================================================
def EzClassMc( classname, classbases, classdict ):
    #-------------------------------------------------------------------------------------------------------
    obligatory  = classdict.get( '__obligatory__',  () )
    optional    = classdict.get( '__optional__',    {} )
    slots       = Set()
    #-------------------------------------------------------------------------------------------------------
    #   classdict[ '__obligatorynames__' ]
    #   classdict[ '__optionalnames__'   ]
    obligatorynames     = Set( obligatory      )
    optionalnames       = Set( optional.keys() )
    namesfoundinboth    = obligatorynames & optionalnames
    if namesfoundinboth:
        raise TypeError( ""
            + "Class %s is not well-formed since the following names are found in both " % classname
            + "__optional__ and __obligatory__: %s." % ', '.join( map( repr, namesfoundinboth ) )
            )
    allnames            = obligatorynames | optionalnames
    #-------------------------------------------------------------------------------------------------------
    for name in allnames:
        #---------------------------------------------------------------------------------------------------
        hiddenname  = getHiddenname( name )
        modname     =  '%s%s' % ( name[ 0 ].upper(), name[ 1 : ] )
        fgetname    = 'get%s' % modname
        fsetname    = 'set%s' % modname
        hasfget     = fgetname in classdict
        hasfset     = fsetname in classdict
        isproperty  = hasfget or hasfset
        #---------------------------------------------------------------------------------------------------
        if isproperty:
            #-----------------------------------------------------------------------------------------------
            if hasfget: fget = classdict[ fgetname ]
            else:       fget = getFget( hiddenname )
            #-----------------------------------------------------------------------------------------------
            if hasfset: fset = classdict[ fsetname ]
            else:       fset = getFset( hiddenname )
            #-----------------------------------------------------------------------------------------------
            classdict[ name ] = property( fget, fset )
            classdict[ fgetname ] = fget
            classdict[ fsetname ] = fset
            slotname = hiddenname
        #---------------------------------------------------------------------------------------------------
        else:
            slotname = name
        #---------------------------------------------------------------------------------------------------
        slots.add( slotname )
        #---------------------------------------------------------------------------------------------------
        if name in optionalnames:
            classdict[ slotname ] = optional[ name ]
        #---------------------------------------------------------------------------------------------------
    #classdict[ '__slots__' ] = tuple( slots_ )
    #-------------------------------------------------------------------------------------------------------
    if '__slots__' in classdict:
        slots_ = classdict[ '__slots__' ]
        if not slots_ in allowableSlotValues:
            raise TypeError( ""
                + "Illegal value for %s.__slots__: %s. " % ( classname, slots_, )
                + "Allowed are %s." % ', '.join( map( repr, allowableSlotValues ) )
                )
        hasslots = bool( slots_ )
        if hasslots:
            classdict[ '__slots__' ] = tuple( slots )
        else:
            del classdict[ '__slots__' ]
    else:
        hasslots = False
    #-------------------------------------------------------------------------------------------------------
    if '__init__' not in classdict:
        initname = '__init__'
    else:
        initname = '__initEzClassMc__'
    classdict[ initname ] = getInit( obligatorynames )
    #-------------------------------------------------------------------------------------------------------
    if '__unicode__' not in classdict:
        allnamessorted = list( allnames )
        allnamessorted.sort()
        classdict[ '__allnamessorted__' ] = allnamessorted
        classdict[ '__unicode__' ] = getUnicode( classname )
    #-------------------------------------------------------------------------------------------------------
    if '__printoverview__' not in classdict:
        classdict[ '__printoverview__' ] = getPrintoverview()
    #-------------------------------------------------------------------------------------------------------
    if ReprMx not in classbases:
        classbases = tuple( list( classbases ) + [ ReprMx ] )
    #-------------------------------------------------------------------------------------------------------
    return type( classname, classbases, classdict )


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    sep = '#' * 108

    #-------------------------------------------------------------------------------------------------------
    class SchemaDescription:
        __metaclass__   = EzClassMc
        __obligatory__  = """foo gnu""".split()
        __optional__    = dict(
            bar     = 'BAR',
            baz     = 'BAZ',
            )
        __slots__ = True

        #---------------------------------------------------------------------------------------------------
        def setFoo( self, value ):
            print 'set foo *', value
            self._foo = value

        #---------------------------------------------------------------------------------------------------
        def getGnu( self ):
            R = self._gnu
            print 'get gnu *', R
            return R

    #-------------------------------------------------------------------------------------------------------
    print sep
    sd = SchemaDescription( foo = 'FOO', gnu = 'GNU' )
    print dir( SchemaDescription )
    print dir( sd )
    print getattr( SchemaDescription, '__slots__', None )
    print sd.foo
    print sd.gnu
    print sd.bar
    print sd.baz
    print sd
    #   print sd.__dict__
    #   sd.ddd = 42
    #   print sd.__dict__

    sd.foo = 42
    print sd.foo
    print sd

    #-------------------------------------------------------------------------------------------------------
    print sep
    se = SchemaDescription( foo = 123, gnu = '++gnu++' )
    print se.foo
    print se.gnu
    print se.bar
    print se.baz
    print se









