#
# -*- coding: utf-8 -*-

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_ReprMx                                 = u.IOU.ReprMx
_Misfit                                 = u.Misfit.Misfit
#-----------------------------------------------------------------------------------------------------------
_S                                      = u.EzSet.S
#-----------------------------------------------------------------------------------------------------------




############################################################################################################
#
#
#
#===========================================================================================================
class Grove( _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self ):
        self.graphs = {}

    #-------------------------------------------------------------------------------------------------------
    def add( self, graph ):
        self[ graph.name ] = graph

    #-------------------------------------------------------------------------------------------------------
    def __iter__( self ):
        return self.graphs.itervalues()

    #-------------------------------------------------------------------------------------------------------
    def __getitem__( self, name ):
        return self.graphs[ name ]

    #-------------------------------------------------------------------------------------------------------
    def __setitem__( self, name, graph ):
        self.graphs[ name ] = graph

    #-------------------------------------------------------------------------------------------------------
    def iteritems( self ):
        return self.graphs.iteritems()

    #-------------------------------------------------------------------------------------------------------
    def __len__( self ):
        return len( self.graphs )

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        graphs = ' '.join( self.graphs.keys() )
        return '<Grove of %d graphs: %s>' % ( len( self ), graphs, )

    #-------------------------------------------------------------------------------------------------------
    def __hash__( self ):
        return id( self )

    #-------------------------------------------------------------------------------------------------------
    def close( self ):
        for graph in self:
            graph.close()

############################################################################################################
#
#
#
#===========================================================================================================
class Graph( _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, name, allowMultipleRootnodes = False ):
        self.name                   = name
        self.nodes                  = {}
        self.rootNode               = None
        self._nodesByTermnrCache    = None
        self.allowMultipleRootnodes = allowMultipleRootnodes
        self._open                  = True
        self._locators              = None

    #-------------------------------------------------------------------------------------------------------
    def close( self ):
        assert self._open
        self.assignTermnrs()
        self._open = False

    #-------------------------------------------------------------------------------------------------------
    def newNodeForLocator( self, locator ):
        assert isinstance( locator, basestring )
        assert len( locator ) > 2
        assert locator.startswith( '/' )
        assert not locator.endswith( '/' )
        if locator.count( '/' ) == 1:
            parentLocator       = None
            name                = locator[ 1 : ]
        else:
            lastSlashPosition   = locator.rfind( '/' )
            parentLocator       = locator[                       : lastSlashPosition ]
            name                = locator[ lastSlashPosition + 1 :                   ]
        return self.newNode( name, parentLocator )

    #-------------------------------------------------------------------------------------------------------
    def newNode( self, name, parentOrParentLocator = None ):
        #---------------------------------------------------------------------------------------------------
        assert self._open
        #---------------------------------------------------------------------------------------------------
        if isinstance( parentOrParentLocator, basestring ):
            parent = self.nodes[ parentOrParentLocator ]
        else:
            parent = parentOrParentLocator
        #---------------------------------------------------------------------------------------------------
        assert parent is None or isinstance( parent, Node )
        #---------------------------------------------------------------------------------------------------
        R = Node( graph = self, name = name, parent = parent )
        #R.parent = parent
        self.nodes[ R.locator ] = R
        if parent is None:
            if ( not self.allowMultipleRootnodes ) and self.rootNode is not None:
                raise ValueError( ''
                    +   "Illegal to set more than one root node."
                    )
            self.rootNode = R
        return R

    #-------------------------------------------------------------------------------------------------------
    def assignTermnrs( self ):
        assert self._open
        self._nodesByTermnrCache = []
        termnr = -1
        for node in self:
            termnr += 1
            node._termnr = termnr
            self._nodesByTermnrCache.append( node )
            parent = node.parent
#               if parent is not None:
#                   parent.lastChildnodeTermnr  = termnr
#                   #parent.lastDescendantTermnr = termnr
        if self._nodesByTermnrCache:
            rootnode = self._nodesByTermnrCache[ 0 ]
            assert rootnode.parent is None

    #-------------------------------------------------------------------------------------------------------
    def getNodeForTermnr( self, termnr ):
        if self._nodesByTermnrCache is None:
            self.assignTermnrs()
        return self._nodesByTermnrCache[ termnr ]

    #-------------------------------------------------------------------------------------------------------
    def getLocators( self ):
        R = self._locators
        if R is None:
            R = self._locators = self.nodes.keys()
            R.sort()
        return R
    locators = property( getLocators )

    #-------------------------------------------------------------------------------------------------------
    def iteritems( self ):
        for locator in self.locators:
            yield locator, self[ locator ]

    #-------------------------------------------------------------------------------------------------------
    def iternodes( self ):
        for locator in self.locators:
            yield self[ locator ]

    #-------------------------------------------------------------------------------------------------------
    def __iter__( self ):
        return self.iternodes()

    #-------------------------------------------------------------------------------------------------------
    def __getitem__( self, locatorOrTermnr ):
        if isinstance( locatorOrTermnr, basestring ):
            assert len( locatorOrTermnr )
            if locatorOrTermnr.isdigit():
                locatorOrTermnr = int( locatorOrTermnr )
        if not isinstance( locatorOrTermnr, basestring ):
            return getNodeForTermnr( self, locatorOrTermnr )
        return self.nodes[ locatorOrTermnr ]

    #-------------------------------------------------------------------------------------------------------
    def __setitem__( self, locator, node ):
        self.nodes[ locator ] = node

    #-------------------------------------------------------------------------------------------------------
    def __len__( self ):
        return len( self.nodes )

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        return '<Graph %s (%d nodes)>' % ( self.name, len( self ), )

    #-------------------------------------------------------------------------------------------------------
    def __hash__( self ):
        return self.name

############################################################################################################
#
#
#
#===========================================================================================================
class Node( _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, graph, name, parent ):
        #---------------------------------------------------------------------------------------------------
        self.graph                  = graph
        self.name                   = name
        self.parent                 = parent
        self.events                 = []
        self.ownEquivalents         = _S()      # set of equivalents defined by secondary structure
        self.ownSupernodes          = _S()      # 'parents'  defined by secondary structure
        self.ownSubnodes            = _S()      # 'children' defined by secondary structure
        self.otherSupernodes        = _S()      # supernodes resulting from this node being a subnode
        self.otherSubnodes          = _S()      # subnodes resulting from this node being a supernode
        self._termnr                = None
        #---------------------------------------------------------------------------------------------------
        self._childnodes            = []
        self._childnodeTermnrs      = None
        self._descendants           = None
        self._subsequents           = None
        self._ancestors             = None
        #---------------------------------------------------------------------------------------------------
        self._isLeaf                = True
        self._lastDescendant        = _Misfit
        self._subsequentTermnrs     = None
        #self._descendantTermnrs     = None
        self._ancestorTermnrs       = None
        #---------------------------------------------------------------------------------------------------
        if parent is not None:
            parent._announceChildnode( self )

        self._ratingsByGraphname    = {}

    #-------------------------------------------------------------------------------------------------------
    def setRating( self, targetTermref, value ):
        for node in self.subsequents:
            targetGraphname, targetTermnr = targetTermref
            targetRegister = node._ratingsByGraphname.setdefault( targetGraphname, {} )
            targetRegister[ targetTermrn ] = targetRegister.get( targetTermnr, 0 ) + value

    #-------------------------------------------------------------------------------------------------------
    def getRating( self, targetTermref ):
        targetGraphname, targetTermnr = targetTermref
        targetRegister = node._ratingsByGraphname.get( targetGraphname, {} )
        return targetRegister.get( targetTermnr, 0 )

    #-------------------------------------------------------------------------------------------------------
    def getTermnr( self ):
        assert not self.graph._open
        return self._termnr
    termnr = property( getTermnr )

    ########################################################################################################
    #   CHILDNODES
    #-------------------------------------------------------------------------------------------------------
    def _announceChildnode( self, node ):
        assert self.graph._open
        if not node.name.startswith( '@' ):
            self._isLeaf = False
        self._childnodes.append( node )

    #-------------------------------------------------------------------------------------------------------
    def getChildnodeTermnrs( self ):
        """Return a sorted list of the term numbers of the childnodes of this node. Side effect:
        ``~._childnodes`` is re-built to conform to the same sorting."""
        assert not self.graph._open
        R = self._childnodeTermnrs
        if R is None:
            childnodes = self._childnodes
            termnrsAndChildnodes = [ ( node.termnr, node ) for node in childnodes ]
            termnrsAndChildnodes.sort()
            del childnodes[:]
            R = self._childnodeTermnrs = []
            for termnr, node in termnrsAndChildnodes:
                R.append( termnr )
                childnodes.append( node )
        return R
    childnodeTermnrs = property( getChildnodeTermnrs )

    #-------------------------------------------------------------------------------------------------------
    def getChildnodes( self ):
        #---------------------------------------------------------------------------------------------------
        #   We issue ``~.getChildnodeTermnrs()`` here to make sure childnodes are properly sorted.
        #   It would be a tad more economical to check for ``~._childnodeTermnrs`` being ``None`` first.
        self.getChildnodeTermnrs()
        return self._childnodes
    childnodes = property( getChildnodes )

    #-------------------------------------------------------------------------------------------------------
    def getHasChildnodes( self ):
        assert not self.graph._open
        return bool( self._childnodes )
    hasChildnodes = property( getHasChildnodes )

    #-------------------------------------------------------------------------------------------------------
    def getLastChildnode( self ):
        #---------------------------------------------------------------------------------------------------
        #   We issue ``~.getChildnodeTermnrs()`` here to make sure childnodes are properly sorted.
        #   It would be a tad more economical to check for ``~._childnodeTermnrs`` being ``None`` first.
        self.getChildnodeTermnrs()
        if not self._childnodes: return None
        return self._childnodes[ -1 ]
    lastChildnode = property( getLastChildnode )

    #-------------------------------------------------------------------------------------------------------
    def getLastSibling( self ):
        assert not self.graph._open
        if self.parent is None: return None
        return self.parent.lastChildnode
    lastSibling = property( getLastSibling )

    #-------------------------------------------------------------------------------------------------------
    def getIsLeaf( self ):
        assert not self.graph._open
        return self._isLeaf
    isLeaf = property( getIsLeaf )


    ########################################################################################################
    #   DESCENDANTS
    #-------------------------------------------------------------------------------------------------------
    def getLastDescendant( self, includeSelf = False ):
        R = self._lastDescendant
        if R is _Misfit:
            assert not self.graph._open
            if self.hasChildnodes:
                R = self._lastDescendant = self.lastChildnode.getLastDescendant( includeSelf = True )
            else:
                R = self._lastDescendant = None
                if includeSelf: return self
        return R
    lastDescendant = property( getLastDescendant )

    #-------------------------------------------------------------------------------------------------------
    def getLastDescendantTermnr( self, includeSelf = False ):
        """Return term number of last descendant node, or ``None`` if node has no descendants; in
        contradistinction to ``~isLeaf``, attributes are included as descendants here. -- The term number
        ``d`` of the last descendant ``D`` of a node ``N`` is suffient for deriving all the descendants in
        graphs where term numbers correspond to the indices of nodes in an alphabetically sorted list of all
        node locators in that graph, since the term numbers of the descendants always form a compact
        interval ``[n,d]`` (with ``self`` included) or ``]n,d]`` (with ``self`` excluded, which is the
        default)."""
        assert not self.graph._open
        return getattr( self.lastDescendant, 'termnr', None )
        #if lastDescendant is None: return None
        #return lastDescendant.termnr
    lastDescendantTermnr = property( getLastDescendantTermnr )

    #-------------------------------------------------------------------------------------------------------
    def iterDescendantTermnrs( self, includeSelf = False ):
        #---------------------------------------------------------------------------------------------------
        if includeSelf:
            firstDescendantTermnr = self.termnr
        else:
            firstDescendantTermnr = self.termnr + 1
        #---------------------------------------------------------------------------------------------------
        lastDescendantTermnr = self.lastDescendantTermnr
        if lastDescendantTermnr is None:
            lastDescendantTermnr = self.termnr
        #---------------------------------------------------------------------------------------------------
        return xrange( firstDescendantTermnr, lastDescendantTermnr + 1 )

    #-------------------------------------------------------------------------------------------------------
    def getDescendants( self, includeSelf = False ):
        R = self._descendants
        if R is None:
            cache = self.graph._nodesByTermnrCache
            R = self._descendants = [ cache[ termnr ] for termnr in self.iterDescendantTermnrs() ]
        if includeSelf:
            R = [ self ] + R
        return R
    descendants = property( getDescendants )


    ########################################################################################################
    #   SUBSEQUENTS
#       #-------------------------------------------------------------------------------------------------------
#       def getSubsequentTermnrs( self, includeSelf = True ):
#           R = self._subsequentTermnrs
#           if R is None:
#               descendantTermnrs   = _S( *self.iterDescendantTermnrs( includeSelf = True ) )
#               ownSubnodeTermnrs   = _S( *[ subnode.termnr for subnode in self.ownSubnodes ] )
#               otherSubnodeTermnrs = _S( *[ subnode.termnr for subnode in self.otherSubnodes ] )
#               R = list( descendantTermnrs | ownSubnodeTermnrs | otherSubnodeTermnrs )
#               R.sort()
#               self._subsequentTermnrs = R
#           if not includeSelf:
#               termnr = self.termnr
#               R = [ subtermnr for subtermnr in R if subtermnr is not termnr ]
#           return R
#       subsequentTermnrs = property( getSubsequentTermnrs )

    #-------------------------------------------------------------------------------------------------------
    def _iterSubsequents( self, includeSelf, seen ):
        R = self._subsequentTermnrs
        #---------------------------------------------------------------------------------------------------
        if R is None:
            if includeSelf:
                seen.add( self )
                yield self
            #-----------------------------------------------------------------------------------------------
            for collection in ( self.descendants, self.ownSubnodes, self.otherSubnodes ):
                for subnode in collection:
                    if not subnode in seen:
                        seen.add( subnode )
                        yield subnode
                        for subsubnode in subnode._iterSubsequents( False, seen ):
                            yield subsubnode

    #-------------------------------------------------------------------------------------------------------
    def getSubsequents( self, includeSelf = True ):
        R = self._subsequents
        if R is None:
            cache = self.graph._nodesByTermnrCache
            R = self._subsequents = [ cache[ termnr ] for termnr in self.subsequentTermnrs ]
        if not includeSelf:
            R = [ subnode for subnode in R if subnode is not self ]
        return R
    subsequents = property( getSubsequents )

    #-------------------------------------------------------------------------------------------------------
    def getSubsequentTermnrs( self, includeSelf = True ):
        R = self._subsequentTermnrs
        if R is None:
            R = self._subsequentTermnrs = [ node.termnr for node in self._iterSubsequents( True, _S() ) ]
            R.sort()
        if not includeSelf:
            termnr = self.termnr
            R = [ subtermnr for subtermnr in R if subtermnr != termnr ]
        return R
    subsequentTermnrs = property( getSubsequentTermnrs )

    ########################################################################################################
    #   ANCESTORS
    #-------------------------------------------------------------------------------------------------------
    def getAncestors( self, includeSelf = False ):
        R = self._ancestors
        if R is None:
            parent = self.parent
            R = self._ancestors = []
            if parent is not None:
                R.append( parent )
                R.extend( list( parent.ancestors ) )
        if includeSelf:
            R = [ self ] + R
        return R
    ancestors = property( getAncestors )

    #-------------------------------------------------------------------------------------------------------
    def getAncestorTermnrs( self, includeSelf = False ):
        return [ node.termnr for node in self.getAncestors( includeSelf ) ]
    ancestorTermnrs = property( getAncestorTermnrs )

    ########################################################################################################
    #   OTHER RELATIVES
    #-------------------------------------------------------------------------------------------------------
    def getAllSupernodes( self ):
        """Return the set of all supernodes in this cluster."""
        raise NotImplementedError
    allSupernodes = property( getAllSupernodes )

    #-------------------------------------------------------------------------------------------------------
    def getAllSubnodes( self ):
        """Return the set of all subnodes in this cluster."""
        raise NotImplementedError
    allSubnodes = property( getAllSubnodes )

    ########################################################################################################
    #   LOCATORS ETC
    #-------------------------------------------------------------------------------------------------------
    def getLocator( self ):
        R = '/%s' % ( self.name, )
        if self.parent is not None:
            R = '%s%s' % ( self.parent.locator, R )
        return R
    locator = property( getLocator )

    #-------------------------------------------------------------------------------------------------------
    def getOntonym( self ):
        return ':%s:%s' % ( self.graph.name, self.locator, )
    ontonym = property( getOntonym )

    #-------------------------------------------------------------------------------------------------------
    def getLevel( self ):
        R = 1
        if self.parent is not None:
            R += self.parent.level
        return R
    level = property( getLevel )

    #-------------------------------------------------------------------------------------------------------
    def getNumeronym( self ):
        assert not self.termnr is None
        return ':%s:%s' % ( self.graph.name, self.termnr, )
    numeronym = property( getNumeronym )

    #-------------------------------------------------------------------------------------------------------
    def _getNodesRepr( self, nodes ):
        R = []
        for node in nodes:
            if isinstance( node, Node ):
                R.append( node.locator )
            else:
                R.append( '%s' % ( node, ) )
        return ', '.join( R )

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        if self.events:
            events = ' (%s)' % ( ' '.join( [ '%s' % ( event, ) for event in self.events ] ) )
        else:
           events = ''
        nodes = [ self.locator ]
        if self.ownSubnodes:       nodes.append(  '=> %s' % ( self._getNodesRepr( self.ownSubnodes    ) ) )
        if self.ownEquivalents:    nodes.append( '<=> %s' % ( self._getNodesRepr( self.ownEquivalents ) ) )
        if self.ownSupernodes:     nodes.append(  '<= %s' % ( self._getNodesRepr( self.ownSupernodes  ) ) )
        nodes = ', '.join( nodes )
        return '<Node %s>' % ( nodes, )

    #-------------------------------------------------------------------------------------------------------
    def __hash__( self ):
        return id( self )

    #-------------------------------------------------------------------------------------------------------
    def addSupernode( self, other ):
        self.ownSupernodes.add( other )
        if isinstance( other, Node ):
            other.otherSubnodes.add( self )

    #-------------------------------------------------------------------------------------------------------
    def addSubnode( self, other ):
        self.ownSubnodes.add( other )
        if isinstance( other, Node ):
            other.otherSupernodes.add( self )

    #-------------------------------------------------------------------------------------------------------
    def addEquivalent( self, other ):
        self.ownEquivalents.add( other )



#   ############################################################################################################
#   #
#   #
#   #
#   #===========================================================================================================
#   class Cluster( _S ):
#
#       pass
#
#       #-------------------------------------------------------------------------------------------------------
#       def __init__( self, *P ):
#           self._nodes = _S( *P )





