#
# -*- coding: utf-8 -*-


"""EzSet"""

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from sets import                        BaseSet
from sets import                        Set
from sets import                        ImmutableSet
from sets import                        ifilter
from sets import                        ifilterfalse
from sets import                        _TemporarilyImmutableSet
#from sets import                        *
#from sets import                        Set
IOU                                     = g.configtool.utilities.IOU
ReprMx                                  = IOU.ReprMx
iounicode                               = IOU.iounicode

############################################################################################################
#
#
#
#===========================================================================================================
class EzSetMx( ReprMx ):

    """A set with a number of enhancements."""

    _classesWithoutClassnamedisplay = Set()

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, *P ):
        #super( EzSetMx, self ).__init__( P )
        if P:
            Set.__init__( self, P )
        else:
            Set.__init__( self )
        #print '~~~~~~~', self  ###DEBUG###

    #-------------------------------------------------------------------------------------------------------
    def __getstate__( self ):
        """Method to make instances ``pickle``\ able. -- Also see ``IOU.ReprMx``."""
        R = super( EzSetMx, self ).__getstate__()
        R[ '_data' ] = self._data
        return R

    #=======================================================================================================
    #   DISPLAY
    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        """More readable representation, relying on ``IOU.ReprMx``."""
        r = []
        if self.__class__ not in self._classesWithoutClassnamedisplay:
            classname  = getattr( self, '_displayClassname', self.__class__.__name__ )
        else:
            classname = None
        name = getattr( self, 'name', None )
        if classname:  r.append( classname )
        if name:       r.append( name      )
        r.extend( map( repr, self ) )
        return '{%s}' % ( ', '.join( r ) )


    #=======================================================================================================
    #   MODIFICATION
    #-------------------------------------------------------------------------------------------------------
    def add( self, *P ):
        for p in P:
            super( EzSetMx, self ).add( p )

    #=======================================================================================================
    #   MODIFIED METHODS OF ORIGINAL SET FORMULATION
    #-------------------------------------------------------------------------------------------------------
    def intersection(self, other):
        """Return the intersection of two sets as a new set.

        (I.e. all elements that are in both sets.)

        (Method copied from module ``sets`` with change in set construction parameters.)"""
        if not isinstance(other, BaseSet):
            other = Set(*other)
        if len(self) <= len(other):
            little, big = self, other
        else:
            little, big = other, self
        common = ifilter(big._data.has_key, little)
        return self.__class__(*common)

    #-------------------------------------------------------------------------------------------------------
    def symmetric_difference(self, other):
        """Return the symmetric difference of two sets as a new set.

        (I.e. all elements that are in exactly one of the sets.)

        (Method copied from module ``sets`` with change in set construction parameters.)"""
        result = self.__class__()
        data = result._data
        value = True
        selfdata = self._data
        try:
            otherdata = other._data
        except AttributeError:
            otherdata = Set(*other)._data
        for elt in ifilterfalse(otherdata.has_key, selfdata):
            data[elt] = value
        for elt in ifilterfalse(selfdata.has_key, otherdata):
            data[elt] = value
        return result

    #-------------------------------------------------------------------------------------------------------
    def difference(self, other):
        """Return the difference of two sets as a new Set.

        (I.e. all elements that are in this set and not in the other.)

        (Method copied from module ``sets`` with change in set construction parameters.)"""
        result = self.__class__()
        data = result._data
        try:
            otherdata = other._data
        except AttributeError:
            otherdata = Set(*other)._data
        value = True
        for elt in ifilterfalse(otherdata.has_key, self):
            data[elt] = value
        return result

    #-------------------------------------------------------------------------------------------------------
    def symmetric_difference_update(self, other):
        """Update a set with the symmetric difference of itself and another.
        (Method copied from module ``sets`` with change in set construction parameters.)"""
        data = self._data
        value = True
        if not isinstance(other, BaseSet):
            other = Set(*other)
        for elt in other:
            if elt in data:
                del data[elt]
            else:
                data[elt] = value

    #-------------------------------------------------------------------------------------------------------
    def difference_update(self, other):
        """Remove all elements of another set from this set.
        (Method copied from module ``sets`` with change in set construction parameters.)"""
        data = self._data
        if not isinstance(other, BaseSet):
            other = Set(*other)
        for elt in ifilter(data.has_key, other):
            del data[elt]

    #-------------------------------------------------------------------------------------------------------
    def union(self, other):
        """Return the union of two sets as a new set.

        (I.e. all elements that are in either set.)

        (Method copied from module ``sets`` with change in set construction parameters.)"""
        result = self.__class__(*self)
        result._update(other)
        return result


############################################################################################################
#
#
#
#===========================================================================================================
class EzChangeNotificationSetMx( EzSetMx ):

    """A set that calls ``~.onChange()`` whenever it detects an imminent change in the elements it holds,
    and ``~.afterChange()`` when the change has been performed. The following methods are wrapped and cause
    change notifications: ``~.__iand__()`` *(update a set with the intersection of itself and another)*;
    ``~.__ior__()`` *(update a set with the union of itself and another)*; ``~.__isub__()`` *(remove all
    elements of another set from this set)*; ``~.__ixor__()`` *(update a set with the symmetric difference
    of itself and another)*; ``~.add()`` *(add an element to a set (this has no effect if the element is
    already present))*; ``~.clear()`` *(remove all elements from this set)*; ``~.difference_update()``
    *(remove all elements of another set from this set)*; ``~.discard()`` *(remove an element from a set if
    it is a member (if the element is not a member, do nothing))*; ``~.intersection_update()`` *(update a
    set with the intersection of itself and another)*; ``~.pop()`` *(remove and return an arbitrary set
    element)*; ``~.remove()`` *(remove an element from a set (it must be a member; if the element is not a
    member, raise a KeyError))*; ``~.symmetric_difference_update()`` *(update a set with the symmetric
    difference of itself and another)*; ``~.union_update()`` *(update a set with the union of itself and
    another)*; ``~.update()`` *(add all values from an iterable (such as a list or file))*."""

    #-------------------------------------------------------------------------------------------------------
    _doOnChange = True # ``~.onChange()`` is only called if ``~._doOnChange`` is true

    #-------------------------------------------------------------------------------------------------------
    def onChange( self ):
        pass

    #-------------------------------------------------------------------------------------------------------
    def afterChange( self ):
        pass

#-----------------------------------------------------------------------------------------------------------
def _x_getChangeAwareMethod( methodname ):
    def R( self, *P, **Q ):
        """Cache-aware version of ``Set.%s()``: calling this method causes ``~_invalidateL1Cache()``
        to be called, q.v."""
        if self._doOnChange:
            self.onChange()
        #print '~~~~+++', self.add.__doc__ ###DEBUG###
        R = getattr( super( EzChangeNotificationSetMx, self ), methodname )( *P, **Q )
        self.afterChange()
        return R
    R.__doc__ %= methodname
    return R

for _x_methodname in """__iand__ __ior__ __isub__ __ixor__ add clear difference_update discard
    intersection_update pop remove symmetric_difference_update union_update update""".split():
    #if ChangeNotificationSetMx.__dict__.get( _x_methodname, None ) is not None:
    setattr( EzChangeNotificationSetMx, _x_methodname, _x_getChangeAwareMethod( _x_methodname ) )


############################################################################################################
#
#   FULL VERSION OF MIXIN CLASSES:
#
#-----------------------------------------------------------------------------------------------------------
class EzSet( EzSetMx, Set ):
    """Full version (including derivation from ``Set``) of mix-in class ``EzSetMx``."""
    pass

#-----------------------------------------------------------------------------------------------------------
S = Zet = EzSet

#-----------------------------------------------------------------------------------------------------------
class EzChangeNotificationSet( EzChangeNotificationSetMx, Set ):
    """Full version (including derivation from ``Set``) of mix-in class ``EzChangeNotificationSetMx``."""
    pass


############################################################################################################
#
#   AFTERTHOUGHTS
#
#-----------------------------------------------------------------------------------------------------------
#EzSetMx._classesWithoutClassnamedisplay = EzSetMx( EzSetMx, EzSet )
EzSetMx._classesWithoutClassnamedisplay = EzSet( EzSetMx, EzSet )


#-----------------------------------------------------------------------------------------------------------
for name in dir():
    if name.startswith( '_x_' ):
        exec 'del %s' % name
del name

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    a = S( 1,2,3 )
    print a | S( 4,5,6,1 )



