#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from __future__ import division
#-----------------------------------------------------------------------------------------------------------
from time import time as                _now


############################################################################################################
#
#
#
#===========================================================================================================
class seconds( object ):
    #-------------------------------------------------------------------------------------------------------
    perMinute                        = 60
    perHour                          = 3600
    perDay                           = 86400
    perWeek                          = 604800
    perMonth                         = 2592000
    perYear                          = 31557600
seconds = seconds()



############################################################################################################
#
#   STOPWATCH
#
#===========================================================================================================
class Stopwatch( object ):
    """A stopwatch class for low-profile profiling purposes.

    Example::

        #---------------------------------------------------------------------------------------------------
        import math
        import md5

        #---------------------------------------------------------------------------------------------------
        def f0():
            for i in range( 100000 ):
                j = math.sin( i )

        #---------------------------------------------------------------------------------------------------
        def f1():
            for i in range( 100000 ):
                m = md5.new()
                m.update( hex( i ) )
                m.digest()

        #---------------------------------------------------------------------------------------------------
        s = Stopwatch( 'some benchmarks' )
        #---------------------------------------------------------------------------------------------------
        s.start( 'sinloop' )
        f0()
        s.stop()
        #---------------------------------------------------------------------------------------------------
        s.start( 'md5loop' )
        f1()
        s.stop()
        #---------------------------------------------------------------------------------------------------
        s.finish()
        print s

    Output::

        Profile some benchmarks
        TOTAL               :    1.8330s
        sinloop             :    0.4110s
        md5loop             :    1.4220s

    """

    #-------------------------------------------------------------------------------------------------------
    now = property( lambda self: _now() )

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, name = 'UNKNOWN' ):
        self.name       = name
        self.starttimes = []
        self.stoptimes  = []
        self.deltatimes = []
        self.marks      = []
        self.displayAsPercentages = False

    #-------------------------------------------------------------------------------------------------------
    def __len__( self ):
        return len( self.marks )

    #-------------------------------------------------------------------------------------------------------
    def getLastIndex( self ):
        return len( self ) - 1
    lastIndex = property( getLastIndex )

    #-------------------------------------------------------------------------------------------------------
    def start( self, mark = None ):
        R = len( self )
        if not R and mark != 'TOTAL':
            self.start( 'TOTAL' )
            R = len( self )
        self.starttimes.append( self.now )
        self.stoptimes.append( None )
        self.deltatimes.append( None )
        self.marks.append( mark )
        return R

    #-------------------------------------------------------------------------------------------------------
    def stop( self, index = None ):
        if index is None:
            index = self.lastIndex
        self.stoptimes[ index ] = stoptime = self.now
        self.deltatimes[ index ] = stoptime - self.starttimes[ index ]

    #-------------------------------------------------------------------------------------------------------
    def finish( self ):
        for index in xrange( len( self ) ):
            if self.stoptimes[ index ] is None:
                self.stop( index )

    #-------------------------------------------------------------------------------------------------------
    def __repr__( self ):
        maxlength = max( map( len, self.marks ) )
        fs = '%%-%ds:\t%%9.4f\t%%s' % maxlength
        if self.displayAsPercentages:
            totaltime = self.deltatimes[ 0 ]
        else:
            totaltime = max( *self.deltatimes[ 1 : ] )
        R = []
        i = -1
        for mark, deltatime in zip( self.marks, self.deltatimes, ):
            i += 1
            if i == 0 and not self.displayAsPercentages:
                percentagebar = ''
            else:
                try:
                    percentage = int( round( deltatime / totaltime * 50 ) )
                except ZeroDivisionError:
                    percentage = 0
                percentagebar = '*' * percentage
            R.append( fs % ( mark, deltatime, percentagebar ) )
        R = 'Profile %s\n%s' % ( self.name, '\n'.join( R ), )
        return R


#===========================================================================================================
if __name__ == '__main__':

    #-------------------------------------------------------------------------------------------------------
    import math
    import md5

    #-------------------------------------------------------------------------------------------------------
    def f0():
        for i in range( 100000 ):
            j = math.sin( i )

    #-------------------------------------------------------------------------------------------------------
    def f1():
        for i in range( 100000 ):
            m = md5.new()
            m.update( hex( i ) )
            m.digest()

    #-------------------------------------------------------------------------------------------------------
    s = Stopwatch( 'some benchmarks' )
    #-------------------------------------------------------------------------------------------------------
    s.start( 'sinloop' )
    f0()
    s.stop()
    #-------------------------------------------------------------------------------------------------------
    s.start( 'md5loop' )
    f1()
    s.stop()
    #-------------------------------------------------------------------------------------------------------
    s.finish()
    print s
    print s.marks
