#
# -*- coding: utf-8 -*-
#

"""


ToDo:
============================================================================================================

-   Semantics of a directory lock should be that no read/write file operations are allowed there at all. As
    a consequence, we should check when acquireing a file lock whether the directory is locked.


$RCSfile: Lock.py,v $
$Revision: 3 $
$Author: wolfgang.lipp $
$Date: 2005-07-01 17:15:59 +0200 (Fri, 01 Jul 2005) $
$Id: Lock.py,v 1.6 2004/11/22 17:35:33 wolfgang.lipp Exp $

"""

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
#   from __future__ import division
#-----------------------------------------------------------------------------------------------------------
import inspect as                       _inspect
import os as                            _os
import os.path as                       _p
import time as                          _time
from random import random as            _random
from glob import glob as                _glob
from tempfile import                    NamedTemporaryFile
#-----------------------------------------------------------------------------------------------------------
_abspath                                = _p.abspath
_basename                               = _p.basename
_dirname                                = _p.dirname
_basename                               = _p.basename
_exists                                 = _p.exists
_isdir                                  = _p.isdir
_now                                    = _time.time
_sleep                                  = _time.sleep
_split                                  = _p.split
osLocatorNameSeparator                  = _os.sep
standardLocatorNameSeparator            = '/'
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_registerFinalizer                      = u.finalizer.register
_unregisterFinalizer                    = u.finalizer.unregister
_Beacon                                 = u.Beacon.Beacon
_BeaconUpError                          = u.Beacon.BeaconUpError
_isLockedError                          = 'islocked'
verbose                                 = False
#   verbose                                 = True

############################################################################################################
#
#
#
#===========================================================================================================
class LockError( IOError ): pass
class UnableToRaiseBeaconError( LockError ): pass
class UnableToEstablishLockError( LockError ): pass
class UnableToLowerBeaconError( LockError ): pass
class UnableToReleaseLockError( LockError ): pass

############################################################################################################
#
#
#
#===========================================================================================================
class Lock( object ):

    #-------------------------------------------------------------------------------------------------------
    autoAcquire                 = True
    anylockExtension            = '.lock'
    anylockPrefix               = 'lock.'
    readlockExtension           = '.r%s' % anylockExtension
    writelockExtension          = '.w%s' % anylockExtension
    beaconExtension             = '.beacon'
    readlockGlobformat          = '%%s%s%%s.*%s'    % ( anylockPrefix, readlockExtension, )
    writelockGlobformat         = '%%s%s%%s%s'      % ( anylockPrefix, writelockExtension, )
    anylockGlobformat           = '%%s%s%%s*%s'     % ( anylockPrefix, anylockExtension, )
    releaseMaxTrials            = 15    # probably not needed
    releaseSleeptime            = 0.1   # same as acquireSleeptime???
    timeout                     = 15
    acquireSleeptime            = 0.1
    releaseLatency              = 0.05 # probably not needed
    sleeptimeVariance           = 0.5 # relative to ~.*Sleeptime
    verbose                     = verbose
    timeWaited                  = None

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, locator, write = False ):
        #---------------------------------------------------------------------------------------------------
        self._write             = write
        #---------------------------------------------------------------------------------------------------
        targetLocator, isdir    = self._normalizeLocator( locator )
        self.targetLocator      = targetLocator
        #---------------------------------------------------------------------------------------------------
        #   We don't assume a file to be locked must be existent, but a directory must, so we check with
        #   isdir:
        if isdir:
            self.isDirectory    = True
            self.homeLocator    = targetLocator
            self.basename       = ''
        else:
            self.isDirectory    = False
            self.homeLocator, self.basename = _split( targetLocator )
            self.homeLocator    = self._normalizeDirectorySlash( self.homeLocator )
            assert _isdir( self.homeLocator )
        self._locator               = None
        self._beacon                = None
        self._readlockGloblocator   = None
        self._writelockGloblocator  = None
        self._anylockGloblocator    = None
        self._locked                = False
        self._lockfile              = None
        self._locator               = None
        if self.autoAcquire:
            self.acquire()
        self._t0                    = None


    #-------------------------------------------------------------------------------------------------------
    def getLocator( self ):
        """Return actual locator of the lockfile, if one exists, or ``None``, if none exists."""
        if self._lockfile is None: return None
        if self._locator is None:
            self._locator, dummy = self._normalizeLocator( self._lockfile.name )
        return self._locator
    locator = property( getLocator )

    #-------------------------------------------------------------------------------------------------------
    def getReadlockGloblocator( self ):
        if self._readlockGloblocator is None:
            self._readlockGloblocator = self._buildGloblocator( self.homeLocator, self.basename, write = False )
        return self._readlockGloblocator
    readlockGloblocator = property( getReadlockGloblocator )

    #-------------------------------------------------------------------------------------------------------
    def getWritelockGloblocator( self ):
        if self._writelockGloblocator is None:
            self._writelockGloblocator = self._buildGloblocator( self.homeLocator, self.basename, write = True )
        return self._writelockGloblocator
    writelockGloblocator = property( getWritelockGloblocator )

    #-------------------------------------------------------------------------------------------------------
    def getAnylockGloblocator( self ):
        if self._anylockGloblocator is None:
            self._anylockGloblocator = self._buildGloblocator( self.homeLocator, self.basename, write = None )
        return self._anylockGloblocator
    anylockGloblocator = property( getAnylockGloblocator )

    #-------------------------------------------------------------------------------------------------------
    def getGloblocator( self ):
        """Depending on whether this is a read or write lock, return a write lock glob locator or a read lock
        glob locator."""
        if self.write:
            return self.writelockGloblocator
        return self.readlockGloblocator
    globlocator = property( getGloblocator )

    #-------------------------------------------------------------------------------------------------------
    def f( this, homeLocator, basename, write ):
        if   write is None: format = this.anylockGlobformat
        elif write:         format = this.writelockGlobformat
        else:               format = this.readlockGlobformat
        #print '~~~~', format
        return format % ( homeLocator, basename, )
    _buildGloblocator = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def getWrite( self ):
        return self._write
    def setWrite( self, value ):
        raise NotImplementedError
        #   if value == self._write: return
        #   release lock
        #   acquire lock
    write = property( getWrite, setWrite )

    #-------------------------------------------------------------------------------------------------------
    def getRead( self ):
        return not self._write
    read = property( getRead )

    #-------------------------------------------------------------------------------------------------------
    def getLocked( self ):
        return self._locked
    locked = property( getLocked )

    #-------------------------------------------------------------------------------------------------------
    def f( this, locator ):
        if not locator.endswith( standardLocatorNameSeparator ):
            locator = '%s%s' % ( locator, standardLocatorNameSeparator, )
        return locator
    _normalizeDirectorySlash = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, locator ):
        locator = _abspath( locator ).replace( osLocatorNameSeparator, standardLocatorNameSeparator )
        isdir = False
        if _isdir( locator ):
            isdir = True
            locator = this._normalizeDirectorySlash( locator )
        return locator, isdir
    _normalizeLocator = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def getHasLock( self ):
        return bool( _glob( self.anylockGloblocator ) )
    hasLock = property( getHasLock )

    #-------------------------------------------------------------------------------------------------------
    def getHasWritelock( self ):
        return bool( _glob( self.writelockGloblocator ) )
    hasWritelock = property( getHasWritelock )

    #-------------------------------------------------------------------------------------------------------
    def getIsLockedForWriting( self ):
        return self.getHasLock()
    isLockedForWriting = property( getIsLockedForWriting )

    #-------------------------------------------------------------------------------------------------------
    def getHasReadlock( self ):
        return bool( _glob( self.readlockGloblocator ) )
    hasReadlock = property( getHasReadlock )

    #-------------------------------------------------------------------------------------------------------
    def getIsLockedForReading( self ):
        #   This method returns ``~.isLockedForWriting`` for non-exclusive locks, i.e. a file is only locked for
        #   reading when it is locked by a write lock.
        return self.getHasWritelock()
    isLockedForReading = property( getIsLockedForReading )

    #-------------------------------------------------------------------------------------------------------
    def getIsLockedByMe( self ):
        return self._locked
    isLockedByMe = property( getIsLockedByMe )

    #   #-------------------------------------------------------------------------------------------------------
    #   def getIsWritelockedByMe( self ):
    #       return self._write and self.isLockedByMe
    #   isWritelockedByMe = property( getIsWritelockedByMe )
    #
    #   #-------------------------------------------------------------------------------------------------------
    #   def getIsReadlockedByMe( self ):
    #       return ( not self._write ) and self.isLockedByMe
    #   isReadlockedByMe = property( getIsReadlockedByMe )

    #   #-------------------------------------------------------------------------------------------------------
    #   def f( self ):
    #       lockfileCount = len( self._glob( self.globlocator, isGlobLocator = True, isNormalized = True ) )
    #       if self.isLockedByMe: lockfileCount -= 1
    #       return lockfileCount > 0
    #   isLockedByOther = property( f ); del f


    #=======================================================================================================
    #   BEACON
    #-------------------------------------------------------------------------------------------------------
    def raiseBeacon( self ):
        try:
            self._beacon = _Beacon(
                dir     = self.homeLocator,
                prefix  = '%s%s' % ( self.anylockPrefix, self.basename, ),
                suffix  = self.beaconExtension,
                )
            #print '******************', self._beacon.name
            return True
        except _BeaconUpError:
            return False

    #-------------------------------------------------------------------------------------------------------
    def lowerBeacon( self ):
        if self._beacon is None: return
        self._beacon.close()
        self._beacon = None

    #=======================================================================================================
    #   TIME
    #-------------------------------------------------------------------------------------------------------
    def _outtimed( self ):
        return _now() - self._t0 > self.timeout

    #-------------------------------------------------------------------------------------------------------
    def _vary( self, time ):
        """Given ``time``, a float, return a value that is randomly off from this value by at most the ratio
        specified by ``~.sleeptimeVariance``. Ex.: when ``time`` is 10 and the variation is
        ``0.5``, then the result will be something between ``10 - 10 * 0.5`` ``==`` ``5`` and ``10 + 10 *
        0.5`` ``==`` ``15``."""
        return time + time * ( _random() * 2 - 1 ) * self.sleeptimeVariance

    #=======================================================================================================
    #   ACQUIRING AND RELEASING LOCK
    #-------------------------------------------------------------------------------------------------------
    def acquire( self ):
        if self._locked: return True
        self._acquireOrRelease( self._acquireLock, 'acquire' )
        self.timeWaited = _now() - self._t0
        return True

    #-------------------------------------------------------------------------------------------------------
    def reacquire( self ):
        if self._locked: self.release()
        return self.acquire()

    #-------------------------------------------------------------------------------------------------------
    def release( self ):
        if not self._locked: return True
        self._acquireOrRelease( self._releaseLock, 'release' )
        self.timeWaited = _now() - self._t0
        return True

    #-------------------------------------------------------------------------------------------------------
    def verboseCallback( self, msg ):
        globLocator = '%s%s*' % ( self.homeLocator, self.anylockPrefix, )
        #globLocator = self.anylockGloblocator
        filenames = map( _basename, _glob( globLocator ) )
        R = 'Lock.verboseCallback %s: (%s) %s' % ( globLocator, msg, ', ' .join( filenames ), )
        logfile = file( 'c:/temp/lock.log', 'a' )
        logfile.write( R )
        logfile.write( '\n' )
        logfile.close()

    #-------------------------------------------------------------------------------------------------------
    def _acquireOrRelease( self, lockmethod, msgword ):
        V = self.verbose
        self._t0 = _now()
        while not self._outtimed():
            try:
                try:
                    if not self.raiseBeacon(): raise _isLockedError
                    if V: self.verboseCallback( 'beacon raised' )
                    locked = lockmethod()
                    if V: self.verboseCallback( 'lock acquired or released' )
                finally:
                    self.lowerBeacon()
                    if V: self.verboseCallback( 'beacon lowered' )
                if locked: return True
            except _isLockedError:
                pass
            _sleep( self._vary( self.acquireSleeptime ) )
            if V: self.verboseCallback( 'try again' )
        raise UnableToEstablishLockError( ''
            +   "Timeout while trying to %s lock for '%s'." % ( msgword, self.targetLocator, )
            )

    #-------------------------------------------------------------------------------------------------------
    def _acquireLock( self ):
        if self._locked: return True
        try:
            if self.write:
                if self.isLockedForWriting:
                    raise _isLockedError
                try:
                    self._lockfile = _Beacon(
                        dir     = self.homeLocator,
                        prefix  = '%s%s' % ( self.anylockPrefix, self.basename, ),
                        suffix  = self.writelockExtension,
                        )
                except _BeaconUpError:
                    raise _isLockedError
            else:
                if self.isLockedForReading:
                    raise _isLockedError
                self._lockfile = NamedTemporaryFile(
                    dir     = self.homeLocator,
                    prefix  = '%s%s.' % ( self.anylockPrefix, self.basename, ),
                    suffix  = self.readlockExtension,
                    )
        except _isLockedError:
            self._locked = False;
            return False
        _registerFinalizer( self.finalize )
        self._locked = True;
        return True

    #-------------------------------------------------------------------------------------------------------
    def _releaseLock( self ):
        self._lockfile.close()
        self._lockfile = None
        self._locked = False
        _unregisterFinalizer( self.finalize )
        return True

    #-------------------------------------------------------------------------------------------------------
    def forceRelease( self ):
        if self._lockfile is not None:
            try:
                self._lockfile.close()
                self._lockfile = None
            except:
                pass
        if self._beacon is not None:
            try:
                self._beacon.close()
                self._beacon = None
            except:
                pass

    #-------------------------------------------------------------------------------------------------------
    def finalize( self ):
        try:
            if not self.release(): raise IOError()
        except:
            self.forceRelease()

    #-------------------------------------------------------------------------------------------------------
    def __del__( self ):
        finalize = getattr( self, 'finalize', None )
        if finalize is not None:
            finalize()

    #-------------------------------------------------------------------------------------------------------
    def report( self ):
        names = """ anylockGloblocator basename hasLock hasReadlock hasWritelock homeLocator isLockedByMe
        isLockedForReading isLockedForWriting locator readlockGloblocator targetLocator writelockGloblocator
        readlockGlobformat writelockGlobformat anylockGlobformat """.split()
        names.sort()
        R = []
        R.append( ', '.join( _glob( '%slock*' % ( self.homeLocator, ) ) ) )
        for name in names:
            R.append( '%-20s: %s' % ( name, getattr( self, name ), ) )
        R = '\n'.join( R )
        return R


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    sep = '#' * 108

#       def engage():
#
#           print sep
#           locator = Lock.normalizeLocator( 'c:/temp' )
#           print Lock._buildGloblocator( locator )
#           print Lock.getIsLocked( locator )
#           lock = Lock( locator )
#           print Lock.getIsLocked( locator )
#           print lock.locator
#           print lock.globlocator
#           print lock.hasLock
#           print lock.isLockedByMe
#           print lock.isLockedByOther
#
#           print sep
#           locator = Lock.normalizeLocator( 'c:/temp/_jscratch.py' )
#           lock = Lock( locator )
#           print lock.locator
#           print lock.globlocator
#           print lock.hasLock
#           print lock.isLockedByMe
#           print lock.isLockedByOther
#           lock.release()
#           #del lock
#           #del u.finalizer.finalizers[:]
#
#       g.callCgi( engage )



