#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from __future__ import division
#-----------------------------------------------------------------------------------------------------------
import os.path as                       ospath
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
EzClassMc                               = u.EzClassMc.EzClassMc
fsi                                     = u.filesystemIdioms
prefixesByNamespaces                    = g.cfg.prefixesByNamespaces
namespacesByPrefix                      = g.cfg.namespacesByPrefix
#-----------------------------------------------------------------------------------------------------------


############################################################################################################
#
#
#
#===========================================================================================================
class SchemaDescriptionException( Exception ): pass

#===========================================================================================================
class InconsistencyException( SchemaDescriptionException, ValueError ):

    #-------------------------------------------------------------------------------------------------------
    def f( this, inconsistentNamespaceUri, namespacePrefix ):
        return this( ""
            +   "Inconsistent setting for namespace URI: %r. " % ( inconsistentNamespaceUri, )
            +   "This setting does not match the existing namespace prefix %r." % ( namespacePrefix, )
            )
    inconsistentNamespaceUri = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, inconsistentNamespacePrefix, namespaceUri ):
        return this( ""
            +   "Inconsistent setting for namespace Prefix: %r. " % ( inconsistentNamespacePrefix, )
            +   "This setting does not match the existing namespace URI %r." % ( namespaceUri, )
            )
    inconsistentNamespacePrefix = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, inconsistentFilename, filelocator ):
        return this( ""
            +   "Inconsistent setting for filename: %r. " % ( inconsistentFilename, )
            +   "This setting does not match the existing locator %r." % ( filelocator, )
            )
    inconsistentFilename = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, inconsistentFilelocator, filename ):
        return this( ""
            +   "Inconsistent setting for filelocator: %r. " % ( inconsistentFilelocator, )
            +   "This setting does not match the existing name %r." % ( filename, )
            )
    inconsistentFilelocator = classmethod( f ); del f



############################################################################################################
#
#
#
#===========================================================================================================
class SchemaDescription:
    __metaclass__   = EzClassMc
    __slots__       = True
    __obligatory__  = """isTemplate""".split()
    __optional__    = dict(
        cmfNamespacePrefix  = None,
        namespaceUri        = None,
        namespacePrefix     = None,
        filename            = None,
        filelocator         = None,
        #   templatename        = None,
        #   templatelocator     = None,
        )

    #-------------------------------------------------------------------------------------------------------
    def setNamespaceUri( self, value ):
        namespacePrefix         = self.gaugeForUri( value )
        if self._namespacePrefix is not None and self._namespacePrefix != namespacePrefix:
            raise InconsistencyException.inconsistentNamespaceUri( value, self._namespacePrefix )
        self._namespaceUri      = value
        self._namespacePrefix   = namespacePrefix

    #-------------------------------------------------------------------------------------------------------
    def setNamespacePrefix( self, value ):
        namespaceUri            = self.uriForGauge( value )
        if self._namespaceUri is not None and self._namespaceUri != namespaceUri:
            raise InconsistencyException.inconsistentNamespacePrefix( value, self._namespaceUri )
        self._namespacePrefix   = value
        self._namespaceUri      = namespaceUri

    #-------------------------------------------------------------------------------------------------------
    def setFilelocator( self, value ):
        filename = ospath.basename( value )
        if self._filename is not None and self._filename != filename:
            raise InconsistencyException.inconsistentFilelocator( value, self._filename )
        self._filelocator       = value
        self._filename          = filename

    #-------------------------------------------------------------------------------------------------------
    def setFilename( self, value ):
        if self._filelocator is not None and value != ospath.basename( self._filelocator ):
            raise InconsistencyException.inconsistentFilename( value, self._filelocator )
        self._filename          = value

    #-------------------------------------------------------------------------------------------------------
    def setCmfNamespacePrefix( self, value ):
        #-------------------------------------------------------------------------------------------------------
        if value is None:
            self._cmfNamespacePrefix = None
            return
        #-------------------------------------------------------------------------------------------------------
        if not isinstance( value, basestring ):
            raise TypeError( ''
                +   "Value for namespace prefix must be a string, is %r" % ( value, )
                )
        #-------------------------------------------------------------------------------------------------------
        if '/' in value:
            value = self.gaugeForUri( value )
        #-------------------------------------------------------------------------------------------------------
        self._cmfNamespacePrefix = value


#           #print '~~~~~~~~~~~', value
#           if value is not None:
#               namespaceRegistry = g.namespaceRegistry
#               #print '#########', namespaceRegistry ###DEBUG###
#               if namespaceRegistry.isUri( value ):
#                   value = namespaceRegistry.id( value )
#               #print '~~~~~~~~~~~', value ###DEBUG###
#           self._cmfNamespacePrefix = value

    #   #-------------------------------------------------------------------------------------------------------
    #   def setTemplatelocator( self, value ):
    #       templatename = ospath.basename( value )
    #       if self._templatename is not None and self._templatename != templatename:
    #           raise InconsistencyException.inconsistentFilelocator( value, self._templatename )
    #       self._templatelocator       = value
    #       self._templatename          = templatename
    #
    #   #-------------------------------------------------------------------------------------------------------
    #   def setTemplatename( self, value ):
    #       if self._templatelocator is not None and value != ospath.basename( self._templatelocator ):
    #           raise InconsistencyException.inconsistentFilename( value, self._templatelocator )
    #       self._templatename          = value
    #
    #   #   #-------------------------------------------------------------------------------------------------------
    #   #   def f():
    #   #       def fget( self ):
    #   #           return self.templateFilename is not None and self.filename is None
    #   #       return locals()
    #   #   isTemplate = property( **f() ); del f
    #
    #   #-------------------------------------------------------------------------------------------------------
    #   def getIsTemplate( self ):
    #       return self.templatename is not None and self.filename is None

    #   #-------------------------------------------------------------------------------------------------------
    #   def setIsTemplate( self, value ):
    #       raise AttributeError( "can't set attribute" )

    #-------------------------------------------------------------------------------------------------------
    def f( this, uri ):
        return prefixesByNamespaces.get( uri, None )

#           #print '++++++++++++++++', uri ###DEBUG###
#           if uri is None: return None
#           namespaceRegistry = g.namespaceRegistry
#           if not namespaceRegistry.isUri( uri ):
#               raise ValueError( "This is not a syntactically valid namespace URI: %r." % ( uri, ) )
#           return namespaceRegistry[ uri ]
    gaugeForUri = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, gauge ):
        return namespacesByPrefix.get( gauge, None )

#           namespaceRegistry = g.namespaceRegistry
#           if not namespaceRegistry.isId( gauge ):
#               raise ValueError( "This is not a syntactically valid namespace prefix: %r." % ( gauge, ) )
#           return namespaceRegistry[ gauge ]
    uriForGauge = classmethod( f ); del f

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    sep = '#' * 108


    #-------------------------------------------------------------------------------------------------------
    print sep
    sd = SchemaDescription( isTemplate = True, namespacePrefix = 'abcd12', filelocator = 'c:/foo/templates/xy.xml' )
    print sd
    try:
        sd.namespacePrefix = 'Vegetweb10'
        assert False
    except ValueError, e:
        print
        print sep
        print '(%s) %s' % ( e.__class__.__name__, e, )
        print sep

    print sd
    try:
        sd.filename = 'fooXXX.xml'
        assert False
    except ValueError, e:
        print
        print sep
        print '(%s) %s' % ( e.__class__.__name__, e, )
        print sep

#       print dir( SchemaDescription )
#       print dir( sd )
#       print getattr( SchemaDescription, '__slots__', None )
    print sd
    #sd.isTemplate = 9


