#
# -*- coding: utf-8 -*-



#   #-----------------------------------------------------------------------------------------------------------
#   import  xml.dom.ext                                                     as          xmlExt
#   #-----------------------------------------------------------------------------------------------------------
#   from    configtool.configuration.container  import cfg                  as          cfg
#   from    configtool.utilities                import xmlIdioms            as          xi
#   from    configtool.utilities                                            import      debugdumpers
#   #-----------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------
from    xml.dom.ext.reader                  import Sax2                 as          mySax
import                                                                              xml
import                                                                              xml.dom
import                                                                              xml.dom.ext
import                                                                              StringIO
import                                                                              xml.xpath
#-----------------------------------------------------------------------------------------------------------

############################################################################################################
#
#
#
#===========================================================================================================
domImplementation = xml.dom.getDOMImplementation( '4DOM' )

############################################################################################################
#
#
#
#===========================================================================================================
class document:

    #=======================================================================================================
    reader = mySax.Reader(
            validate        = False,
            keepAllWs       = True,
            catName         = None,
            saxHandlerClass = mySax.XmlDomGenerator,
            parser          = None,
            )

    #=======================================================================================================
    def fromString( this, text, normalize = True, ownerDocument = None ):
        return this.fromStream( StringIO.StringIO( text ), normalize, ownerDocument )
    fromString = classmethod( fromString )

    #=======================================================================================================
    def fromStream( this, stream, normalize = True, ownerDocument = None ):
        R = this.reader.fromStream( stream, ownerDocument )
        if normalize:
            this.normalize( R )
        return R
    fromStream = classmethod( fromStream )

    #=======================================================================================================
    def fromFilename( this, filename, normalize = True, ownerDocument = None ):
        """"""
        try:
            stream = file( filename, 'rb' )
            R = this.fromStream( stream = stream, normalize = normalize, ownerDocument = ownerDocument )
        finally:
            try:    stream.close()
            except: pass
        return R
    fromFilename = classmethod( fromFilename )

    #=======================================================================================================
    def normalize( this, node ):
        """Normalize a ``node`` (which defaults to the document represented by the instance) so all text
        nodes it contains are non-empty with whitespace both trimmed and equalized (see
        `g.typo.normalizeWhitespace()`)."""
        selectParentsOfTextNodes = './/text()/..'
        for node in select( selectParentsOfTextNodes, node ):
            #   Link all adjacent text nodes:
            node.normalize()
            #-----------------------------------------------------------------------------------------------
            #   Normalize texts contained in all children of node which are text nodes. If normalized
            #   text turns out to be empty (i.e. it consisted of nothing but whitespace), remove
            #   that child from node; otherwise, replace original text by normalized text:
            for child in node.childNodes:
                if child.nodeType == node.TEXT_NODE:
                    text = g.typo.normalizeWhitespace( child.nodeValue )
                    if not text:
                        node.removeChild( child )
                    else:
                        node.nodeValue = text
    normalize = classmethod( normalize )

    #===========================================================================================================
    def create( this, namespaceURI = None, qualifiedName = None, doctype = None ):
        R = domImplementation.createDocument( namespaceURI, qualifiedName, doctype )
        return R
    create = classmethod( create )

############################################################################################################
#
#
#
#===========================================================================================================
def repr( node, usePrettyprint = True ):
    #-------------------------------------------------------------------------------------------------------
    S = StringIO.StringIO()
    #-------------------------------------------------------------------------------------------------------
    if usePrettyprint:
        method = xml.dom.ext.PrettyPrint
    else:
        method = xml.dom.ext.Print
    method( node, S )
    R = S.getvalue().strip()
    S.close()
    #-------------------------------------------------------------------------------------------------------
    return R

#===========================================================================================================
def select( pattern, context ):
    return xml.xpath.Evaluate( pattern, context )

#===========================================================================================================
def xSelect( node, pattern ):
    return xml.xpath.Evaluate( pattern, node )

#===========================================================================================================
def selectFirst( pattern, context ):
    R = select( pattern, context )
    if R:
        return R[ 0 ]
    return None

#===========================================================================================================
def xSelectFirst( node, pattern = None ):
    if pattern is None:
        return node
    R = xSelect( node, pattern )
    if R:
        return R[ 0 ]
    return None

#===========================================================================================================
def getNodes_DEPRECATED_( node, name ):
    pattern = './/%(name)s' % vars()
    return select( pattern, node ) # .nodeValue

#===========================================================================================================
def getNode_DEPRECATED_( node, name ):
    R = getNodes_DEPRECATED_( node, name )
    if R:
        return R[ 0 ]
    return None

#===========================================================================================================
def getTextNode_DEPRECATED_( node, name = None ):
    if name is None:
        pattern = './text()'
    else:
        pattern = './/%s/text()' % ( name, )
    return selectFirst( pattern, node )

#===========================================================================================================
def getTextNode_DEPRECATED_( node, name = None ):
    if name is None:
        pattern = './text()'
    else:
        pattern = './/%s/text()' % ( name, )
    return selectFirst( pattern, node )

#===========================================================================================================
def getText_DEPRECATED_( node, name = None ):
    R = getTextNode_DEPRECATED_( node, name )
    if R is not None:
        R = R.nodeValue
    return R

#===========================================================================================================
def getTextNode( node, pattern = None ):
    node = xSelectFirst( node, pattern )
    if node is not None:
        return xSelectFirst( node, './text()' )

#===========================================================================================================
def getText( node, pattern = None ):
    textNode = getTextNode( node, pattern )
    if textNode is not None:
        return textNode.nodeValue

#   #===========================================================================================================
#   def getChildNode( node, name ):
#       return xi.select( './%(name)s' % vars(), node ) # .nodeValue

#===========================================================================================================
class Selector:

    #selection_method = lambda self, *P, **Q: xSelectFirst( *P, **Q )

    def __init__( self, node, pattern = None ):
        self.node       = node
        self.pattern    = pattern
        self.select()

    def new( self ):
        self.node       = None
        self.pattern    = None
        self.selection  = None

    def select( self ):
        if self.pattern is None:
            self.selection = self.node
        else:
            self.selection = xSelectFirst( self.node, self.pattern )
        return self.selection

#===========================================================================================================
def appendChild1( selector0, selector1 ):
    if selector0.selection is None:
        raise TypeError( "Trunk node can be None: %s" % ( selector0, ) )
    if selector1.selection is None:
        raise TypeError( "Graft node can be None: %s" % ( selector1, ) )
    node1 = selector0.node.ownerDocument.importNode( selector1.selection, deep = True )
    selector0.selection.appendChild( node1 )

#===========================================================================================================
def appendChild2( node0, node1, pattern0 = None, pattern1 = None ):
    owner0 = node0.ownerDocument
    owner1 = node1.ownerDocument
    if pattern0 is not None: node0 = selectFirst( pattern0, node0 )
    if pattern1 is not None: node1 = selectFirst( pattern1, node1 )
    if node0 is None or node1 is None:
        raise TypeError( "Neither Node can be None: %s, %s" % ( node0, node1 ) )
    if owner0 is not owner1:
        node1 = owner0.importNode( node1, deep = True )
    node0.appendChild( node1 )

#===========================================================================================================
def getCopyForNewOwner( node0, node1 ):
    owner0 = node0.ownerDocument
    return owner0.importNode( node1, deep = True )

#===========================================================================================================
def appendChild( node0, node1 ):
    node0.appendChild( getCopyForNewOwner( node0, node1 ) )


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':
    import xmlIdioms
    d = xmlIdioms.createDocument()
    e = d.createElement( 'foo' )
    text = d.createTextNode( 'Some textual content.' )
    d.appendChild( e )
    e.appendChild( text )
    e.setAttribute( "info", "hello world" )
    print xmlIdioms.repr( d )

