#
# -*- coding: utf-8 -*-

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from __future__ import division
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_feds                                   = u.feds
_S                                      = u.EzSet.S
_acquire                                = u.pylon.acquire
#   getDatasourceDescriptors                = u.DatasourceDescriptor.getDatasourceDescriptors
_DatabaseDescription                    = u.databaseMetadataReader.DatabaseDescription
#-----------------------------------------------------------------------------------------------------------
_xsdbDatasource                         = _feds.xsdbDatasource.xsdbDatasource


############################################################################################################
#
#
#
#===========================================================================================================
class buildConnectionString( object ):

    #-------------------------------------------------------------------------------------------------------
    def f( this, choice ):
        return [ 'no', 'yes' ][ bool( choice ) ]
    _chooseYesNo = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    class forOdbc( object ):

        #---------------------------------------------------------------------------------------------------
        connectionStringFormat = ( ''
            +   'dsn=%(dsn)s;uid=%(user)s;pwd=%(password)s;clear_auto_commit=%(clear_auto_commit)s'
            )

        #---------------------------------------------------------------------------------------------------
        def f( this, dsn, user = '', password = '', clear_auto_commit = True ):
            return dict(
                dsn                 = dsn,
                user                = user,
                password            = password,
                clear_auto_commit   = clear_auto_commit,
                )
        asDict = classmethod( f ); del f

        #---------------------------------------------------------------------------------------------------
        def f( this, *P, **Q ):
            vars = this.asDict( *P, **Q )
            vars[ 'clear_auto_commit' ] = buildConnectionString._chooseYesNo( vars[ 'clear_auto_commit' ] )
            return this.connectionStringFormat % vars
        asString = classmethod( f ); del f



############################################################################################################
#
#
#
#===========================================================================================================
class DsFactsGrabber( object ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, dsa, dsCredentials = None ):
        self.dsa                    = dsa
        self._dsDescriptions        = None
        self._dsCredentials         = None
        self._psf                   = None
        #---------------------------------------------------------------------------------------------------
        if dsCredentials is not None:
            self.dsCredentials = dsCredentials

    #-------------------------------------------------------------------------------------------------------
    def getPsf( self ):
        psfi = u.PsfCmfIdioms.PsfIdioms
        if self._psf is None:
            self._psf = psfi.psfForDsa( self.dsa )
        return self._psf
    psf = property( getPsf )

    #-------------------------------------------------------------------------------------------------------
    #   DATA SOURCE ACCESS CREDENTIALS
    #-------------------------------------------------------------------------------------------------------
    def getDsCredentials( self, fromPsf = True ):
        """Try to return data source access credentials either from provider setup file or from the
        data source meta data representation."""
        if self._dsCredentials is None:
            if fromPsf:
                psfi = u.PsfCmfIdioms.PsfIdioms
                self._dsCredentials = psfi.getDsCredentials( self.psf )
            else:
                raise NotImplementedError
        return self._dsCredentials

    #-------------------------------------------------------------------------------------------------------
    def setDsCredentials( self, Q ):
        self._dsCredentials = {}
        self._dsCredentials.update( Q )

    #-------------------------------------------------------------------------------------------------------
    def delDsCredentials( self ):
        self._dsCredentials = None

    #-------------------------------------------------------------------------------------------------------
    dsCredentials = property( getDsCredentials, setDsCredentials, delDsCredentials )

    #-------------------------------------------------------------------------------------------------------
    #   DATA SOURCE META DATA
    #-------------------------------------------------------------------------------------------------------
    def getDsDescription( self ):
        if self._dsDescriptions is None:
            #self._dsDescriptions = psfi.getDsDescription( self.psf )
            self._dsDescriptions = _DatabaseDescription.fromDsCredentials( self.dsCredentials )
        return self._dsDescriptions
    dsDescription = property( getDsDescription )

    #-------------------------------------------------------------------------------------------------------
    def iterAllFacts( self ):
        #---------------------------------------------------------------------------------------------------
        yield getFact.datasource(
            dsa             = self.dsa,
            dsCredentials   = self.dsCredentials,
            )
        #---------------------------------------------------------------------------------------------------
        for tableDescription in self.dsDescription:
            tablename = tableDescription.name
            yield getFact.table(
                dsa         = self.dsa,
                tablename   = tablename,
                )
            #-----------------------------------------------------------------------------------------------
            for fieldDescription in tableDescription:
                yield getFact.field(
                    dsa         = self.dsa,
                    tablename   = fieldDescription._tablename,
                    fieldname   = fieldDescription.name,
                    datatype    = fieldDescription.type,
                    )

    #   #-------------------------------------------------------------------------------------------------------
    #   def iterAllFacts( self ):
    #       dsa = self.dsa
    #       #---------------------------------------------------------------------------------------------------
    #       R = dict(
    #           category    = 'datasource',
    #           dsa         = dsa,
    #           credentials = dict( self.dsCredentials ),
    #           )
    #       yield R
    #       for tableDescription in self.iterTableDescriptions():
    #           for whatever in self.iterFieldDescriptionsForTableDescription( tableDescription ):
    #               yield whatever
    #
    #   #-------------------------------------------------------------------------------------------------------
    #   def iterTableDescriptions( self ):
    #       for tableDescription in self.dsDescription:
    #           tablename = tableDescription.name
    #           R = dict(
    #               category    = 'table',
    #               dsa         = self.dsa,
    #               tablename   = tablename,
    #               )
    #           yield R
    #
    #   #-------------------------------------------------------------------------------------------------------
    #   def iterFieldDescriptionsForTableDescription( self, tableDescription ):
    #       for fieldDescription in tableDescription:
    #           R = dict(
    #               category    = 'column',
    #               dsa         = self.dsa,
    #               tablename   = fieldDescription._tablename,
    #               fieldname   = fieldDescription.name,
    #               datatype    = fieldDescription.type,
    #               )
    #           yield R
    #



############################################################################################################
#
#
#
#===========================================================================================================
class getFact( object ):

    #-------------------------------------------------------------------------------------------------------
    class category:
        pass
        """Class that enumerates Fact categories used by ``DsDescriptions``::

            category.datasource = 'datasource'
            category.table      = 'table'

        and so on, for all names. In addition, there is a set of names::

            category.__names__  = Set( 'datasource', 'table', ... )

        """
    category.__names__ = __names__ = _S()
    for _name in """datasource table tablealias field""".split():
        setattr( category, _name, _name )
        __names__.add( _name )
    del __names__, _name


    #-------------------------------------------------------------------------------------------------------
    #   FACT DEFINITIONS FOR EACH CATEGORY
    #-------------------------------------------------------------------------------------------------------
    def f( this, dsa, dsCredentials = None, **Q ):
        return dict(
            category        = this.category.datasource,
            dsCredentials   = dsCredentials,
            **Q
            )
    datasource = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, dsa, tablename, **Q ):
        return dict(
            category        = this.category.table,
            dsa             = dsa,
            tablename       = tablename,
            **Q
            )
    table = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, dsa, tablename, tablealias, **Q ):
        return dict(
            category        = this.category.tablealias,
            dsa             = dsa,
            tablename       = tablename,
            tablealias      = tablealias,
            **Q
            )
    tablealias = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f(
        this,
        dsa,
        tablename,
        fieldname,
        datatype                = None,
        isPrimarykey            = None,
        foreignkeyTablealias    = None,
        foreignkeyFieldname     = None,
        **Q
        ):
        return dict(
            category                = this.category.field,
            dsa                     = dsa,
            tablename               = tablename,
            fieldname               = fieldname,
            datatype                = datatype,
            isPrimarykey            = isPrimarykey,
            foreignkeyTablealias    = foreignkeyTablealias,
            foreignkeyFieldname     = foreignkeyFieldname,
            **Q
            )
    field = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, xsdbDescription ):
        """The ``xsdb`` description objects stored in the ``xsdb`` database should be converted using this
        class method instead of being returned raw to the user. The first reason is that they are, as
        objects, probably unknown to the consumer and of little use to them, which is why we want our own
        objects here or standard dictionaries. The second reason is that ``getFact.forXsdbDescription()``
        tries to apply the standard fact factory functions like ``getFact.table()``, which ensures a higher
        degree of consistency."""
        Q = xsdbDescription.mapping
        category = Q.pop( 'category', None )
        if category is not None and category in this.category.__names__:
            return getattr( this, category )( **Q )
        return dict( Q, category = category )
    forXsdbDescription = classmethod( f ); del f

############################################################################################################
#
#
#
#===========================================================================================================
class DsDescriptions( _xsdbDatasource ):

    #-------------------------------------------------------------------------------------------------------
    DsFactsGrabber    = DsFactsGrabber
    getFact                 = getFact
    buildConnectionString   = buildConnectionString

    #-------------------------------------------------------------------------------------------------------
    #   ADDING ENTITIES
    #-------------------------------------------------------------------------------------------------------
    def addDatasource( self, dsa, dsCredentials = None ):
        fact = getFact.datasource(
            dsa                 = dsa,
            dsCredentials       = dsCredentials,
            )
        self.add( None, **fact )

    #-------------------------------------------------------------------------------------------------------
    def addTable( self, dsa, tablename ):
        fact = getFact.table(
            dsa                 = dsa,
            tablename           = tablename,
            )
        self.add( None, **fact )

    #-------------------------------------------------------------------------------------------------------
    def addTablealias( self, dsa, tablename, tablealias ):
        fact = getFact.tablealias(
            dsa                 = dsa,
            tablename           = tablename,
            tablealias          = tablealias,
            )
        self.add( None, **fact )

    #-------------------------------------------------------------------------------------------------------
    def addField( self,
        dsa,
        tablename,
        fieldname,
        datatype                = None,
        isPrimarykey            = None,
        foreignkeyTablealias    = None,
        foreignkeyFieldname     = None,
        ):
        fact = getFact.field(
            dsa                     = dsa,
            tablename               = tablename,
            fieldname               = fieldname,
            datatype                = datatype,
            isPrimarykey            = isPrimarykey,
            foreignkeyTablealias    = foreignkeyTablealias,
            foreignkeyFieldname     = foreignkeyFieldname,
            )
        self.add( None, **fact )

    #-------------------------------------------------------------------------------------------------------
    def tryToAddDatabase( self, dsa, dsCredentials = None ):
        R = 0
        dsFactsGrabber = DsFactsGrabber( dsa, dsCredentials = dsCredentials )
        for fact in dsFactsGrabber.iterAllFacts():
            self.add( None, **fact )
            R += 1
        return R

    #-------------------------------------------------------------------------------------------------------
    def catchFacts( self, *P, **Q ):
        for xsdbDescriptionList in self.catchDescriptionsByOid( *P, **Q ).itervalues():
            for xsdbDescription in xsdbDescriptionList:
                yield getFact.forXsdbDescription( xsdbDescription )

