#
# -*- coding: utf-8 -*-

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from __future__ import division
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_MkDatasource                           = u.feds.MkDatasource.MkDatasource
_Fact                                   = u.feds.MkDatasource.Fact
_FactHyper                              = u.feds.MkDatasource.FactHyper
_Table                                  = u.feds.MkDatasource.Table
_TableHyper                             = u.feds.MkDatasource.TableHyper
_ontonymAnalyzer                        = u.OntonymAnalyzer.OntonymAnalyzer()
#-----------------------------------------------------------------------------------------------------------
_T                                      = _MkDatasource.underlyingDatatypes
D                                       = dict

############################################################################################################
#
#   PROPERTIES
#
############################################################################################################
#   PROPERTY FACT
#===========================================================================================================
class PropertyFact( _Fact ):

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return g.pickler.loads( getattr( -self, 'value' ) )
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            setattr( -self, 'value', g.pickler.dumps( value ) )
        #---------------------------------------------------------------------------------------------------
        return locals()
    value = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return (-self).gauge
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            if ':' in value:
                value = value.replace( ':', '' )
            (-self).gauge = value
        #---------------------------------------------------------------------------------------------------
        return locals()
    gauge = property( **f() ); del f


############################################################################################################
#   PROPERTY TABLE HYPER
#===========================================================================================================
class PropertyTableHyper( _TableHyper ):

    #-------------------------------------------------------------------------------------------------------
    factclass = PropertyFact
    typesByFacetname = D(
        gauge               = _T.string,
        locator             = _T.string,
        verb                = _T.string,
        value               = _T.string,    # pickled representations of Python objects
        )

    #-------------------------------------------------------------------------------------------------------
    class characterize( object ):
        #-------------------------------------------------------------------------------------------------------
        def f( this, ontonym, verb = None ):
            graphname, locator = _analyzeOntonym( ontonym, termnrsAsIntegers = False )
            if locator.isdigit():   R = this.forTermnr(  graphname, locator, verb )
            else:                   R = this.forLocator( graphname, locator, verb )
            return R
        forOntonym = classmethod( f ); del f

        #---------------------------------------------------------------------------------------------------
        def f( this, graphname, termnr, verb = None ):
            d = D(
                gauge           = graphname,
                termnr          = termnr,
                )
            if verb is not None:
                d[ 'verb' ] = verb
            return D( property = d )
        forTermnr = classmethod( f ); del f

        #---------------------------------------------------------------------------------------------------
        def f( this, graphname, locator, verb = None ):
            d = D(
                gauge           = graphname,
                locator         = locator,
                )
            if verb is not None:
                d[ 'verb' ] = verb
            return D( property = d )
        forLocator = classmethod( f ); del f

############################################################################################################
#   PROPERTY TABLE
#===========================================================================================================
class PropertyTable( _Table ):
    __hyperclass__      = PropertyTableHyper


############################################################################################################
#
#   GRAPHS
#
############################################################################################################
#   GRAPH FACT
#===========================================================================================================
class GraphFact( _Fact ):
    pass

############################################################################################################
#   GRAPH TABLE HYPER
#===========================================================================================================
class GraphTableHyper( _TableHyper ):

    #-------------------------------------------------------------------------------------------------------
    factclass = GraphFact

    #-------------------------------------------------------------------------------------------------------
    typesByFacetname = D(
        name                = _T.string,
        length              = _T.integer,
        )

        #   #---------------------------------------------------------------------------------------------------
        #   def f( this, name, length = -1 ):
        #       return D(
        #           name            = name,
        #           length          = length,
        #           )
        #   graph = classmethod( f ); del f


############################################################################################################
#   GRAPH TABLE
#===========================================================================================================
class GraphTable( _Table ):
    __hyperclass__      = GraphTableHyper

############################################################################################################
#
#   NODES
#
############################################################################################################
#   NODE FACT
#===========================================================================================================
class NodeFact( _Fact ):
    pass

############################################################################################################
#   NODE TABLE HYPER
#===========================================================================================================
class NodeTableHyper( _TableHyper ):

    #-------------------------------------------------------------------------------------------------------
    factclass = NodeFact

    #-------------------------------------------------------------------------------------------------------
    typesByFacetname = D(
        gauge           = _T.string,
        locator         = _T.string,
        name            = _T.string,
        level           = _T.integer,
        termnr          = _T.integer,
        parentTermnr    = _T.integer,    # Primary structure -- must be in same graph! -1 indicates root
        isLeaf          = _T.integer,    # also True if all childnames start with ``'@'``
        )

    #-------------------------------------------------------------------------------------------------------
    class characterize( object ):

        #-------------------------------------------------------------------------------------------------------
        def f( this, ontonym ):
            graphname, locator = _ontonymAnalyzer.analyze( ontonym, termnrsAsIntegers = False )
            if locator.isdigit():   R = this.forTermnr(  graphname, locator )
            else:                   R = this.forLocator( graphname, locator )
            return R
        forOntonym = classmethod( f ); del f

        #---------------------------------------------------------------------------------------------------
        def f( this, graphname, termnr ):
            termnr = int( termnr )
            return D( node = D(
                gauge           = graphname,
                termnr          = termnr,
                ))
        forTermnr = classmethod( f ); del f

        #---------------------------------------------------------------------------------------------------
        def f( this, graphname, locator ):
            return D( node = D(
                gauge           = graphname,
                locator         = locator,
                ))
        forLocator = classmethod( f ); del f

        #   #---------------------------------------------------------------------------------------------------
        #   def f(
        #       this,
        #       gauge,
        #       name,
        #       locator             = '',
        #       level               = -1,
        #       termnr              = -1,
        #       parentTermnr        = -1,
        #       isLeaf              = False,
        #       ):
        #       return D(
        #           gauge           = gauge,
        #           locator         = locator,
        #           name            = name,
        #           level           = level,
        #           termnr          = termnr,
        #           parentTermnr    = parentTermnr,
        #           isLeaf          = isLeaf,
        #           )
        #   node = classmethod( f ); del f

############################################################################################################
#   NODE TABLE
#===========================================================================================================
class NodeTable( _Table ):
    __hyperclass__      = NodeTableHyper



############################################################################################################
#
#
#
#===========================================================================================================
class Grove( _MkDatasource ):

    #-------------------------------------------------------------------------------------------------------
#       characterize        = characterize
#       describe            = describe
    tableclassesByTablename  = D(
        property    = PropertyTable,
        graph       = GraphTable,
        node        = NodeTable,
        )

    #-------------------------------------------------------------------------------------------------------
    def addAllFactoids( self ):
        self.addFactoids(
            graph       =    GraphTableHyper.typesByFacetname,
            node        =     NodeTableHyper.typesByFacetname,
            property    = PropertyTableHyper.typesByFacetname,
            )

    #-------------------------------------------------------------------------------------------------------
    def addProperty( self, gauge, locator, verb, value ):
        self.addFacts(
            property = D(
                gauge   = gauge,
                locator = locator,
                verb    = verb,
                value   = value,
                )
            )

    #-------------------------------------------------------------------------------------------------------
    def catchPropertyTable( self, ontonym = None, verb = None ):
        graphname, locator, termnr  = self.tripletForOntonymOrNones( ontonym )
        resultsForLocator           = resultsForTermnr = None
        tableclass                  = self._getTableclassForTablename( 'property' )
        if bool( ontonym ) and graphname is None: return ()
        #---------------------------------------------------------------------------------------------------
        Q = {}
        if graphname is not None:   Q[ 'gauge' ]    = graphname
        if locator is not None:     Q[ 'locator' ]  = locator
        if verb is not None:        Q[ 'verb' ]     = verb
        #---------------------------------------------------------------------------------------------------
        if Q:
            resultsForLocator       = self.catchTable( property = Q )
        #---------------------------------------------------------------------------------------------------
        if termnr is not None:      Q[ 'locator' ]  = termnr
        if Q:
            resultsForTermnr        = self.catchTable( property = Q )
        #---------------------------------------------------------------------------------------------------
        if ( resultsForLocator is not None ) and ( resultsForTermnr is not None ):
            mkViewObject            = ( -resultsForLocator ).union( -resultsForTermnr )
        elif resultsForLocator is not None:
            mkViewObject            = resultsForLocator
        elif resultsForTermnr  is not None:
            mkViewObject            = resultsForTermnr
        else:
            mkViewObject            = -self.factoid( 'property' )
        #---------------------------------------------------------------------------------------------------
        R                           = tableclass( self, mkViewObject, 'property' )
        return R

    #-------------------------------------------------------------------------------------------------------
    def iterRelatedNodefacts( self, ontonym = None, verb = None ):
        propertytable = self.catchPropertyTable( ontonym = ontonym, verb = verb )
        for propertyfact in propertytable:
            contextGauge = ':%s:' % ( propertyfact.gauge, )
            targetlocators = propertyfact.value
            if not targetlocators: continue
            for targetlocator in targetlocators:
                if isinstance( targetlocator, ( list, tuple, ) ):
                    assert len( targetlocator ) == 2
                    targetGraphname, targetLocator = targetlocator
                    targetGauge = ':%s:' % ( targetGraphname, )
                    targetOntonym = ( targetGauge, targetlocator, )
                else:
                    targetOntonym = ( contextGauge, targetlocator, )
                nodefact = self.nodefactForOntonymOrNone( targetOntonym )
                yield nodefact

    #-------------------------------------------------------------------------------------------------------
    def iterGauges( self ):
        for fact in self.catchTable( graph = None ):
            yield fact.name

    #-------------------------------------------------------------------------------------------------------
    def iterAllNodes( self ):
        return self.iterNodesForGauges()

    #-------------------------------------------------------------------------------------------------------
    def iterNodesForGauges( self, *gauges ):
        if not gauges: gauges = self.iterGauges()
        for gauge in gauges:
            for fact in self.catchTable( node = D( gauge = gauge ) ):
                yield fact

    #-------------------------------------------------------------------------------------------------------
    def nodefactForOntonymOrNone( self, ontonym ):
        try:
            return self.nodefactForOntonym( ontonym )
        except KeyError:
            return None

    #-------------------------------------------------------------------------------------------------------
    def nodefactForOntonym( self, ontonym ):
        nodeTableHyperclass = self.tableclassesByTablename[ 'node' ].__hyperclass__
        characterization    = nodeTableHyperclass.characterize.forOntonym( ontonym )
        table = self.catchTable( **characterization )
        facts = list( (+table).facts )
        if len( facts ) > 1:
            raise KeyError( ''
                +   "The description %s did not deliver a unique characterization." % ( ontonym, )
                )
        if len( facts ) < 1:
            raise KeyError( ''
                +   "The description %s did not deliver any result." % ( ontonym, )
                )
        return facts[ 0 ]

    #-------------------------------------------------------------------------------------------------------
    def analyzeOntonym( self, ontonym, termnrsAsIntegers = False ):
        return _ontonymAnalyzer.analyze( ontonym, termnrsAsIntegers = termnrsAsIntegers )

    #-------------------------------------------------------------------------------------------------------
    def tripletForOntonymOrNones( self, ontonym, termnrsAsIntegers = False ):
        try:
            return self.tripletForOntonym( ontonym, termnrsAsIntegers = termnrsAsIntegers )
        except KeyError:
            return None, None, None

    #-------------------------------------------------------------------------------------------------------
    def tripletForOntonym( self, ontonym, termnrsAsIntegers = False ):
        nodefact = self.nodefactForOntonym( ontonym )
        termnr = nodefact.termnr
        if not termnrsAsIntegers:
            termnr = _ontonymAnalyzer.castNumericalTermnrToText( termnr )
        return nodefact.gauge, nodefact.locator, termnr

        #   graphname, locator = _ontonymAnalyzer.analyze( ontonym, termnrsAsIntegers = False )
        #   if locator.isdigit():
        #       termnr = int( locator )
        #       locator = get locator ( termnr )
        #   else:
        #       termnr = resolve termnr( locator )


