#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
#   from __future__ import division
#-----------------------------------------------------------------------------------------------------------
import                                  sys
import                                  imp
import os.path as                       _ospath
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_feds                                   = u.feds
_S                                      = u.EzSet.S
#-----------------------------------------------------------------------------------------------------------
_FederatorAbc                           = _feds.FederatorAbc.FederatorAbc
_MetaFederator                          = _feds.MetaFederator.MetaFederator


############################################################################################################
#
#   FEDERATOR IMPLEMENTATION
#
#===========================================================================================================
class SessionsFed( _FederatorAbc ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, metatopic, readonly = False ):
        #---------------------------------------------------------------------------------------------------
        self.dsData     = _MetaFederator.getDatasource( 'Sessiondata',  readonly = readonly )
        self.dsSessions = _MetaFederator.getDatasource( 'Sessions',     readonly = readonly )
        #---------------------------------------------------------------------------------------------------
        self.dsSessions.setDsData( self.dsData )
        #---------------------------------------------------------------------------------------------------
        super( SessionsFed, self ).__init__( metatopic, readonly, self.dsSessions, self.dsData )

    #-------------------------------------------------------------------------------------------------------
    def reopen( self, readonly = None ):
        if readonly is None:    readonly = self.readonly
        else:                   self.readonly = readonly
        self.reopenDsSessions( readonly = readonly )
        self.reopenDsData( readonly = readonly )

    #-------------------------------------------------------------------------------------------------------
    def reopenDsSessions( self, readonly = None ):
        if readonly is None:    readonly = self.readonly
        else:                   self.readonly = readonly
        self.dsSessions.reopen( readonly = readonly )

    #-------------------------------------------------------------------------------------------------------
    def reopenDsData( self, readonly = None ):
        if readonly is None:    readonly = self.readonly
        else:                   self.readonly = readonly
        self.dsData.reopen( readonly = readonly )

    #-------------------------------------------------------------------------------------------------------
    def recreateDatasources( self ):
#           self.dsSessions.close()
#           self.dsData.close()
#           dsSessions = _MetaFederator.getDatasource( 'Sessions', readonly = False )
#           _MetaFederator.clearDatasourceCacheForMetatopic(  )
        self.dsSessions.recreateDatasource()
        self.dsData.recreateDatasource()
#           dsSessions.close()
#           _MetaFederator.clearDatasourceCacheForMetatopic( SessionsFed )

    #-------------------------------------------------------------------------------------------------------
    def _getOidsOfDanglingData( self, includeInactive = True ):
        #---------------------------------------------------------------------------------------------------
        dsSessions  = self.dsSessions
        dsData      = self.dsData
        #---------------------------------------------------------------------------------------------------
        R = _S()
        if includeInactive:
            activesids      = _S( *self.activeSids   )
            inactivesids    = _S( *self.inactiveSids )
        else:
            activesids      = dsSessions.sids
            inactivesids    = ()
        #---------------------------------------------------------------------------------------------------
        for oid, descriptions in dsData.catchDescriptionsByOid().iteritems():
            #print oid
            for description in descriptions:
                #print description
                sid = description[ 'sid' ]
                if sid in R: continue
                if sid not in activesids or sid in inactivesids: R.add( oid )
        #---------------------------------------------------------------------------------------------------
        return R

    #-------------------------------------------------------------------------------------------------------
    def removeDanglingData( self, includeInactive = True ):
        #---------------------------------------------------------------------------------------------------
        dsSessions  = self.dsSessions
        dsData      = self.dsData
        #---------------------------------------------------------------------------------------------------
        for oid in self._getOidsOfDanglingData( includeInactive ):
            dsData.remove( oid )
        #self.reopenDsData()

    #-------------------------------------------------------------------------------------------------------
    def addData( self, session, data, *P, **Q ):
        #---------------------------------------------------------------------------------------------------
        dsSessions  = self.dsSessions
        dsData      = self.dsData
        #---------------------------------------------------------------------------------------------------
        #print '++++++++++++', session, session.__class__, session.sid
        dsData.addData( session, data, *P, **Q )
        dsSessions.touch( session )
        #---------------------------------------------------------------------------------------------------

    #-------------------------------------------------------------------------------------------------------
    def replaceData( self, session, data, *P, **Q ):
        return self.dsData.replaceData( session, data, *P, **Q )

    #-------------------------------------------------------------------------------------------------------
    def getData( self, session, *P, **Q ):
        return self.dsData.getData( session, *P, **Q )

#       #-------------------------------------------------------------------------------------------------------
#       #   PROBLEMS AND WARNINGS
#       #-------------------------------------------------------------------------------------------------------
#       def addProblemForSender( self, sender, session, msg ):
#           return self.dsData.addProblemForSender( session, sender, msg )
#
#       #-------------------------------------------------------------------------------------------------------
#       def popProblemsForSender( self, session, sender ):
#           return self.dsData.popProblemsForSender( session, sender )
#
#       #-------------------------------------------------------------------------------------------------------
#       def addWarningForSender( self, session, sender, msg ):
#           return self.dsData.addWarningForSender( session, sender, msg )
#
#       #-------------------------------------------------------------------------------------------------------
#       def popWarningsForSender( self, session, sender ):
#           return self.dsData.popWarningsForSender( session, sender, msg )

    #-------------------------------------------------------------------------------------------------------
    #   SESSIONS RETRIEVAL
    #-------------------------------------------------------------------------------------------------------
    def getSessions( self ):
        return self.dsSessions.sessions
    sessions = property( getSessions )

    #-------------------------------------------------------------------------------------------------------
    def deactivateSession( self, session ):
        self.dsSessions.deactivateSession( session )

    #-------------------------------------------------------------------------------------------------------
    def getActiveSessions( self ):
        dsSessions = self.dsSessions
        return dsSessions.catch( active = True, category = dsSessions.CATEGORY_SESSION )
    activeSessions = property( getActiveSessions )

    #-------------------------------------------------------------------------------------------------------
    def getInactiveSessions( self ):
        dsSessions = self.dsSessions
        return dsSessions.catch( active = False, category = dsSessions.CATEGORY_SESSION )
    inactiveSessions = property( getInactiveSessions )

    #-------------------------------------------------------------------------------------------------------
    def getActiveSids( self ):
        dsSessions = self.dsSessions
        return dsSessions.catchOids( active = True, category = dsSessions.CATEGORY_SESSION )
    activeSids = property( getActiveSids )

    #-------------------------------------------------------------------------------------------------------
    def getInactiveSids( self ):
        dsSessions = self.dsSessions
        return dsSessions.catchOids( active = False, category = dsSessions.CATEGORY_SESSION )
    inactiveSids = property( getInactiveSids )

    #-------------------------------------------------------------------------------------------------------
    def getNewSession( self ):
        dsSessions = self.dsSessions
        return dsSessions.getNewSession()

    #-------------------------------------------------------------------------------------------------------
    def getOldOrNewSidAndSessionForSid( self, sid ):
        dsSessions = self.dsSessions
        return dsSessions.getOldOrNewSidAndSessionForSid( sid )

    #-------------------------------------------------------------------------------------------------------
    def getValidSessionOrNoneForSid( self, sid = None ):
        dsSessions = self.dsSessions
        try:
            if sid is None: raise KeyError()
            R = dsSessions.getSessionForSid( sid )
            if not R.active or R.tooOld:
                raise KeyError()
        except KeyError:
            R = None
        return R

    #-------------------------------------------------------------------------------------------------------
    def removeSession( self, session ):
        self.dsSessions.remove( session.sid )
        self.removeDanglingData() # this should be more targetted: remove data that belongs to session
        #self.reopen()

    #-------------------------------------------------------------------------------------------------------
    def removeSessionAndDataForSidIfPossible( self, sid ):
        R = self.dsSessions.removeSessionForSidIfPossible( sid )
        R = self.dsData.removeDataForSidIfPossible( sid ) or R
        return R

    #-------------------------------------------------------------------------------------------------------
    def deactivateOldSessions( self, remove = False ):
        for session in list( self.sessions ):
            if not session.tooOld: continue
            if remove:
                self.dsSessions.remove( session.sid )
            else:
                self.deactivateSession( session )
        self.removeDanglingData()

    #-------------------------------------------------------------------------------------------------------
    def touchSession( self, session ):
        dsSessions = self.dsSessions
        dsSessions.touch( session )

    #-------------------------------------------------------------------------------------------------------
    def __del__( self ):
        if self.dsSessions is not None: self.dsSessions.close()
        if self.dsData     is not None: self.dsData.close()

    #-------------------------------------------------------------------------------------------------------
    def _acquireLock( self ):
        super( SessionsFed, self )._acquireLock( write = True ) # always acquire write lock


