#
# -*- coding: utf-8 -*-


"""Module to ensure finalizer code is run on script termination. This is especially important given that the
presence of the ``globalNamebase`` module may mean that reference counts never drop to zero, thus blocking
the execution of ``__del__`` methods. Consumers that want to have a callable be executed on script
termination pass that executable to ``register()``; ``configtoolDispatcher.cgy`` will take care of executing
``finalize()``."""


#-----------------------------------------------------------------------------------------------------------
finalizerIds    = {}
finalizers      = []

#-----------------------------------------------------------------------------------------------------------
def register( x ):
    xid = id( x )
    if xid not in finalizerIds:
        finalizerIds[ xid ] = len( finalizers )
        finalizers.append( x )

#-----------------------------------------------------------------------------------------------------------
def unregister( x ):
    xid = id( x )
    if xid in finalizerIds:
        finalizers.pop( finalizerIds[ xid ] )
        del finalizerIds[ xid ]

#-----------------------------------------------------------------------------------------------------------
def finalize():
    for finalizer in finalizers:
        finalizer()


