#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
fsi                                     = u.filesystemIdioms
_flowReader                             = u.flowReader
#   _S                                      = u.EzSet.S
#-----------------------------------------------------------------------------------------------------------
_EzGraph                                = u.EzGraph
_ontonymAnalyzer                        = u.OntonymAnalyzer.OntonymAnalyzer()
#-----------------------------------------------------------------------------------------------------------
Grove                                   = _EzGraph.Grove
Graph                                   = _EzGraph.Graph
Node                                    = _EzGraph.Node



#===========================================================================================================
def parse( source, name = None, allowMultipleRootnodes = False, verbose = False ):
    R = Graph( name, allowMultipleRootnodes = allowMultipleRootnodes )
    class GoOn: pass
    levelstack          = {}
    lastLevel           = 0
    lastLine            = ''
    lastLineWasCommand  = False
    thisLineIsCommand   = False
    messages = _flowReader.Flow.messages
    #-------------------------------------------------------------------------------------------------------
    flow = _flowReader.Flow( source )
    for event in flow.iterEvents():
        #---------------------------------------------------------------------------------------------------
        message     = event.message
        line        = event.line
        lineIndex   = event.lineIndex
        level       = event.level
        commandName = event.commandName
        #---------------------------------------------------------------------------------------------------
        try:
            #-----------------------------------------------------------------------------------------------
            if verbose:
                print '%04x%10s %3d %2d %r' % (
                    message, messages.marker( message ), lineIndex, level, line, ),
            #-----------------------------------------------------------------------------------------------
            if message & messages.start:    raise GoOn                                              #+   ==>
            if message & messages.stop:     raise GoOn                                              #+   ==>
            if message & messages.empty:    raise GoOn                                              #+   ==>
            if message & messages.comment:  raise GoOn                                              #+   ==>
            #-----------------------------------------------------------------------------------------------
            if lastLineWasCommand and ( message & messages.indent ):
                #-------------------------------------------------------------------------------------------
                #   Option 1: raise exception
                #   line = flow.lines[ lineIndex ]
                #   raise _flowReader.IndentationError._errorMsgWithLine(
                #       "No indentation after command allowed", **vars() )
                #-------------------------------------------------------------------------------------------
                #   Option 2: do nothing
                pass
                #-------------------------------------------------------------------------------------------
                #   Option 3: go on
                #   raise GoOn
            #-----------------------------------------------------------------------------------------------
            if message & messages.command:
                thisLineIsCommand = True
                if level > 1:
                    parent = levelstack[ level - 1 ]
                else:
                    parent = None
                #-------------------------------------------------------------------------------------------
                if verbose:
                    print 'belongs to: %s, argument: %s' % ( parent, event.commandArgument, ),
                    print '#+ CMD %s' % ( commandName, ),
                if commandName == 'graphname':
                    if verbose: print '#+ SET GRAPH NAME %s' % name,
                    name = event.commandArgument
                    assert R.name is None
                    R.name = event.commandArgument
                    raise GoOn                                                                      #+   ==>
                if parent is not None:
                    parent.events.append( event )
            #-----------------------------------------------------------------------------------------------
            if message & messages.plain:
                name = line.strip()
                #print 'start adding elements to', name
                try:
                    parent = levelstack[ level - 1 ]
                except KeyError:
                    parent = None
                if verbose: print '#+ NEW NODE %s, parent = %s' % ( name, parent, ),
                node = R.newNode( name, parentOrParentLocator = parent )
                levelstack[ level ] = node
            #-----------------------------------------------------------------------------------------------
            if message & messages.indent:   pass
            if message & messages.undent:   lastLevel = level
            if message & messages.plain:    raise GoOn                                              #+   ==>
            if message & messages.other:    raise NotImplementedError
        #---------------------------------------------------------------------------------------------------
        except GoOn:                                                                                #+   <==
            lastLine = line
        #---------------------------------------------------------------------------------------------------
        lastLineWasCommand = thisLineIsCommand
        if verbose: print
    #-------------------------------------------------------------------------------------------------------
    return R


#===========================================================================================================
def _getGraphnameAndLocatorFromCommandArgument( contextGraph, commandArgument ):
    gauge = ':%s:' % ( contextGraph.name, )
    graphname, locator = _ontonymAnalyzer.analyze( ( commandArgument, gauge, ) )
    return graphname, locator

#===========================================================================================================
def _resolveCommand(
    grove, thisGraph, thisNode, thatGraphname, thatLocator, event, verbose = False ):
    commandSymbol   = event.commandSymbol
    commandName     = event.commandName
    attributeName   = event.attributeName
    try:
        thatGraph = grove[ thatGraphname ]
        thatNode = thatGraph.nodes[ thatLocator ]
    except KeyError:
#           graphname = thatGraphname
#           raise _flowReader.ValueError.unknownGraphname( **vars() )
        thatNode = ( thatGraphname, thatLocator )
    #if verbose: print '\t%s %s %s' % ( commandName, commandSymbol, thatNode, )
    if   commandName == 'subnode':      thisNode.addSubnode(    thatNode )
    elif commandName == 'equivalent':   thisNode.addEquivalent( thatNode )
    elif commandName == 'supernode':    thisNode.addSupernode(  thatNode )
    else:
        raise NotImplementedError( ''
            +   "Unknown command: %s" % ( event, )
            )

#===========================================================================================================
def resolveCommands( grove, verbose = False ):
    for thisGraphname, thisGraph in grove.iteritems():
        if verbose: print '=' * 108
        if verbose: print 'graph :%s:' % ( thisGraphname, )
        for thisNode in thisGraph:
            events = thisNode.events
            if events:
                if verbose: print '-' * 108
                if verbose: print '\t%s' % ( thisNode, )
                for event in events:
                    thatGraphname, thatLocator = \
                        _getGraphnameAndLocatorFromCommandArgument( thisGraph, event.commandArgument )
                    if verbose: print '\t%s:%s:%s' % ( event.commandSymbol, thatGraphname, thatLocator, )
                    _resolveCommand(
                        grove,
                        thisGraph,
                        thisNode,
                        thatGraphname,
                        thatLocator,
                        event,
                        verbose,
                        )


############################################################################################################
#
#   SAMPLE SOURCES
#
#===========================================================================================================
_exampleSource1 = """

:bion:
    bion
        collection
            collection-type
                botanical-garden
                    <= /bion/collection/organism-group/plants
                    <= /bion/collection/state/living
                herbarium
                    <= /bion/collection/organism-group/plants
                    # <= /bion/collection/state/conserved
            organism-group
                plants
                animals
            state
                living
                conserved
                    => /bion/collection/collection-type/herbarium
        foo
            botanical-garden
                <=> /bion/collection/collection-type/botanical-garden
                => :unknown-graph:/some/path

#   try to have more than one graph in one source
#           :next:
#               foo

#   try to have several sources (generalized roots) in one graph
#       :sources:
#           # this is source0
#           root
#               foo
#           source1x
#               bar
#           source2
#               baz
"""

_exampleSource2 = """
:othergraph:
    world
        collections
            state
                dried
                    => :bion:/bion/collection/collection-type/herbarium
                live
                    => :bion:/bion/collection/collection-type/botanical-garden
"""

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit(50)
    sep0  = '#' * 108
    sep1  = '=' * 108
    sep2  = '-' * 108

    grove = Grove()
    graph1 = parse( _exampleSource1, verbose = True )
    graph2 = parse( _exampleSource2, verbose = True )
    grove.add( graph1 )
    grove.add( graph2 )
    resolveCommands( grove, verbose = True )

    print sep0
    print grove
    for graph in grove:
        print sep1
        print graph
        #print 'Gauge: %s; root node: %s' % ( gauge, graph.rootNode.locator, )
        #print graph.locators
        for node in graph:
            print node #, node.events



