#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
import                                  re
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
ri                                      = u.resourceIdioms
_flowConventions                        = u.flowConventions
_ReprMx                                 = u.IOU.ReprMx
#-----------------------------------------------------------------------------------------------------------
EMPTYSTRING                             = ''


#===========================================================================================================
class FlowError( Exception ):

    #-------------------------------------------------------------------------------------------------------
    _lineFs = "On line #%(lineIndex)d: \n%(line)r:\n"

    #-------------------------------------------------------------------------------------------------------
    def _errorMsgWithLine( this, __msg__, **Q ):
        return this( ( '%s%s' % ( FlowError._lineFs, __msg__, ) ) % Q )
    _errorMsgWithLine = classmethod( _errorMsgWithLine )

#===========================================================================================================
class ValueError( FlowError, ValueError ):

    #-------------------------------------------------------------------------------------------------------
    def unknownGraphname( this, **Q ):
        __msg__ = "No graph called %(graphname)r found in grove %(grove)s."
        return this._errorMsgWithLine( __msg__, **Q )
    unknownGraphname = classmethod( unknownGraphname )

#===========================================================================================================
class SyntaxError( ValueError ):

    #-------------------------------------------------------------------------------------------------------
    def commandArgumentMalformed( this, **Q ):
        __msg__ = "Text after command symbol %(commandSymbol)s must match pattern %(argumentPattern)r."
        return this._errorMsgWithLine( __msg__, **Q )
    commandArgumentMalformed = classmethod( commandArgumentMalformed )


#===========================================================================================================
class IndentationError( ValueError ):

    #-------------------------------------------------------------------------------------------------------
    def didNotKeepMargin( this, **Q ):
        __msg__ = "Line did not keep margin (expected %(marginLength)d characters of whitespace)."
        return this._errorMsgWithLine( __msg__, **Q )
    didNotKeepMargin = classmethod( didNotKeepMargin )

    #-------------------------------------------------------------------------------------------------------
    def wrongIndentationCharacters( this, **Q ):
        __msg__ = "Line has wrong indentation characters -- must use %(dent)r."
        return this._errorMsgWithLine( __msg__, **Q )
    wrongIndentationCharacters = classmethod( wrongIndentationCharacters )

    #-------------------------------------------------------------------------------------------------------
    def unexpectedIndentation( this, **Q ):
        __msg__ = "Line indents more than a single dent step."
        return this._errorMsgWithLine( __msg__, **Q )
    unexpectedIndentation = classmethod( unexpectedIndentation )

#   ############################################################################################################
#   #
#   #
#   #
#   #===========================================================================================================
#   class Node:
#
#       #=======================================================================================================
#       def __init__( self, name, parent = None, *antecedents ):
#           self.new()
#           if parent is not None:
#               self.parent = parent
#           self.antecedents.extend( antecedents )
#       #-------------------------------------------------------------------------------------------------------
#       def new( self ):
#           self.parent         = None
#           self.antecedents    = []

############################################################################################################
#
#
#
#===========================================================================================================
class Operator:

    #=======================================================================================================
    def __init__( self, name, pattern ):
        self.new()
        self.name = name
        self.pattern = pattern
    #-------------------------------------------------------------------------------------------------------
    def new( self ):
        pass



############################################################################################################
#
#
#
#===========================================================================================================
class Event( _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    def __init__(
        self,
        lineIndex, line, level, message,
        commandSymbol = None, commandArgument = None,
        ):
        self.lineIndex          = lineIndex
        self.line               = line
        self.level              = level
        self.message            = message
        self.commandSymbol      = commandSymbol
        self.commandArgument    = commandArgument
        self.commandName        = None
        self.attributeName      = None
        if commandSymbol:
            commandInfo = Flow.commands[ commandSymbol ]
            self.commandName        = commandInfo[ 'commandName' ]
            self.attributeName      = commandInfo[ 'attributeName' ]

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        return '<%s %s %s>' % (
            #self.message,
            #self.attributeName,
            self.commandName,
            self.commandSymbol,
            self.commandArgument,
            )


############################################################################################################
#
#
#
#===========================================================================================================
class Flow:

    #-------------------------------------------------------------------------------------------------------
    commands                = _flowConventions.commands
    _commandsymbolsByLength = _flowConventions.commandsymbolsByLength
    #-------------------------------------------------------------------------------------------------------
    class messages:

        #---------------------------------------------------------------------------------------------------
        def __init__( self ):
            i = 1
            self.start        = i; i *= 2
            self.stop         = i; i *= 2

            self.empty        = i; i *= 2
            self.comment      = i; i *= 2
            self.command      = i; i *= 2
            self.plain        = i; i *= 2
            self.other        = i; i *= 2

            self.indent       = i; i *= 2
            self.nodent       = i; i *= 2
            self.undent       = i; i *= 2
            #self.line         = self.undent | self.plain | self.indent
        #---------------------------------------------------------------------------------------------------
        def marker( self, msg ):
            R = [ ' ' ] * 10
            if msg & self.start:                    R[0] =  '+'
            if msg & self.stop:                     R[1] =  '-'

            if msg & self.empty:                    R[2] =  '0'
            if msg & self.comment:                  R[3] =  '#'
            if msg & self.command:                  R[4] =  '$'
            if msg & self.plain:                    R[5] =  '~'
            if msg & self.other:                    R[6] =  '?'

            if msg & self.indent:                   R[7] =  '>'
            if msg & self.nodent:                   R[8] =  '|'
            if msg & self.undent:                   R[9] =  '<'

            #if msg & self.line == self.line:        R[ 9 ] =  '~'
            return ''.join( R )
    messages = messages()

    #-------------------------------------------------------------------------------------------------------
#       commands            = [
#           Operator( 'isImpliedBy',    '<='           ),
#           Operator( 'Implies',        '=>'           ),
#           Operator( 'isEquivalentTo', '<=>'          ),
#           Operator( 'isChildOf',      '<-'           ),
#           Operator( 'isParentOf',     '->'           ),
#           ]
    _reGetLeadingSpace  = re.compile( r'^(\s+)(.*)$' )
    _reIsCommentLine    = re.compile( r'^\s*#' )

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, source ):
        self.lines, self.margin, self.dent = self._getLinesBaselineAndDent( source )
        self.dentLength = len( self.dent )

#       #=======================================================================================================
#       def fromFilename( this, filename ):
#           return getFlow.fromFilename( filename, this )
#       fromFilename = classmethod( fromFilename )
#
#       #=======================================================================================================
#       def fromText( this, text ):
#           return this().parse( text )
#       fromText = classmethod( fromText )

    #=======================================================================================================
    def iterEvents( self, verbose = False ):
        lineIndex           = -1
        #dent               = ''
        lastLeadingWspc     = self.dent
        level               = 0
        lastLevel           = level
        strippedLine        = None
        #---------------------------------------------------------------------------------------------------
        yield Event( lineIndex + 1, None, level, self.messages.start )                              #+##
        #---------------------------------------------------------------------------------------------------
        for line in self.lines:
            lineIndex += 1
            if not line:
                yield Event( lineIndex, line, level, self.messages.empty )                          #+##
                continue
            #-----------------------------------------------------------------------------------------------
            if self._isCommentLine( line ):
                yield Event( lineIndex, line.strip(), level, self.messages.comment )                #+##
                continue
            #-----------------------------------------------------------------------------------------------
            leadingWspc = self._getLeadingWhitespace( line )
            if leadingWspc is None: leadingWspc = EMPTYSTRING
            if leadingWspc == lastLeadingWspc:
                msg = self.messages.nodent
            else:
                try:
                    level = self._getIndentationLevel( line )
                except ValueError:
                    dent = self.dent
                    raise IndentationError.wrongIndentationCharacters( **vars() )
                if level - lastLevel > 1:
                    if verbose: print lineIndex; print vars()
                    raise IndentationError.unexpectedIndentation( **vars() )
                if level < lastLevel:
                    msg = self.messages.undent
                else:
                    msg = self.messages.indent
                lastLevel = level
                lastLeadingWspc  = leadingWspc
                #print '#################%s' % line ###DEBUG###
            #-----------------------------------------------------------------------------------------------
            line = line.strip()
            commandSymbol, commandArgument = self._getCommandSymbolAndArgument( line )
            if commandSymbol:
                msg         |= self.messages.command
            else:
                msg         |= self.messages.plain
            yield Event( lineIndex, line, level, msg, commandSymbol, commandArgument )              #+##
        #---------------------------------------------------------------------------------------------------
        yield Event( lineIndex, line, 0, self.messages.stop )                                       #+##

    #=======================================================================================================
    def _isCommentLine( self, line ):
        return self._reIsCommentLine.search( line ) is not None

    #=======================================================================================================
    def _getCommandSymbolAndArgument( self, line ):
        # line must be strip()ped of whitespace
        for commandSymbol in self._commandsymbolsByLength:
            if line.startswith( commandSymbol ):
                commandArgument    = line[ len( commandSymbol ) : ].strip()
                commandArgumentRe  = self.commands[ commandSymbol ][ 'argumentRe' ]
                if commandArgumentRe:
                    commandArgumentPattern = commandArgumentRe.pattern
                    mo                     = commandArgumentRe.search( commandArgument )
                    if mo is None:
                        raise SyntaxError.commandArgumentMalformed( **vars() )
                    commandArgument = mo.group( 0 )
                return commandSymbol, commandArgument
        return None, None

    #=======================================================================================================
    def _getIndentationLevel( self, text, R = 0 ):
        if text.startswith( self.dent ):
            dummy, text = text[ : self.dentLength ], text[ self.dentLength : ]
            return self._getIndentationLevel( text, R ) + 1
        else:
            if self._getLeadingWhitespace( text ):
                raise ValueError()
        return R

    #=======================================================================================================
    def _getLeadingWhitespace( self, text ):
        mo = self._reGetLeadingSpace.search( text )
        if mo is None:
            return None
        return mo.group( 1 )

    #=======================================================================================================
    def _getLines( self, source ):
        return [ line.rstrip() for line in source.splitlines() ]

    #=======================================================================================================
    def _getLinesBaselineAndDent( self, source ):
        lines           = self._getLines( source )
        dent            = EMPTYSTRING
        margin          = EMPTYSTRING
        marginLength    = 0
        lookForMargin   = True
        lookForDent     = False
        for lineIndex, line in enumerate( lines ):
            #print '***', lineIndex, `line` ###DEBUG###
            if not line:        continue
#               mo                  = self._reGetLeadingSpace.search( line )
#               if mo is None:      continue
#               leadingWhitespace   = mo.group( 1 )
            leadingWhitespace = self._getLeadingWhitespace( line )
            if leadingWhitespace is None: leadingWhitespace = ''
            if lookForMargin:
                margin            = leadingWhitespace
                marginLength      = len( margin )
                lookForMargin     = False
                lookForDent       = True
            throway, line       = line[ : marginLength ], line[ marginLength : ]
            lines[ lineIndex ]  = line
            throwayStripped = throway.strip()
            if throwayStripped:# and not self._isCommentLine( line ):
                line    = '%(throway)s%(line)s' % vars()
                raise IndentationError.didNotKeepMargin( **vars() )
            if lookForDent:
#                   mo = self._reGetLeadingSpace.search( line )
#                   if mo is None: continue
                leadingWhitespace = self._getLeadingWhitespace( line )
                if leadingWhitespace is None: continue
                dent = leadingWhitespace
                lookForDent = False
        return lines, margin, dent

#   ############################################################################################################
#   #
#   #
#   #
#   #===========================================================================================================
#   class Message( int ):
#       def __init__( self, name, i ):
#           super( Message, self ).__init__( i )
#           self.name = name
#
#       def __new__( this, name, i ):
#           return int.__new__( this, i )
#
#       def __unicode__( self ):
#           return '<Message %d %s>' % ( int( self ), self.name, )
#       __str__ = __repr__ = __unicode__
#
#   #-----------------------------------------------------------------------------------------------------------
#   _x_count = 1
#   for _x_eventname in Flow.eventnames:
#       setattr( Flow, _x_eventname, Message( _x_eventname, _x_count ) )
#       _x_count *= 2

############################################################################################################
#
#
#
#===========================================================================================================
class OntologonFlow( Flow ):
    pass


#   ############################################################################################################
#   #
#   #
#   #
#   #===========================================================================================================
#   class getFlow:
#
#       targetClassDefault = Flow
#
#       #=======================================================================================================
#       def fromFilename( this, filename, targetClass = None ):
#           return this.fromText( ri.getText.fromFilename( filename ), targetClass )
#       fromFilename = classmethod( fromFilename )
#
#       #=======================================================================================================
#       def fromText( this, text, targetClass = None ):
#           if targetClass is None:
#               targetClass = this.targetClassDefault
#           R = targetClass().fromText( text )
#       fromText = classmethod( fromText )



############################################################################################################
#
#
#
#===========================================================================================================
class _cfg:
    inputFilename = 'bion.flow'

def _test00():
    pass

def _main():
    import os.path as osp
    fsi = g.filesystemIdioms
    here = fsi.mydir( lambda:None )
    inputFilepath = osp.join( here, cfg.inputFilename )
    test00()
    #test01()

#-----------------------------------------------------------------------------------------------------------
for name in dir():
    if name.startswith( '_x_' ):
        exec 'del %s' % name
del name

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    #_main()

    source = """
        topic
            collection
                botanicalGarden
                    <= /topic/science/biology/botany
                    <= /topic/unit/state/physicalObject/live
                herbarium

        second-root
    """

    flow = Flow( source )
    print 'margin: %d characters (%r).' % ( len( flow.margin ), flow.margin, )
    print '    dent: %d characters (%r).' % ( len( flow.dent     ), flow.dent,     )
    #   for line in flow.lines:
    #       print `line`

    R = []
    levelstack = [ R ]
    lastLevel = 0
    cm = Flow.messages
    for event in flow.iterEvents():
        msg, lineIndex, level = event.message, event.lineIndex, event.level
        line = flow.lines[ lineIndex ]
        print '%10s %3d %2d %r' % ( cm.marker( msg ), lineIndex, level, line )
        #---------------------------------------------------------------------------------------------------
        if msg & cm.start:
            continue #print 'start'
        #---------------------------------------------------------------------------------------------------
        if msg & cm.stop:
            continue #print 'stop'
        #---------------------------------------------------------------------------------------------------
        if msg & cm.empty:
            continue #print 'empty'
        #---------------------------------------------------------------------------------------------------
        if msg & cm.comment:
            continue #print 'comment'
        #---------------------------------------------------------------------------------------------------
        if msg & cm.command:
            continue #print 'command'
        #---------------------------------------------------------------------------------------------------
        if msg & cm.indent:
            S = []
            R.append( S )
            levelstack.append( S )
        #---------------------------------------------------------------------------------------------------
        if msg & cm.plain:
            levelstack[ level ].append( line.strip() ) #print 'plain'
        #---------------------------------------------------------------------------------------------------
        if msg & cm.undent:
            for i in range( lastLevel - level ):
                levelstack.pop()
            lastLevel = level
        #---------------------------------------------------------------------------------------------------
        if msg & cm.plain:
            continue
        #---------------------------------------------------------------------------------------------------
        if msg & cm.other:
            print 'other'
    print R

#       for event in Flow().iterEvents(
#           print event




