#
# -*- coding: utf-8 -*-

#-----------------------------------------------------------------------------------------------------------
#from    configtool.utilities.htmlMaker.htmlMaker                        import      h
from    htmlMaker                                                       import      h
#-----------------------------------------------------------------------------------------------------------

############################################################################################################
#
#
#
#===========================================================================================================
class MultiElementABC( object ):

    #=======================================================================================================
    def __init__( self, **Q ):
        self.new()
        self.Q.update( Q )

    #=======================================================================================================
    def new( self ):
        self.Q          = {}

    #=======================================================================================================
    def getQsForPrefix( self, prefix, **Q ):
        """Return a dictionary that results from the named values present in the method call, updated by
        those values as are found in ``~.Q`` to associated with names that start exactly with ``prefix``.
        Resulting names and names for values in the method call are not prefixed. Ex.::

            >>>x.Q = dict( x_foo = 42, y_foo = 84 )
            >>>x.getQsForPrefix( 'x_', foo = 100, bar = 200 )
            {'foo': 42, 'bar': 200}

        This routine allows us to store prefixed, named values in the instance that are assoicated with the
        HTML tags when called for. OBS. that after disassociation of the labels from the instance (i.e.
        after calling e.g. ``~.label`` or ``~.input``), changes to attributes of the present class will not
        be reflected in the separate tags.
        """
        prefixLength = len( prefix )
        for name, value in self.Q.items():
            if name.startswith( prefix ):
                Q[ name[ prefixLength : ] ] = value
        return Q

    #=======================================================================================================
    def asList( self ):
        return [] # self.label, self.input ]

    #=======================================================================================================
    def repr( self ):
        return ''.join( map( unicode, self.asList() ) )

    #=======================================================================================================
    def __repr__( self ):
        return self.repr()

############################################################################################################
#
#
#
#===========================================================================================================
class LabelledElementABC( MultiElementABC ):

    #=======================================================================================================
    def __init__( self, lbl_text = None, **Q ):
        MultiElementABC.__init__( self, **Q )
        if lbl_text is not None: self.lbl_text = lbl_text

    #=======================================================================================================
    def new( self ):
        MultiElementABC.new( self )
        self.lbl_text   = None

    #=======================================================================================================
    def label():
        def fget( self ):
            Q = self.getQsForPrefix( 'lbl_', klass = 'input_label' )
            return h.div( self.lbl_text, **Q )
        return locals()
    label = property( **label() )

############################################################################################################
#
#
#
#===========================================================================================================
class LabelledInput( LabelledElementABC ):

    #=======================================================================================================
    def __init__( self, lbl_text = None, inp_name = None, inp_text = None, **Q ):
        LabelledElementABC.__init__( self, lbl_text = lbl_text, **Q )
        if inp_name is not None: self.inp_name = inp_name
        if inp_text is not None: self.inp_text = inp_text

    #=======================================================================================================
    def new( self ):
        LabelledElementABC.new( self )
        self.inp_text   = ''
        self.inp_name       = '***UNNAMED***'

    #=======================================================================================================
    def input():
        def fget( self ):
            Q = dict( name = self.inp_name, type = 'text', klass = 'input_input' )
            if self.inp_text:
                Q[ 'value' ] = self.inp_text
            Q = self.getQsForPrefix( 'inp_', **Q )
            return h.input( **Q )
        return locals()
    input = property( **input() )

    #=======================================================================================================
    def asList( self ):
        return [ self.label, self.input ]

############################################################################################################
#
#
#
#===========================================================================================================
class Option:

    #=======================================================================================================
    def __init__( self, text, value, isSelected = None ):
        self.new()
        self.text           = text
        self.value          = value
        if isSelected is not None: self.isSelected = isSelected

    #=======================================================================================================
    def new( self ):
        self.text           = ''
        self.value          = None
        self.isSelected     = False

    #=======================================================================================================
    def getP( self ):
        return [ self.text ]

    #=======================================================================================================
    def getQ( self ):
        R = {}
        if self.value:          R[ 'value' ]    = self.value
        if self.isSelected:     R[ 'selected' ] = 'selected'
        return R

############################################################################################################
#
#
#
#===========================================================================================================
class LabelledDropdown( LabelledElementABC ):

    #=======================================================================================================
    def __init__( self, lbl_text = None, inp_name = None, inp_options = None, **Q ):
        LabelledElementABC.__init__( self, lbl_text = lbl_text, **Q )
        if inp_name is not None: self.inp_name = inp_name
        if inp_options is not None:
            self.inp_options.extend( inp_options )

    #=======================================================================================================
    def new( self ):
        LabelledElementABC.new( self )
        self.inp_text       = ''
        self.inp_name       = '***UNNAMED***'
        self.inp_options    = []

    #=======================================================================================================
    def input():
        def fget( self ):
            Qinp = dict( name = self.inp_name, type = 'text', klass = 'input_selector' )
            if self.inp_text:
                Qinp[ 'value' ] = self.inp_text
            Qinp = self.getQsForPrefix( 'inp_', **Qinp )
            #Qopt = self.getQsForPrefix( 'opt_' )
            R = h.select( **Qinp )
            for option in self.inp_options:
                R.append( h.option( *option.getP(), **option.getQ() ) )
            return R
        return locals()
    input = property( **input() )

