#
# -*- coding: utf-8 -*-

"""




::
    +----------------------------+
    |           DictHyper            |
    |   Functionality Provider   |
    +----------------------------+
          |             ^
          |             |
    hypo(DictHyper())  hyper(Medium())
          |             |
          v             |
    +----------------------------+
    |           Medium           |
    |  Namespace-like Accessor   |
    +----------------------------+
          |
          |
    hypo(Medium())
          |
          v
    +----------------------------+
    |           Hypo             |
    |      Data Provider         |
    +----------------------------+

    """

############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
#   from __future__ import division
#-----------------------------------------------------------------------------------------------------------
#   import os as                            _os
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_ReprMx                                 = u.IOU.ReprMx
#-----------------------------------------------------------------------------------------------------------
__hypername__                           = '__hyper__'
__hyponame__                            = '__hypo__'


############################################################################################################
#
#   HELPERS
#
#===========================================================================================================
def iterCaselessSort( words ):
    wordpairs = [ ( word.lower(), word ) for word in words ]
    wordpairs.sort()
    for wordpair in wordpairs:
        yield wordpair[ 1 ]


############################################################################################################
#
#
#
#===========================================================================================================
def hyper( obj ):
    return obj.__dict__[ obj.__hypername__ ]

#-----------------------------------------------------------------------------------------------------------
def hypo( obj ):
    return obj.__dict__[ obj.__hyponame__ ]

#-----------------------------------------------------------------------------------------------------------
def _setproperty( instance, name, value ):
    #print '&&&&&&&&&&&&&&&&', instance.__class__
    potentialproperty = getattr( instance.__class__, name, None )
    if potentialproperty is None: return False
    set = getattr( potentialproperty, '__set__', None )
    if set is None: return False
    set( instance, value )
    return True

############################################################################################################
#
#
#
#===========================================================================================================
class HyperHypoMx( _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    __hyponame__        = __hyponame__
    __hypername__       = __hypername__

    #-------------------------------------------------------------------------------------------------------
    def __pos__( self ):                    return hyper( self )
    def __neg__( self ):                    return hypo( self )


############################################################################################################
#
#
#
#===========================================================================================================
class HyperAbc( HyperHypoMx, object ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, medium ):
        self.__dict__[ self.__hyponame__ ] = medium

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        hypoU = unicode( hypo( self ) )
        return '<%s %s>' % ( self.__class__.__name__, hypoU, )

    #-------------------------------------------------------------------------------------------------------
    def f( self ):
        return hypo( self.medium )
    hypo = property( f )

    #-------------------------------------------------------------------------------------------------------
    def f( self ):
        return hypo( self )
    medium = property( f )

############################################################################################################
#
#
#
#===========================================================================================================
class DictHyper( HyperAbc ):

    #-------------------------------------------------------------------------------------------------------
    def dict( self ):
        R = {}
        R.update( self.hypo )
        return R

    #-------------------------------------------------------------------------------------------------------
    def update( self, Q ):
        self.hypo.update( Q )

    #-------------------------------------------------------------------------------------------------------
    def getNames( self, sorted = True ):
        R = self.hypo.keys()
        if not sorted: return R
        return list( iterCaselessSort( R ) )
        #return iterCaselessSort( self.hypo.keys() )
    names = property( getNames )

    #-------------------------------------------------------------------------------------------------------
    def items( self ):
        return [ ( name, self.medium[ name ] ) for name in self.names ]

    #-------------------------------------------------------------------------------------------------------
    def values( self ):
        return [ self.medium[ name ] for name in self.names ]

############################################################################################################
#
#
#
#===========================================================================================================
class Medium( HyperHypoMx, object ): # MySerializableMx, _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    #   Data provider class:
    __hypoclass__       = dict
    #-------------------------------------------------------------------------------------------------------
    #   Functionalities provider class:
    __hyperclass__      = DictHyper

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, *P, **Q ):
        self.__dict__[ self.__hypername__ ] = self.__hyperclass__( self )
        self.__dict__[ self.__hyponame__  ] = self.__hypoclass__( *P, **Q )

    #-------------------------------------------------------------------------------------------------------
    def __getattr__( self, name ):              return hypo( self )[ name ]

    #-------------------------------------------------------------------------------------------------------
    def __setattr__( self, name, value ):
        if not _setproperty( self, name, value ):
            hypo( self )[ name ] = value

    #-------------------------------------------------------------------------------------------------------
    def __getitem__( self, name ):              return getattr( self, name )
    def __setitem__( self, name, value ):       setattr( self, name, value )

    #-------------------------------------------------------------------------------------------------------
    #   REPRESENTATION
    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        names = hyper( self ).names
        return '<%s %s>' % ( self.__class__.__name__, ', '.join(
            [ '%s=%r' % ( name, self[ name ], )
                for name in names ]
            ) )

############################################################################################################
#
#
#
#===========================================================================================================
ItemAccessingMedium = Medium


############################################################################################################
#
#
#
#===========================================================================================================
class AttributeAccessingMedium( Medium ): # MySerializableMx, _ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    def __getattr__( self, name ):              return getattr( hypo( self ), name )

    #-------------------------------------------------------------------------------------------------------
    def __setattr__( self, name, value ):
        _setproperty( self, name, value ) or setattr( hypo( self ), name, value )


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit( 50 )
    sep0 = '#' * 108
    sep1 = '=' * 108
    sep2 = '-' * 108

    def show( x, y ):
        print '%40s: %s' % ( x, y, )

    print sep2
    show( 'Class used as Data Provider', Medium.__hypoclass__ )
    show( 'Class used as Functionality Provider', Medium.__hyperclass__ )

    print sep2
    data = dict( { '***': 'three stars'}, answer = 42, weight = '108kg' )
    show( 'Data used for input (a dictionary)', data )

    print sep2
    medium = Medium( **data )
    #print -medium
    show( 'Medium derived from data', medium )

    print sep2
    show( 'Underlying data (1)', hypo( medium ) )
    show( 'Underlying data (2)', -medium )
    show( 'Underlying data (3)', --+medium )
    show( 'Function provider (1)', hyper( medium ) )
    show( 'Function provider (2)', +medium )

    print sep2
    medium.weight = '216lb'
    show( 'Data manipulated with attribute access', medium )
    medium[ '***' ] = 'three **bright** stars'
    show( 'Data manipulated with item access', medium )

    print sep2
    show( 'Names in namespace (1)', hyper( medium ).names )
    show( 'Names in namespace (2)', ( +medium ).names )

    print dir( medium )

    print sep2
    print 'Defining properties in the medium class should work for both item and attribute access:'
    print medium.answer
    def getAnswer( self ): return '-%s-' % ( (-self)[ 'answer' ] )
    def setAnswer( self, value ): raise'*';(-self)[ 'answer' ] = '+%s+' % ( answer, )

    Medium.answer = property( getAnswer, setAnswer )
    print medium.answer
    print medium[ 'answer' ]
    medium.answer = 'x'
    print medium.answer
    print medium[ 'answer' ]

