#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
import os as                            _o
import sys
import time
import os.path as                       _p
import operator as                      _operator
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_S                                      = u.EzSet.EzSet
_fsi                                    = u.filesystemIdioms
_indexhomename                          = 'lupyindex'
#-----------------------------------------------------------------------------------------------------------
_AbcdPathTokenizer                      = u.nlnormalizer.AbcdTokenizers.AbcdPathTokenizer
_AbcdTextTokenizer                      = u.nlnormalizer.AbcdTokenizers.AbcdTextTokenizer
#-----------------------------------------------------------------------------------------------------------
_indexhomelocator                       = _fsi.join( g.cfg.persistenceLocator, _indexhomename )
#-----------------------------------------------------------------------------------------------------------
_dsGrove                                = None
_groveFed                               = None
_lupy                                   = u.lupywrapper.lupy
_fsi.includeThisDirectoryInPath( u.lupywrapper )
#-----------------------------------------------------------------------------------------------------------
_Index                                  = _lupy.indexer.Index
_IndexSearcher                          = _lupy.search.indexsearcher.IndexSearcher
_TermQuery                              = _lupy.search.term.TermQuery
_PhraseQuery                            = _lupy.search.phrase.PhraseQuery
_BooleanQuery                           = _lupy.search.boolean.BooleanQuery
_Term                                   = _lupy.index.term.Term
#-----------------------------------------------------------------------------------------------------------
VERBOSE                                 = False
NORMALIZEQUERIES                        = False
#-----------------------------------------------------------------------------------------------------------
VERBOSE                                 = True
NORMALIZEQUERIES                        = True
#-----------------------------------------------------------------------------------------------------------
LUPYHARDCODEDMAXRESULTS                 = 1000
MAXRESULTS                              = LUPYHARDCODEDMAXRESULTS

############################################################################################################
#
#
#
#===========================================================================================================
def getTermQuery( phrase ):
    return _TermQuery( _Term( 'text', phrase ) )

#-----------------------------------------------------------------------------------------------------------
def getPhraseQuery( phrase ):
    terms = phrase.split()
    R = _PhraseQuery()
    for term in terms:
        R.add( _Term( 'text', term ) )
    return R

#-----------------------------------------------------------------------------------------------------------
def getBooleanQuery( ands = [], ors = [], nots = [] ):
    """Build a simple boolean query."""
    R = _BooleanQuery()
    for a in ands: R.add( _TermQuery( _Term( 'text', a ) ), required = True,  prohibited = False )
    for a in ors:  R.add( _TermQuery( _Term( 'text', a ) ), required = False, prohibited = False )
    for a in nots: R.add( _TermQuery( _Term( 'text', a ) ), required = False, prohibited = True  )
    return R

############################################################################################################
#
#
#
#===========================================================================================================
def getIndexlocatorForGraphname( graphname ):
    return _fsi.join( _indexhomelocator, graphname )

#-----------------------------------------------------------------------------------------------------------
def getSearcherForGraphname( graphname ):
    indexlocator = getIndexlocatorForGraphname( graphname )
    try:
        return _IndexSearcher( indexlocator )
    except IOError:
        raise KeyError( ''
            +   "No index for this graph: :%s:." % ( graphname, )
            )

#-----------------------------------------------------------------------------------------------------------
def getIndexForGraphname( graphname, create = False ):
    indexlocator = getIndexlocatorForGraphname( graphname )
    try:
        return _Index( indexlocator, create = create )
    except IOError:
        raise KeyError( ''
            +   "No index for this graph: :%s:." % ( graphname, )
            )


############################################################################################################
#
#
#
#===========================================================================================================
def getQueryForPhrase( phrase ):
    phrase = phrase.strip()
    if len( phrase ) > 2 and phrase[ 0 ] == '"' and phrase[ -1 ] == '"':
        return getPhraseQuery( phrase[ 1 : -1 ] )
    else:
        terms   = phrase.split()
        ands    = [ term[ 1 : ] for term in terms if term[ 0 ] == '+' and len( term ) > 1 ]
        nots    = [ term[ 1 : ] for term in terms if term[ 0 ] == '-' and len( term ) > 1 ]
        ors     = [ term for term in terms if term[ 0 ] not in ( '+', '-' ) ]
        return getBooleanQuery( ands, ors, nots )


############################################################################################################
#
#
#
#===========================================================================================================
class Vector( list ):
    """Ad-hoc vectorial class do perform normalization on collections of values."""

    #-------------------------------------------------------------------------------------------------------
    def descale( self ):
        if not self: return
        maximum = max( self )
        if maximum == 0: return
        self.divS( maximum )

    #-------------------------------------------------------------------------------------------------------
    #   PROPRIOCEPTIVE SCALAR METHODS
    #-------------------------------------------------------------------------------------------------------
    def _doBinaryScalarOperation( self, scalar, op ):
        for k in range( len( self ) ):
            self[ k ] = op( self[ k ], scalar )

    #-------------------------------------------------------------------------------------------------------
    def divS( self, scalar ):
        self._doBinaryScalarOperation( scalar, _operator.truediv )

############################################################################################################
#
#
#
#===========================================================================================================
def searchOnSingleGraphForScoresAndTermnrs( phrase, graphname ):
    """Return a triplet containing the normalized phrase, the effective graphname, and a list of
    ``(score,termnr)`` pairs. In case gauges were included with colons (à la ``:graphname:``) in the
    ``phrase``, the last one mentioned wins over the method argument and all graphnames mentioned earlier in
    the ``phrase``."""
    #-------------------------------------------------------------------------------------------------------
    terms       = []
    for term in phrase.split():
        if term.startswith( ':' ) and term.endswith( ':' ):
            graphname = term[ 1 : -1 ]
        else:
            terms.append( term )
    phrase      = ' '.join( terms )
    graphnames  = _S( graphname )
    normalphrase, graphnames, triplets = _search( phrase, graphnames )
    R = [ ( score, termnr, ) for score, graphname, termnr in triplets ]
    return normalphrase, graphname, R


############################################################################################################
#
#
#
#===========================================================================================================
def _getAllGraphnames():
    global _groveFed, _dsGrove
    _groveFed   = u.feds.resolve( 'Grove' )
    _dsGrove    = _groveFed.dsGrove
    return _S( *_dsGrove.iterGauges() )


############################################################################################################
#
#
#
#===========================================================================================================
def search( phrase, *graphnames ):
    """Return a triplet containing the normalized phrase, the set of graphnames searched, and a list of
    ``(score,graphname,termnr)`` triplets."""
    #-------------------------------------------------------------------------------------------------------
    graphnames  = _S( *graphnames )
    terms       = []
    maxresults  = MAXRESULTS
    for term in phrase.split():
        if term.startswith( ':' ) and term.endswith( ':' ):
            term = term[ 1 : -1 ]
            if term.isdigit():
                try:
                    maxresults = int( term )
                except:
                    pass
            else:
                graphnames.add( term )
        else:
            terms.append( term )
    phrase = ' '.join( terms )
    if not graphnames:
        global _groveFed, _dsGrove
        _groveFed   = u.feds.resolve( 'Grove' )
        _dsGrove    = _groveFed.dsGrove
        graphnames  = _getAllGraphnames()
    #   #   Uncomment these lines to enable searching by namespace URLs:
    #   else:
    #       global _groveFed, _dsGrove
    #       _groveFed   = u.feds.resolve( 'Grove' )
    #       _dsGrove    = _groveFed.dsGrove
    #       graphnames  = _S( *[ _dsGrove.getGraphnameForGraphnameOrNamespace( n ) for n in graphnames ] )
    return _search( phrase, graphnames, maxresults )

############################################################################################################
#
#
#
#===========================================================================================================
def _search( phrase, graphnames, maxresults = MAXRESULTS ):
    #-------------------------------------------------------------------------------------------------------
    R               = []
    textTokenizer   = _AbcdTextTokenizer( allowSigns = True )
    normalphrase    = ' '.join( textTokenizer.tokenize( phrase ) )
    #-------------------------------------------------------------------------------------------------------
    for graphname in graphnames:
        try:
            searcher        = getSearcherForGraphname( graphname )
        except KeyError:
            continue
        query           = getQueryForPhrase( normalphrase )
        hits            = searcher.search( query )
        hitcount        = min( len( hits ), maxresults, LUPYHARDCODEDMAXRESULTS )
        if not hitcount: continue
        scores          = [ hits.score( i ) for i in range( hitcount ) ]
        #---------------------------------------------------------------------------------------------------
        if NORMALIZEQUERIES:
            scores = Vector( scores )
            scores.descale()
        #---------------------------------------------------------------------------------------------------
        for i, hit in enumerate( hits ):
            if i >= hitcount: break
            score           = scores[ i ]
            termnr          = int( hit.get( 'title' ) )
            R.append( ( score, graphname, termnr ) )
    #-------------------------------------------------------------------------------------------------------
    maxhitcount = min( maxresults, LUPYHARDCODEDMAXRESULTS )
    if len( R ) > maxhitcount:
        del R[ maxhitcount : ]
    R.sort()
    R.reverse()
    return normalphrase, graphnames, R

############################################################################################################
#
#
#
#===========================================================================================================
def nodefactsForSearch( phrase, *graphnames ):
    """Return a triplet containing the normalized phrase, the set of graphnames searched, and a list of
    ``(score,nodefact)`` pairs (node facts retrieved from the ``Grove`` datasource)."""
    #-------------------------------------------------------------------------------------------------------
    global _groveFed, _dsGrove
    _groveFed   = u.feds.resolve( 'Grove' )
    _dsGrove    = _groveFed.dsGrove
    normalphrase, graphnames, triplets = search( phrase, *graphnames )
    R = [
        ( score, _dsGrove.nodefactForTermref( ( graphname, termnr ) ), )
            for score, graphname, termnr in triplets ]
    return normalphrase, graphnames, R

############################################################################################################
#
#
#
#===========================================================================================================
def show( phrase, *graphnames ):
    print
    print '#' * 108
    print "Searching corpus of %d graphs for '%s'" % ( len( graphnames ), phrase, )
    normalphrase, graphnames, resultpairs = nodefactsForSearch( phrase, *graphnames )
    graphnames = list( graphnames )
    graphnames.sort()
    graphnames = ' '.join( graphnames )
    print "Normalized query: '%s'" % ( normalphrase, )
    print "Graphnames: %s" % ( graphnames, )
    print "Search returned %d results (MAXRESULTS = %d)." % ( len( resultpairs ), MAXRESULTS, )
    i = 0
    for score, nodefact in resultpairs:
        i += 1
        locator = nodefact.locator
        ontonum = ':%s:%d' % ( nodefact.gauge, nodefact.termnr, )
        print '\t%2d  %6.4f  %-20s: %s' % ( i, score, ontonum, locator )


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    phrases = """

#           different
#           same
#           picture
#           gif
#           foo
#           digital
        image :abcd149:
#           digital image
        +digital +image
        +bion +media
        +bion +media collection
        gathering
        gathering :bion:
#           gathering name
#           gathering +name +xxx
#           name
#           xxx


    """.splitlines()
    phrases = [ phrase.strip() for phrase in phrases ]
    phrases = [ phrase for phrase in phrases if not phrase.startswith( '#' ) ]
    phrases = [ phrase for phrase in phrases if phrase ]

    maxresults = 25
    #graphnames = list( _dsGrove.iterGauges() )
    #graphnames = 'abcd12 bion darwin2'.split()
    for phrase in phrases:
        show( phrase )
        print '-' * 108
        normphrase, pairs = searchOnSingleGraphForScoresAndTermnrs( phrase, 'abcd12' )
        for score, termnr in pairs:
            print '%8.5f %s' % ( score, termnr, )


