#
# -*- coding: utf-8 -*-


"""

Multilateral facets are named values that live several incarnations at (more or less) the same time: they
are formulated in Python, can export themselves to an HTTP querystring and be interpolated into a JavaScript
source text.

"""


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
import                                  cgi
from sets import                        Set
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
IOU                                     = u.IOU
Misfit                                  = u.Misfit
acquire                                 = u.pylon.acquire
#-----------------------------------------------------------------------------------------------------------
U                                       = IOU.U
ReprMx                                  = IOU.ReprMx
jsUC                                    = IOU.jsUC
iterableJsU                             = IOU.iterableJsU

#===========================================================================================================
def __notimplemented__( self, *P, **Q ):
    raise NotImplemented()


#   ############################################################################################################
#   #
#   #
#   #
#   #===========================================================================================================
#   class Ianus( ReprMx, dict ):
#   class Ianus( dict ):



############################################################################################################
#
#
#
#===========================================================================================================
class ItemAsAttribute( ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, owner ):
        self.__dict__[ '__owner__' ] = owner

    #-------------------------------------------------------------------------------------------------------
    def __getattr__( self, name ):
        return self.__owner__[ name ]

    #-------------------------------------------------------------------------------------------------------
    def __setattr__( self, name, value ):
        self.__owner__[ name ] = value

    #-------------------------------------------------------------------------------------------------------
    def __delattr__( self, name ):
        del self.__owner__[ name ]


############################################################################################################
#
#
#
#===========================================================================================================
class Vocabulary( ReprMx, dict ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, owner, *facets ):
        self.owner = owner
        self.i = self.itemAsAttribute = ItemAsAttribute( owner = self )

    #-------------------------------------------------------------------------------------------------------
    def _invert( self ):
        R = {}
        for key, value in self.iteritems():
            R[ value ] = key
        return R

    #-------------------------------------------------------------------------------------------------------
    def nameForValue( self, value = Misfit ):
        if value is Misfit:
            value = self.owner.value
        return self._invert().get( value, Misfit )

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        return unicode( dict( self ) )

############################################################################################################
#
#
#
#===========================================================================================================
class MultilateralFacetContainer( ReprMx, dict ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, *facets ):
        self.itemAsAttribute        = ItemAsAttribute( owner = self )
        self.i                      = self.itemAsAttribute
        self.originalCgiQuerydict   = None
        self.originalQuerydict      = None
        self.auxvars                = None
        self.add( *facets )

    #-------------------------------------------------------------------------------------------------------
    def add( self, *facets ):
        for facet in facets:
            if not isinstance( facet, MultilateralFacet ):
                raise TypeError( "Only facets of class MultilateralFacet allowed." )
            self[ facet.name ] = facet

    #-------------------------------------------------------------------------------------------------------
    def addDescriptions( self, descriptions ):
        for description in descriptions:
            self.add( MultilateralFacet( **description ) )

    #-------------------------------------------------------------------------------------------------------
    def addNewDescriptions( self, *descriptions ):
        for description in descriptions:
            facet = MultilateralFacet( **description )
            facet.isNew = True
            self.add( facet )

    #-------------------------------------------------------------------------------------------------------
    def addNewSimpleFacets( self, *names ):
        descriptions = [
            dict( name = name, default = '', isNew = True )
                for name in names
            ]
        self.addNewDescriptions( *descriptions )

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get JavaScript representation of facets."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            R = [ facet.javascript for facet in self.iterJavascriptFacets() ]
            if self.auxvars is not None:
                auxvars = self.auxvars
                getJavascript = MultilateralFacet.getJavascript
                _vectortypes = MultilateralFacet._vectortypes
                for name in dir( auxvars ):
                    if not name.startswith( '_' ):
                        value = getattr( auxvars, name )
                        isVector = isinstance( value, _vectortypes )
                        R.append(
                            getJavascript(
                                name        = name,
                                value       = value,
                                isVector    = isVector,
                                ) )
            return '\n'.join( R )
        #---------------------------------------------------------------------------------------------------
        return locals()
    javascript = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get or set query string representation of facets. Setting a query string means setting the
        values of defined facets (identified by equal names in the dictionary and the query string) and
        implies the skipping of all other values."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return self.getQuerystring()
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            self.updateFromQuerystring( value )
        #---------------------------------------------------------------------------------------------------
        return locals()
    querystring = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def iterQuerystringFacets( self, **Q ):
        """Iterate over all facets to be included in querystring. If named values are given, names should
        match facet names in container and values should be valid replacements for the actual facet
        values."""
        for facet in self.iterfacets( __filter__ = lambda facet: facet.inQuerystring, **Q ):
            yield facet

    #-------------------------------------------------------------------------------------------------------
    def iterJavascriptFacets( self, **Q ):
        """Iterate over all facets to be included in JavaScript source. If named values are given, names
        should match facet names in container and values should be valid replacements for the actual facet
        values."""
        for facet in self.iterfacets( __filter__ = lambda facet: facet.inJavascript, **Q ):
            yield facet

    #-------------------------------------------------------------------------------------------------------
    def iterfacets( self, __filter__ = None, **Q ):
        """Iterate over all facets for which ``__filter__(facet)`` returns true, or all facets if
        ``__filter__`` is ``None``. If named values are given, names should match facet names in container
        and values should be valid replacements for the actual facet values."""
        if not Q:
            I = self.itervalues( __filter__ )
        else:
            I = {}
            for facet in self.itervalues( __filter__ ):
                name = facet.name
                value = Q.get( name, Misfit )
                if value is not Misfit:
                    facet = facet.copy( value = value )
                I[ name ] = facet
            I = I.itervalues()
        for facet in I:
            yield facet

    #-------------------------------------------------------------------------------------------------------
    def iterkeys( self, filter = None ):
        for key in super( MultilateralFacetContainer, self ).iterkeys():
            if filter is None or filter( key ):
                yield key

    #-------------------------------------------------------------------------------------------------------
    def itervalues( self, filter = None ):
        for value in super( MultilateralFacetContainer, self ).itervalues():
            #print '+++++++++++', value
            #if filter is not None: print '+++++++++++', filter( value )
            if filter is None or filter( value ):
                yield value

    #-------------------------------------------------------------------------------------------------------
    def iteritems( self, filter = None ):
        for key, value in super( MultilateralFacetContainer, self ).iteritems():
            if filter is None or filter( key, value ):
                yield key, value

    #-------------------------------------------------------------------------------------------------------
    def getQuerystring( self, **Q ):
        """Same as ``~.querystring``, but with an option to specify modified facets. Ex.::

            fc.getQuerystring( action = 'help', size = 42 )

        may return::

            action=help&size=42&foo=bar

        even if facets ``action`` and ``size`` had different values."""
        return '&'.join( [ facet.querystring for facet in self.iterQuerystringFacets( **Q ) ] )

    #-------------------------------------------------------------------------------------------------------
    def getUrl( self, url, **Q ):
        return '%s?%s' % ( url, self.getQuerystring( **Q ) )

    #-------------------------------------------------------------------------------------------------------
    def updateFromQuerystring( self, querystring ):
        querydict = IOU.querydict( querystring )
        self.updateFromQuerydict( querydict )

    #-------------------------------------------------------------------------------------------------------
    def updateFromQuerydict( self, querydict ):
        for name, rawvalues in querydict.iteritems():
            facet = self.get( name, None )
            if facet is None: continue
            facet.setValueFromRawvalues( rawvalues )

    #-------------------------------------------------------------------------------------------------------
    def updateFromCgiEnvironment( self ):
        if self.originalCgiQuerydict is None:
            self.originalCgiQuerydict = cgi.parse( keep_blank_values = True, strict_parsing = False )
        querydict = IOU.querydictFromCgiQuerydict( self.originalCgiQuerydict )
        self.originalQuerydict = querydict
        self.updateFromQuerydict( querydict )

    #-------------------------------------------------------------------------------------------------------
    def updateNewFacets( self ):
        if self.originalQuerydict is None: return
        for name, rawvalues in self.originalQuerydict.iteritems():
            facet = self.get( name, None )
            if facet is None or not facet.isNew: continue
            facet.isNew = False
            facet.setValueFromRawvalues( rawvalues )

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        return '{%s}' % ( ' '.join( [ facet.__unicode__() for facet in self.itervalues() ] ) )
        #return unicode( dict( self ) )

#   class _X_Pickle( object ):
#
#       def __call__( self, x ):
#           print '++++++++++++++++', x
#           return g.pickler.loads( x )

############################################################################################################
#
#
#
#===========================================================================================================
class MultilateralFacet( ReprMx ):

    #-------------------------------------------------------------------------------------------------------
    _scalartypes = ( Misfit.__class__, int, long, float, basestring, ) # _X_Pickle )
    _vectortypes = ( Misfit.__class__, list, tuple, )

    #-------------------------------------------------------------------------------------------------------
    def __init__( self,
        name,
        value           = Misfit,
        default         = Misfit,
        controlled      = False,
        vocabulary      = Misfit,
        isVector        = False,
        scalartype      = Misfit,
        inQuerystring   = True,
        inJavascript    = True,
        holder          = Misfit,
        strict          = False,
        isNew           = False,
        ):
        self._changed       = True
        self._values        = []
        self._javascript    = Misfit
        self._querystring   = Misfit
        self._isVector      = Misfit
        self._default       = Misfit
        self._scalartype    = Misfit
        self._vocabulary    = Vocabulary( owner = self )
        #---------------------------------------------------------------------------------------------------
        self._isVector      = isVector
        self.inQuerystring  = inQuerystring
        self.inJavascript   = inJavascript
        self.holder         = holder
        self.scalartype     = scalartype
        self.name           = name
        self.strict         = strict
        self.isNew          = isNew
        self.controlled     = controlled
        self.vocabulary     = vocabulary
        self.default        = default
        self.value          = value
        self.i              = self.vocabulary.i

    #-------------------------------------------------------------------------------------------------------
    def copy( self, **Q ):
        Q = acquire(
            Q,
            name            = self.name,
            value           = self.value,
            default         = self.default,
            controlled      = self.controlled,
            vocabulary      = self.vocabulary,
            isVector        = self.isVector,
            scalartype      = self.scalartype,
            inQuerystring   = self.inQuerystring,
            inJavascript    = self.inJavascript,
            holder          = self.holder,
            strict          = self.strict,
            isNew           = self.isNew,
            )
        return self.__class__( **Q )

    ########################################################################################################
    #   PROPERTIES
    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get or set value of facet. Underlyingly, ``~._values`` is always a list that may or may not
        contain elements. In case ``~.isVector`` is true, ``~._values`` is always returned as-is for
        ``~.value``. In case ``~.isVector`` is false, then either the first element of the list is returned,
        or, if ``~._values`` should be empty, ``Misfit`` is returned instead to signal the non-existence of
        the value. If ``~.controlled`` is true, then only those values that also appear as values of
        ``~.vocabulary`` are taken over (in other words, all values are guranteed to be a match to a key in
        the vocabulary)."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            if self.holder is not None:
                return getattr( self.holder, self.name, self.default )
            return self._getValueFromInstance()
        #---------------------------------------------------------------------------------------------------
        def fset( self, values ):
            self._assertValueHasAcceptableType( values, 'value' )
            self._changed = True
            del self._values[:]
            if values is Misfit:
                if self.default is not Misfit:
                    self.value = self.default
                return
            if not self.isVector:
                values = [ values ]
            if self.controlled:
                allowedValues = Set( self.vocabulary.values() )
                #print '++++++++++++', values
                values = filter( lambda x: x in allowedValues, values )
                #print '++++++++++++', values
            if not values:
                if self.isVector:
                    values = self.default
                else:
                    values = [ self.default ]
            #print '~~~~~~~~~~~~', self._values, values
            if values is not Misfit:
                self._values.extend( values )
            if self.holder is not None:
                setattr( self.holder, self.name, self._getValueFromInstance() )
        #---------------------------------------------------------------------------------------------------
        def fdel( self ):
            self.value = Misfit
        #---------------------------------------------------------------------------------------------------
        return locals()
    value = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def _getValueFromInstance( self ):
        if self.isVector:
            return self._values
        if self._values:
            return self._values[ 0 ]
        return Misfit

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get JavaScript source representation of facet."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            if self._changed or self._javascript is Misfit:
                self._javascript = self.getJavascriptFromValue()
            return self._javascript
        #---------------------------------------------------------------------------------------------------
        return locals()
    javascript = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get HTTP query string respresentation of facet. This will be an 8bit string like
        ``x=42&x=108`` (with difficult characters duly escaped) if ``~.isVector`` is True or else a string
        like ``x=42`` if ``~.isVector`` is False."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            if self._changed or self._querystring is Misfit:
                self._querystring = self.getQuerystringFromValue()
            return self._querystring
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            self._changed = True
            self._values = self.getValuesFromQuerystring( value )
        #---------------------------------------------------------------------------------------------------
        return locals()
    querystring = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get ``isVector`` of facet."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return self._isVector
        #   #---------------------------------------------------------------------------------------------------
        #   def fset( self, value ):
        #       self._changed = True
        #       self._isVector = value
        #       self._assertValueHasAcceptableType( self.value, 'value' )
        #       self._assertValueHasAcceptableType( self.default, 'default' )
        #---------------------------------------------------------------------------------------------------
        return locals()
    isVector = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get, set or delete ``default`` of facet."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return self._default
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            self._changed = True
            self._assertValueHasAcceptableType( value, 'default' )
            self._assertAllValuesOkWithVocabulary( value )
            self._default = value
        #---------------------------------------------------------------------------------------------------
        def fdel( self ):
            self._changed = True
            self._default = Misfit
            pass
        #---------------------------------------------------------------------------------------------------
        return locals()
    default = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get, set or delete ``scalartype`` of facet."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return self._scalartype
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            self._changed = True
            if value is None:
                value = Misfit
            if ( value is not Misfit ) and ( not callable( value ) ):
                raise TypeError( "Value for scalartype must be callable or Misfit or None." )
            self._scalartype = value
        #---------------------------------------------------------------------------------------------------
        def fdel( self ):
            self._changed = True
            self._scalartype = Misfit
            pass
        #---------------------------------------------------------------------------------------------------
        return locals()
    scalartype = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get, set or delete ``vocabulary`` of facet."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return self._vocabulary
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            #self._vocabulary = Vocabulary( {} )
            if value is Misfit or value is None:
                del self.vocabulary
                return
            self._vocabulary.update( value )
        #---------------------------------------------------------------------------------------------------
        def fdel( self ):
            self._vocabulary.clear()
            pass
        #---------------------------------------------------------------------------------------------------
        return locals()
    vocabulary = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f():
        #---------------------------------------------------------------------------------------------------
        doc = """Get, set or delete ``holder`` of facet."""
        #---------------------------------------------------------------------------------------------------
        def fget( self ):
            return self._holder
        #---------------------------------------------------------------------------------------------------
        def fset( self, value ):
            self._changed = True
            if value is Misfit: value = None
            self._holder = value
        #---------------------------------------------------------------------------------------------------
        def fdel( self ):
            self.holder = Misfit
        #---------------------------------------------------------------------------------------------------
        return locals()
    holder = property( **f() ); del f

#       #-------------------------------------------------------------------------------------------------------
#       def f():
#           #---------------------------------------------------------------------------------------------------
#           doc = """Get, set or delete ``NAME`` of facet."""
#           #---------------------------------------------------------------------------------------------------
#           def fget( self ):
#               return self._NAME
#           #---------------------------------------------------------------------------------------------------
#           def fset( self, value ):
#               self._changed = True
#               self._NAME = value
#           #---------------------------------------------------------------------------------------------------
#           def fdel( self ):
#               self._changed = True
#               self._NAME = Misfit
#               pass
#           #---------------------------------------------------------------------------------------------------
#           return locals()
#       NAME = property( **f() ); del f


    ########################################################################################################
    #   METHODS
    #-------------------------------------------------------------------------------------------------------
    def getQuerystringFromValue( self, value = Misfit ):
        #---------------------------------------------------------------------------------------------------
        if value is Misfit: value = self.value
        if value is Misfit: return ''
        #---------------------------------------------------------------------------------------------------
        if not self.isVector:
            value = [ value ]
        #---------------------------------------------------------------------------------------------------
        Q = { self.name: value }
        R = IOU.querystring( Q, strict = self.strict )
        return R

    #-------------------------------------------------------------------------------------------------------
    def getValuesFromQuerystring( self, querystring = Misfit ):
        if querystring is Misfit: querystring = self.querystring
        rawvalues = IOU.querydict( querystring ).get( self.name, [] )
        return self.getValuesFromRawvalues( rawvalues )

    #-------------------------------------------------------------------------------------------------------
    def getValuesFromRawvalues( self, rawvalues ):
        #---------------------------------------------------------------------------------------------------
        assert isinstance( rawvalues, list )
        #   if not self.isVector:
        #       rawvalues = rawvalues[ : 1 ]
        #---------------------------------------------------------------------------------------------------
        if self.scalartype is Misfit:
            return rawvalues
        #---------------------------------------------------------------------------------------------------
        R = []
        for rawvalue in rawvalues:
            try:
                #print '###########', self.scalartype ###DEBUG###
                value = self.scalartype( rawvalue )
            except:
                if self.strict:
                    raise
            else:
                R.append( value )
        #---------------------------------------------------------------------------------------------------
        return R

    #-------------------------------------------------------------------------------------------------------
    def setValueFromRawvalues( self, rawvalues ):
        values = self.getValuesFromRawvalues( rawvalues )
        if not values:
            self.value = Misfit
            return
        if self.isVector:
            self.value = values
        else:
            for value in values:
                self.value = value
                if self.value == value:
                    break

    #-------------------------------------------------------------------------------------------------------
    def getJavascriptFromValue( self, value = Misfit ):
        R = []
        #---------------------------------------------------------------------------------------------------
        if value is Misfit: value = self.value
        if value is Misfit: return ''
        #---------------------------------------------------------------------------------------------------
        R.append( self.getJavascript( self.name, value, self.isVector ) )
        #---------------------------------------------------------------------------------------------------
        if self.vocabulary:
            fs = '%s_%%s' % ( self.name, )
            for name, value in self.vocabulary.iteritems():
                R.append( self.getJavascript( fs % name, value, self.isVector ) )
        #---------------------------------------------------------------------------------------------------
        return '\n'.join( R )

    #-------------------------------------------------------------------------------------------------------
    def f( this, name, value, isVector ):
        #---------------------------------------------------------------------------------------------------
        if isVector:
            value = iterableJsU( value )
            R = '%s = %s;' % ( name, value, )
        #---------------------------------------------------------------------------------------------------
        else:
            value = jsUC( value ) # .__repr__()
            R = '%s = %s;' % ( name, value, )
        #---------------------------------------------------------------------------------------------------
        return R
    getJavascript = classmethod( f ); del f

    ########################################################################################################
    #   TESTS AND ASSERTIONS
    #-------------------------------------------------------------------------------------------------------
    def _testValueHasAcceptableTypeGetMsg( self, w, attrname, ts, value ):
        ts = ', '.join( [ '%r' % ( t, ) for t in ts ] )
        t  = value.__class__.__name__
        return ( "Only %(w)s of these types allowed: %(ts)s; class of attribute ~.%(attrname)s is %(t)s."
            % vars() )

    #-------------------------------------------------------------------------------------------------------
    def _assertValueHasAcceptableType( self, value, attrname ):
        if value is Misfit: return
        #---------------------------------------------------------------------------------------------------
        if not self.isVector and not isinstance( value, self._scalartypes ):
            raise TypeError(
                self._testValueHasAcceptableTypeGetMsg( 'scalars', attrname, self._scalartypes, value ) )
        #---------------------------------------------------------------------------------------------------
        if self.isVector and not isinstance( value, self._vectortypes ):
            raise TypeError(
                self._testValueHasAcceptableTypeGetMsg( 'vectors', attrname, self._vectortypes, value ) )

    #-------------------------------------------------------------------------------------------------------
    def _assertAllValuesOkWithVocabulary( self, value ):
        if value is Misfit or not self.controlled: return
        if self.isVector:
           values = value
        else:
           values = [ value ]
        allowedValues = self.vocabulary.values()
        for value in values:
            if value in allowedValues: continue
            allowedValues = ', '.join( [ `v` for v in allowedValues ] )
            raise ValueError(( ''
                + "In case of a controlled vocabulary, facet attribute ~.default must be among listed "
                + "values: %(allowedValues)s; tried to set: %(value)r" ) % vars() )

    #-------------------------------------------------------------------------------------------------------
    def __unicode__( self ):
        return '(%s = %r)' % ( self.name, self.value, )


#===========================================================================================================
MultilateralFacetContainer.Facet = MultilateralFacet

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit(50)
    sep = '#' * 108
    delws = lambda x: x.replace( ' ', '' )

    #=======================================================================================================
    print
    print sep
    f0 = MultilateralFacet(
        name        = 'x',
        value       = 42, # [ 42, u'ä', u'\uffff' ],
        isVector    = False,
        scalartype  = int,
        default     = 111,
        )
    #-------------------------------------------------------------------------------------------------------
    print f0
    assert f0.value == 42
    assert f0.isVector is False
    assert f0.querystring == 'x=42'
    assert delws( f0.javascript ) == 'x=42;'
    assert f0.default == 111
    del f0.value
    assert f0.value == 111
    f0.querystring = 'x=108'
    assert f0.value == 108
    f0.querystring = ''
    #assert f0.value == f0.default

    #-------------------------------------------------------------------------------------------------------
    f0 = MultilateralFacet(
        name        = 'x',
        value       = [ 42 ], # [ 42, u'ä', u'\uffff' ],
        isVector    = True,
        scalartype  = int,
        default     = [ 111, ],
        )
    f0.querystring  = 'x=43&&x=%C3%A4&x=%EF%BF%BF'
    print f0
    assert f0.value == [ 43 ]
    assert f0.querystring == 'x=43'
    assert delws( f0.javascript ) == 'x=[43];'
    assert f0.default == [ 111 ]
    del f0.value
    print f0.value
    assert f0.value == [ 111 ]

    #-------------------------------------------------------------------------------------------------------
    f0 = MultilateralFacet(
        name        = 'x',
        isVector    = True,
        scalartype  = int,
        default     = [ 111, ],
        )
    f0.querystring  = 'x=43&x=108'
    print f0
    assert f0.value == [43, 108]
    assert f0.querystring == 'x=43&x=108'
    assert delws( f0.javascript ) == 'x=[43,108];'

    #-------------------------------------------------------------------------------------------------------
    f0 = MultilateralFacet(
        name        = 'x',
        isVector    = True,
        )
    print f0
    f0.querystring = 'x=42&x=%C3%A4&x=%EF%BF%BF'
    assert f0.value == [u'42', u'\xe4', u'\uffff'] # [ 42, u'ä', u'\uffff' ]
    assert delws( f0.javascript ) == "x=['42','\\xe4','\\uffff'];"

    #=======================================================================================================
    print
    print sep
    facets = MultilateralFacetContainer()
    facets.add(
        MultilateralFacet(
            name        = 'x',
            value       = 42,
            isVector    = False,
            scalartype  = float,
            ),
        MultilateralFacet(
            name        = 'foo',
            value       = u'hello world & ä & \uffff;',
            isVector    = False
            ),
        )
    #-------------------------------------------------------------------------------------------------------
    js = delws( facets.javascript )
    assert "x=42;" in js
    assert "foo='helloworld&\\xe4&\\uffff;';" in js


    #=======================================================================================================
    print
    print sep
    facets = MultilateralFacetContainer()
    facets.add(
        MultilateralFacet(
            name        = 'x',
            isVector    = True,
            default     =  [ 0 ],
            scalartype  = int,
            controlled  = True,
            vocabulary  = dict( hello = 0, eech = 1, world = 42, flush = 123 ),
            ),
        MultilateralFacet(
            name        = 'y',
            isVector    = False,
            default     = 0,
            scalartype  = int,
            controlled  = True,
            vocabulary  = dict( hello = 0, eech = 1, world = 42, flush = 123 ),
            ),
        MultilateralFacet(
            name        = 'foo',
            isVector    = False,
            ),
        )
    print facets
    facets.querystring = 'x=42&x=123&x=1&x=90&x=%C3%A4&x=%EF%BF%BF&foo=hello+world+%26+%C3%A4+%26+%EF%BF%BF%3B&foo=XXX'
    print facets
    print facets.i.y.vocabulary.nameForValue( 77 )
    print facets.i.y.vocabulary.nameForValue( 42 )

    zz

    allowedValues = Set( facets.i.x.vocabulary.values() )
    for subvalue in facets.i.x.value:
        assert subvalue in allowedValues

    print facets.i.x
    print `facets.i.x.value`
    print `facets.i.foo.value`

    print facets.i.x.vocabulary
    assert facets.i.x.vocabulary.i.hello == 0
    print facets.i.foo.vocabulary

    #=======================================================================================================
    print
    print sep
    facets = MultilateralFacetContainer()
    facets.add(
        MultilateralFacet(
            name        = 'x',
            isVector    = True,
            default     = Misfit,
            scalartype  = int,
            controlled  = True,
            vocabulary  = dict( arc = 0, bo = 1, cy = 2, dean = 3, world = 42, flush = 123 ),
            ),
        MultilateralFacet(
            name        = 'foo',
            isVector    = False,
            ),
        )
    print 'x' in facets
    print 'foo' in facets
    print 'bar' in facets
    #import sys; sys.exit()

    try:
        facets.i.x.default = 23
        assert False
    except TypeError, e:
        print 'Caught exception, OK: %s' % ( e, )

    try:
        facets.i.x.default = [ -3 ]
        assert False
    except ValueError, e:
        print 'Caught exception, OK: %s' % ( e, )

    print facets
    import os
    os.environ[ 'QUERY_STRING' ] = 'x=42&x=123&x=1&x=90&x=%C3%A4&x=%EF%BF%BF&foo=hello+world+%26+%C3%A4+%26+%EF%BF%BF%3B&foo=XXX'
    facets.updateFromCgiEnvironment()
    print facets
    print facets.i.x.copy( name = 'xy', value = [ 42, 100, 99, 1 ] )

    print facets.getQuerystring( x = facets.i.x.value + [ facets.i.x.i.dean ], )
    for facet in facets.iterQuerystringFacets( x = [ 0, 1 ], ):
        print
        print '-' * 108
        print facet.name
        print facet.default
        print facet.value


    #=======================================================================================================
    print
    print sep

    class X( object ):
        facetDescriptions = dict(
            im = dict(
                name        = 'im',
                default     = 'no',
                controlled  = True,
                vocabulary  = dict(
                        none    = 'no',
                        debug   = 'db',
                        foo     = 'fo',
                )),
            )

    self = X()
    FacetContainer = MultilateralFacetContainer
    Facet = MultilateralFacet
    import os
    os.environ[ 'QUERY_STRING' ] = 'im=dbx&im=fo&x=42'


    self.FacetContainer     = FacetContainer()
    self.fc                 = self.FacetContainer
    for facetDescription in self.facetDescriptions.itervalues():
        self.fc.add( Facet( **facetDescription ) )
    #---------------------------------------------------------------------------------------------------
    self.fc.updateFromCgiEnvironment()
    #---------------------------------------------------------------------------------------------------
    #self.fc.i.im.querystring = 'im=fo' # 'im=fox'
    #print '---'
    #self.fc.i.im.value = 'fo' # 'im=fox'
    #print `self.fc.i.im.value`
    #---------------------------------------------------------------------------------------------------
    self.missionVocabulary  = self.fc.i.im.vocabulary
    self.mission            = self.missionVocabulary.nameForValue()
    for name, value in self.missionVocabulary.iteritems():
        setattr( self, 'mission_%s' % name, value )
    #print dir( self )
    print `self.fc.i.im.value`
    #print `self.fc.i.im.default`
    #print `self.mission`

    #=======================================================================================================
    print
    print sep

    class X( object ):
        facetDescriptions = dict(
            im = dict(
                name            = 'im',
                inJavascript    = False,
                default         = 'no',
                controlled      = True,
                vocabulary      = dict(
                    none            = 'no',
                    debug           = 'db',
                    foo             = 'fo',
                    )),
            goo = dict(
                name            = 'goo',
                default         = 'yes',
                controlled      = True,
                vocabulary      = dict(
                    yes             = 'yes',
                    none            = 'no',
                )),
            width = dict(
                name            = 'width',
                inQuerystring   = False,
                default         = 100,
                ),
            height = dict(
                name            = 'height',
                inQuerystring   = False,
                default         = 200,
                ),
            )

    self = X()
    FacetContainer = MultilateralFacetContainer
    Facet = MultilateralFacet
    import os
    os.environ[ 'QUERY_STRING' ] = 'im=dbx&im=fo&x=42'


    self.FacetContainer     = FacetContainer()
    self.fc                 = self.FacetContainer
    for facetDescription in self.facetDescriptions.itervalues():
        self.fc.add( Facet( **facetDescription ) )
    #---------------------------------------------------------------------------------------------------
    self.fc.updateFromCgiEnvironment()

    print self.fc.getQuerystring( x = facets.i.x.value + [ facets.i.x.i.dean ], )
    for facet in self.fc.iterQuerystringFacets():
        print
        print '-' * 108
        print facet.name
        print facet.default
        print facet.value
        print facet.inQuerystring
        print facet.inJavascript

    print 'Facets in JavaScript:'
    for facet in self.fc.itervalues( lambda facet: facet.inJavascript ):
        print '\t', facet
    print 'Facets in Querystring:'
    for facet in self.fc.itervalues( lambda facet: facet.inQuerystring ):
        print '\t', facet
    print self.fc.javascript

    class Holder( object ):
        def __init__( self ):
            self.goo = 1024

    class AuxJavascriptHolder( object ):
        pass

    h = Holder()
    ajh = AuxJavascriptHolder()

    self.fc.i.goo.holder = h
    self.fc.auxvars = ajh
    print self.fc.querystring
    h.goo = 'hello world'
    print self.fc.querystring
    print h.goo

    self.fc.i.goo.value = 'yes'
    print self.fc.querystring
    print h.goo
    print sep
    print self.fc.javascript
    print sep
    ajh.extra = 'like crazy'
    print self.fc.javascript

    facetB = MultilateralFacet(
        name            = 'sizewithsourcepaneids',
        isVector        = True,
        value           = [],
        )
    print facetB.javascript

