#
# -*- coding: utf-8 -*-



############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
import re
from re import error as ReError
from sets import Set
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
nlnormalizer                            = u.nlnormalizer
#-----------------------------------------------------------------------------------------------------------
Tokenizer                               = nlnormalizer.Tokenizer.Tokenizer
CamelCaseTokenizer                      = nlnormalizer.Tokenizer.CamelCaseTokenizer
ListPhrase                              = nlnormalizer.Tokenizer.ListPhrase
typo                                    = u.typo
#-----------------------------------------------------------------------------------------------------------

############################################################################################################
#
#   HELPERS
#
#===========================================================================================================

iterLinetuples = typo.IterLinetuples( splitter = '=>' )

#   def simpleDict( text ):
#       R = {}
#       for old, new in iterLinetuples( text ):
#           R[ old ] = new
#       return R

def pairsFromLinetuples( text ):
    return [ ( lhs, rhs ) for lhs, rhs in iterLinetuples( text ) ]

def regexStrPairsFromLinetuples( text ):
    R = []
    try:
        #   R = [
        #       ( re.compile( pattern ), substitution )
        #           for pattern, substitution in iterLinetuples( text ) ]
        for pattern, substitution in iterLinetuples( text ):
            #print `pattern`, `substitution`
            R.append( ( re.compile( pattern ), substitution ) )
    except ReError:
        raise ReError( 'Unable to compile pattern: %r' % ( pattern, ) )
    return R

prepurgativeTextCorrections = regexStrPairsFromLinetuples( r"""
    (?i)pers\. comm\.               => personal communication
    Examples:.*$                =>
    """ )
postpurgativeTextCorrections = regexStrPairsFromLinetuples( r"""
    (?i)(m|M)easurment                  => \g<1>easurement
    (?i)WMSURL                          => WmsUrl
    (?i)apearing                        => appearing
    (?i)areacode                        => area code
    (?i)degress                         => degrees
    (?i)filename                        => file name
    (?i)geomormophological              => geomorphological
    (?i)languge                         => language
    (?i)lat/long                        => latitude longitude
    (?i)namedarea                       => named area
    (?i)opengis                         => open gis
    (?i)palaeont                        => paleont
    (?i)personteam                      => person team
    (?i)placenames                      => place names
    (?i)preferredflag                   => preferred flag
    (?i)preferredidentificationflag     => preferred identification flag
    (?i)primarycollectorflag            => primary collector flag
    (?i)provenancetype                  => provenance type
    (?i)reftype                         => reference type
    (?i)shortname                       => short name
    (?i)specimenunit                    => specimen unit
    (?i)taxonrank                       => taxon rank
    (?i)unitid                          => unit id
    (thelocation)                       => the location

    """ )
#       (?i)geospatial              => geography space

tokenCorrections = pairsFromLinetuples( """
    #-------------------------------------------------------------------------------------------------------
    #   Typos
    allocthonous                => allochthonous
    assigened                   => assigned
    atomised                    => atomized
    autho                       => author
    chronostraigraphic          => chronostratigraphic
    chronstratigraphic          => chronostratigraphic
    identifer                   => identifier
    lithstratigraphic           => lithostratigraphic
    orgininal                   => original
    origninal                   => original
    representatioin             => representation
    speciemen                   => specimen
    ructured                    => structured
    subgenenus                  => subgenus
    txonomic                    => taxonomic

    #-------------------------------------------------------------------------------------------------------
    #   Variant Spellings
    arbour                      => arbor
    armour                      => armor
    behaviour                   => behavior
    calibre                     => caliber
    candour                     => candor
    catalogue                   => catalog
    centre                      => center
    chimaera                    => chimera
    colour                      => color
    defence                     => defense
    demeanour                   => demeanor
    favour                      => favor
    flavour                     => flavor
    fon                         => telephone
    fone                        => telephone
    glamour                     => glamor
    goitre                      => goiter
    harbour                     => harbor
    honour                      => honor
    labour                      => labor
    litre                       => liter
    manoeuvre                   => maneuver
    meter                       => metre
    meters                      => metres
    mitre                       => miter
    neighbour                   => neighbor
    offence                     => offense
    organisation                => organization
    organise                    => organize
    phone                       => telephone
    pretence                    => pretense
    programme                   => program
    rancour                     => rancor
    realise                     => realize
    rigour                      => rigor
    saviour                     => savior
    spectre                     => specter
    splendour                   => splendor
    sympathise                  => sympathize
    telefone                    => telephone
    theatre                     => theater
    vapour                      => vapor
    visualise                   => visualize

    #-------------------------------------------------------------------------------------------------------
    #   Abbreviations
    alt                         => alternative
    incl                        => include
    lat                         => latitude
    pal                         => paleontological

    #-------------------------------------------------------------------------------------------------------
    #   Plurals
    accessions                  => accession
    addresses                   => address
    agents                      => agent
    applications                => application
    applies                     => apply
    areas                       => area
    assemblages                 => assemblage
    associations                => association
    attributes                  => attribute
    authors                     => author
    bases                       => basis
    cases                       => case
    categories                  => category
    chemicals                   => chemical
    circumstances               => circumstance
    codes                       => code
    collections                 => collection
    collectors                  => collector
    comments                    => comment
    conditions                  => condition
    constraints                 => constraint
    contributions               => contribution
    coordinates                 => coordinate
    counts                      => count
    datasources                 => datasource
    degrees                     => degree
    derivations                 => derivation
    designations                => designation
    details                     => detail
    dimensions                  => dimension
    doubts                      => doubt
    elements                    => element
    events                      => event
    facts                       => fact
    features                    => feature
    geocodes                    => geocode
    hours                       => hour
    images                      => image
    individuals                 => individual
    items                       => item
    kbytes                      => kbyte
    levels                      => level
    lichens                     => lichen
    locations                   => location
    maps                        => map
    marks                       => mark
    materials                   => material
    measurements                => measurement
    meters                      => meter
    methods                     => method
    names                       => name
    notes                       => note
    numbers                     => number
    observations                => observation
    parts                       => part
    parties                     => party
    persons                     => person
    pictures                    => picture
    places                      => place
    plants                      => plant
    points                      => point
    projects                    => project
    properties                  => property
    reasons                     => reason
    records                     => record
    references                  => reference
    remarks                     => remark
    responsibles                => responsible
    restrictions                => restriction
    rights                      => right
    services                    => service
    sets                        => set
    specimens                   => specimen
    stages                      => stage
    statements                  => statement
    subtypes                    => subtype
    systems                     => system
    terms                       => term
    uris                        => uri
    urls                        => url
    urns                        => urn
    weekdays                    => weekday

    #-------------------------------------------------------------------------------------------------------
    #   3rd Person
    takes                       => take
    stands                      => stand
    states                      => state
    describes                   => describe
    adheres                     => adhere
    belongs                     => belong
    handles                     => handle
    defines                     => define

    #-------------------------------------------------------------------------------------------------------
    #   Participles
    accessioned                 => accession
    added                       => add
    affiliated                  => affiliate
    assigned                    => assign
    assumed                     => assume
    based                       => base
    changed                     => change
    collected                   => collect
    composed                    => compose
    conserved                   => conserve
    constructed                 => construct
    controlled                  => control
    created                     => create
    cultivated                  => cultivate
    derived                     => derive
    edited                      => edit
    entered                     => enter
    estimated                   => estimate
    excluded                    => exclude
    expressed                   => express
    gathered                    => gather
    grown                       => grow
    held                        => hold
    inferred                    => infer
    unknown                     => know
    latinized                   => latin
    linked                      => link
    marked                      => mark
    measured                    => measure
    modified                    => modify
    named                       => name
    needed                      => need
    observed                    => observe
    obtained                    => obtain
    operated                    => operate
    preferred                   => prefer
    proposed                    => propose
    protected                   => protect
    provided                    => provide
    published                   => publish
    resulted                    => result
    shared                      => share
    structured                  => structure
    supplied                    => supply
    transferred                 => transfer
    typified                    => typify
    updated                     => update
    unverified                  => verify

    #-------------------------------------------------------------------------------------------------------
    #   Gerunds
    acting                      => act
    allowing                    => allow
    assigning                   => assign
    beginning                   => begin
    coding                      => code
    collecting                  => collect
    containing                  => contain
    contributing                => contribute
    converting                  => convert
    corresponding               => correspond
    describing                  => describe
    designating                 => designate
    enclosing                   => enclose
    excluding                   => exclude
    extracting                  => extract
    holding                     => hold
    including                   => include
    linking                     => link
    living                      => live
    mapping                     => map
    measuring                   => measure
    moving                      => move
    numbering                   => number
    pertaining                  => pertain
    providing                   => provide
    ranking                     => rank
    reading                     => read
    referring                   => refer
    sampling                    => sample
    sorting                     => sort
    structuring                 => structure
    writing                     => write

    #-------------------------------------------------------------------------------------------------------
    #   Other
    accreditation               => accredit
    acknowledgement             => acknowledge
    acknowledgements            => acknowledge
    acknowledging               => acknowledge
    applicable                  => apply
    application                 => apply
    applied                     => apply
    assessment                  => assess
    association                 => associate
    categorisation              => category
    citation                    => cite
    citing                      => cite
    classification              => classify
    collection                  => collect
    completeness                => complete
    composite                   => compose
    conservation                => conserve
    contribution                => contribute
    creation                    => create
    declaration                 => declare
    derivation                  => derive
    description                 => describe
    designation                 => designate
    estimation                  => estimate
    expression                  => express
    gathering                   => gather
    interpretation              => interpret
    measurement                 => measure
    mineralisation              => mineral
    observation                 => observe
    preservation                => preserve
    publication                 => publish
    registration                => register
    restriction                 => restrict
    retrieval                   => retrieve
    transcription               => transcribe
    usage                       => use
    zonation                    => zone

    #-------------------------------------------------------------------------------------------------------
    #   Deverbal Adjectives
    conformant                  => conform
    doubtful                    => doubt
    descriptive                 => describe
    additional                  => add
    addendum                    => add
    distinctive                 => distinguish

    #-------------------------------------------------------------------------------------------------------
    #   Denominal Adjectives
    algal                       => algae
    bacterial                   => bacteriology
    botanical                   => botany
    chemical                    => chemistry
    cyanobacterial              => cyanobacteria
    directional                 => direction
    geoecological               => geoecology
    geographic                  => geography
    geographical                => geography
    geological                  => geology
    geomorphological            => geomorphology
    hierarchical                => hierarchy
    horticultural               => horticulture
    infraspecific               => infraspecies
    instutional                 => institution
    lichenological              => lichenology
    mathematical                => mathematics
    microbial                   => microbiology
    mycological                 => mycology
    nomenclatural               => nomenclature
    numeric                     => number
    original                    => origin
    palaeontological            => palaeontology
    personal                    => person
    scientific                  => science
    sexual                      => sex
    simplified                  => simple
    spatial                     => space
    standardised                => standard
    symmetric                   => symmetry
    synecological               => synecology
    textual                     => text
    universal                   => universe
    viral                       => virus
    zoological                  => zoology

    #-------------------------------------------------------------------------------------------------------
    #   Deverbal Nouns
    growth                      => grow

    #-------------------------------------------------------------------------------------------------------
    #   Adverbs
    validly                     => valid
    commonly                    => common
    tightly                     => tight
    previously                  => previous
    preferably                  => prefer
    necessarily                 => necessary
    fully                       => full

    #-------------------------------------------------------------------------------------------------------
    #   Comparatives and Superlatives
    lower                       => low
    lowest                      => low
    nearest                     => near
    higher                      => high
    upper                       => up

    #-------------------------------------------------------------------------------------------------------
    #   Terms unification
    atomized                    => atom
    basis                       => base
    biostratigraphic            => biostratigraphy
    kbyte                       => byte
    chronostratigraphic         => chronostratigraphy
    classify                    => class
    collector                   => collect
    component                   => compose
    container                   => contain
    contains                    => contain
    content                     => contain
    contents                    => contain
    cultivar                    => culture
    cultivate                   => culture
    dates                       => date
    dating                      => date
    datum                       => date
    editor                      => edit
    formal                      => form
    format                      => form
    formatted                   => form
    formed                      => form
    forms                       => form
    identification              => id
    identifications             => id
    identified                  => id
    identifier                  => id
    identifiers                 => id
    identifies                  => id
    identify                    => id
    identity                    => id
    avi                         => image
    gif                         => image
    jpg                         => image
    mpg                         => image
    pcx                         => image
    photographs                 => image
    picture                     => image
    tiff                        => image
    indicating                  => indicate
    indication                  => indicate
    indicator                   => indicate
    lichenology                 => lichen
    locality                    => local
    location                    => local
    markup                      => mark
    ownership                   => owner
    parentheses                 => parenthese
    parenthesis                 => parenthese
    parenthetical               => parenthese
    preparation                 => prepare
    preparing                   => prepare
    qualified                   => qualify
    qualifier                   => qualify
    recorded                    => record
    recording                   => record
    reference                   => refer
    rejected                    => reject
    rejection                   => reject
    related                     => relation
    relating                    => relation
    relationship                => relation
    relationships               => relation
    representation              => represent
    represented                 => represent
    represents                  => represent
    scheme                      => schema
    specific                    => specify
    specifically                => specify
    specification               => specify
    specified                   => specify
    statement                   => state
    stating                     => state
    status                      => state
    supplementary               => supply
    supplier                    => supply
    suppressed                  => suppress
    suppression                 => suppress
    taxa                        => taxon
    taxonomic                   => taxon
    types                       => type
    typify                      => type
    uri                         => url
    urn                         => url
    verification                => verify
    verifier                    => verify
    verifying                   => verify

    """ )

shortTerms = Set( """id iso wfs gml utm uri urn url lat pal ipr""".split() )
stopwords  = Set( """dataset datasets set sets units unit cauliflower""".split() )
#   Tokens to be taken out of stopwords:
gowords    = Set( """get""".split() )

#   def update( d0, d1 ):
#       R = d0.copy()
#       R.update( d1 )
#       return R

class AbcdParameters:
    shortTerms                      = Tokenizer.shortTerms                   | shortTerms
    stopwords                       = ( Tokenizer.stopwords                  | stopwords ) ^ gowords
    prepurgativeTextCorrections     = Tokenizer.prepurgativeTextCorrections  + prepurgativeTextCorrections
    postpurgativeTextCorrections    = Tokenizer.postpurgativeTextCorrections + postpurgativeTextCorrections
    tokenCorrections                = Tokenizer.tokenCorrections             + tokenCorrections

class AbcdTextTokenizer( AbcdParameters, Tokenizer ):
    pass

class AbcdPathTokenizer( AbcdParameters, CamelCaseTokenizer ):
    pass

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    groveFed = u.feds.resolve( 'Grove' )
    dsGrove = groveFed.dsGrove
    tokenizer = AbcdPathTokenizer()
    view = dsGrove.catchTable( node = dict( gauge = 'abcd12' ) )
    i = -1
    for nodefact in view:
        i += 1
        if i > 50: break
        locator = nodefact.locator
        print ListPhrase( tokenizer.tokenize( locator ) ).distinctTokensByAlphabet
    for text in (
        """/CamelCase/CMFLister/ABCDIsASchema/isAGoodThing/KLM/WMSURL/4Picture/43picture/image/gif""",
        u"""areacode namedarea provenancetype shortname palaeontological hällo""",
        ):
        print ' '.join( tokenizer.tokenize( text ) )

