#
# -*- coding: utf-8 -*-

"""

Routines to reduce sequences of complex latin characters such as ligatures and accented letters to sequences
of 'plain' latin characters from the range ``[a-zA-Z]``.


"""


############################################################################################################
#
#
#
#===========================================================================================================
class LatinNormalizer:

    #=======================================================================================================
    complexCharacters = u"""Š  Œ  š  œ  Ÿ  À  Á  Â  Ã  Ä  Å  Æ  Ç  È  É  Ê  Ë  Ì  Í  Î  Ï  Ð  Ñ
                            Ò  Ó  Ô  Õ  Ö  Ø  Ù  Ú  Û  Ü  Ý  Þ  ß  à  á  â  ã  ä  å  æ  ç  è  é
                            ê  ë  ì  í  î  ï  ð  ñ  ò  ó  ô  õ  ö  ø  ù  ú  û  ü  ý  þ  ÿ""".split()
    #-------------------------------------------------------------------------------------------------------
    plainCharacters  =  u"""S  Oe s  oe Y  A  A  A  A  A  A  A  C  E  E  E  E  I  I  I  I  D  N
                            O  O  O  O  O  O  U  U  U  U  Y  th ss a  a  a  a  a  a  ae c  e  e
                            e  e  i  i  i  i  dh n  o  o  o  o  o  o  u  u  u  u  y  th y""".split()

    #=======================================================================================================
    def __call__( this, text ):
        tt = this.fromComplexToPlain
        R = []
        for c in text:
            R.append( tt.get( c, c ) )
        return ''.join( R )
    __call__ = classmethod( __call__ )

    #=======================================================================================================
    fromComplexToPlain = {}
    for cc, pc in zip( complexCharacters, plainCharacters ):
        fromComplexToPlain[ cc ] = pc

LatinNormalizer = LatinNormalizer()

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    t0 = u"""Hällö Wörld, I still hæve drÿ Øre."""

    print LatinNormalizer( t0 )



