#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
#   from __future__ import division
#-----------------------------------------------------------------------------------------------------------
EzClassMc                               = g.configtool.utilities.EzClassMc.EzClassMc
XmlMaker                                = g.configtool.utilities.XmlMaker
#   ReprMx                                  = g.configtool.utilities.IOU.ReprMx
VeryBasicHandler                        = g.configtool.utilities.veryBasicSaxParser.VeryBasicHandler
parseXml                                = g.configtool.utilities.veryBasicSaxParser.parseXml
#   fsi                                     = g.configtool.utilities.filesystemIdioms
#   import os.path as                       ospath
#-----------------------------------------------------------------------------------------------------------
X                                       = XmlMaker.XmlMaker()

xmlpi = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'

#   <psfx>
#       <TableAlias alias='BelegArten'>
#           <Attribute dbType='unicode'>Aufnahme_ID</Attribute>
#           <Attribute dbType='unicode'>goo</Attribute>
#       </TableAlias>
#       <TableAlias alias='Orte'>
#           <Attribute dbType='date'>added_when</Attribute>
#       </TableAlias>
#   </psfx>


############################################################################################################
#
#
#
#===========================================================================================================
class VeryBasicPsfxHandler( VeryBasicHandler ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self ):
        super( VeryBasicPsfxHandler, self ).__init__()
        self.psfx               = Psfx()
        self.currentTableAlias  = None
        self.currentDbType      = None

#       ########################################################################################################
#       #   GENERIC START AND END ELEMENT HANDLING
#       #-------------------------------------------------------------------------------------------------------
#       def startElementDefault( self, name, A ):
#           pass # print '???', name, A.items()

    ########################################################################################################
    #   SPECIFIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def start_TableAlias( self, name, A ):
        assert self.locator == '/psfx/TableAlias'
        #   self.bufferCharacters = False
        self.currentTableAlias = A.get( 'alias', None )
        assert self.currentTableAlias is not None
        assert self.currentTableAlias != ''

    #-------------------------------------------------------------------------------------------------------
    def start_Attribute( self, name, A ):
        assert self.locator == '/psfx/TableAlias/Attribute'
        self.bufferCharacters = True
        self.currentDbType = A.get( 'dbType', 'unicode' )
        assert self.currentTableAlias != ''

    #-------------------------------------------------------------------------------------------------------
    def end_Attribute( self, name ):
        assert self.locator == '/psfx/TableAlias/Attribute'
        self.bufferCharacters = False
        currentAttributeName = self.getAndClearCharacters()
        self.psfx.addAttribute(
            tableAlias      = self.currentTableAlias,
            dbType          = self.currentDbType,
            name            = currentAttributeName,
            )
        self.currentDbType = None

    #-------------------------------------------------------------------------------------------------------
    def end_TableAlias( self, name ):
        assert self.locator == '/psfx/TableAlias'
        self.currentTableAlias = None


############################################################################################################
#
#
#
#===========================================================================================================
class PsfxAttribute:
    __metaclass__   = EzClassMc
    __slots__       = True
    __obligatory__  = """name tableAlias""".split()
    __optional__    = dict(
        dbType              = 'text',
        )

    #-------------------------------------------------------------------------------------------------------
    def asXmlObject( self ):
        return X.Attribute( self.name, dbType = self.dbType )


############################################################################################################
#
#
#
#===========================================================================================================
class Psfx( object ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self ):
        self.attributesByTableAlias = {}
        self.filelocator = None
        self._attributesByTablealiasDsatPairs = {}
        self.changed = False

    #-------------------------------------------------------------------------------------------------------
    def addAttribute( self, tableAlias, name, dbType = 'text' ):
        target = self.attributesByTableAlias.setdefault( tableAlias, [] )
        attribute = PsfxAttribute( name = name, dbType = dbType, tableAlias = tableAlias )
        target.append( attribute )
        self._attributesByTablealiasDsatPairs[ ( tableAlias, name ) ] = attribute
        self.changed = True

    #-------------------------------------------------------------------------------------------------------
    def removeAttribute( self, tableAlias, name ):
        attributes = self.attributesByTableAlias[ tableAlias ]
        for nr, attribute in enumerate( attributes ):
            if attribute.name != name: continue
            del attributes[ nr ]
            break
        del self._attributesByTablealiasDsatPairs[ ( tableAlias, name ) ]
        self.changed = True

    #-------------------------------------------------------------------------------------------------------
    def removeTablealias( self, tableAlias ):
        attributes = self.attributesByTableAlias[ tableAlias ]
        for attribute in attributes:
            name = attribute.name
            del self._attributesByTablealiasDsatPairs[ ( tableAlias, name ) ]
        del self.attributesByTableAlias[ tableAlias ]
        self.changed = True

    #-------------------------------------------------------------------------------------------------------
    def filloutFromDBAttributeObjectPlus( self, dBAttributeObject, allowUpdate = False ):
        self.changed = False
        messages = []
        problems = []
        tablealias  = dBAttributeObject.table
        name        = dBAttributeObject.attr
        dbType      = dBAttributeObject.type
        attribute = self.getAttributeForTablealiasAndDsatOrNone( tablealias, name )
        if attribute is None:
            self.addAttribute( tablealias, name, dbType )
        else:
            if dbType != attribute.dbType:
                if dbType not in ( 'text', 'unicode' ) or attribute.dbType not in ( 'text', 'unicode' ):
                    messages.append( ''
                        +   "Potential problem regarding type of %s.%s: " % ( tablealias, name, )
                        +   "cached value '%s', value now '%s'. " % ( attribute.dbType, dbType, )
                        +   "Setting new value (from CMF)."
                        )
                    attribute.dbType = dbType
                    self.changed = True
        return messages, problems

    #-------------------------------------------------------------------------------------------------------
    def getAttributeForTablealiasAndDsatOrNone( self, tablealias, name ):
        return self._attributesByTablealiasDsatPairs.get( ( tablealias, name ), None )

    #-------------------------------------------------------------------------------------------------------
    def asXmlObject( self ):
        #*self.attributesByTableAlias.itervalues()
        R = X._.VirtualTag()
        R.append( XmlMaker.Rawstring( xmlpi ) )
        R.append( '\n' )
        psfx = ~R.X.psfx()
        for tableAlias, attributes in self.attributesByTableAlias.iteritems():
            tableAliasTag = ~psfx.X.TableAlias( alias = tableAlias )
            for attribute in attributes:
                tableAliasTag.append( attribute.asXmlObject() )
        R.append( '\n' )
        return R

    #-------------------------------------------------------------------------------------------------------
    def write( self ):
        if self.filelocator is None:
            raise TypeError( "Unable to write file when attribute filelocator not set." )
        psfxXmlSource = '%s' % ( self.asXmlObject(), )
        fso = file( self.filelocator, 'wb' )
        fso.write( psfxXmlSource )
        fso.flush()
        fso.close
        del fso


############################################################################################################
#
#
#
#===========================================================================================================
class getPsfx( object ):

    #-------------------------------------------------------------------------------------------------------
    def f( this, filelocator ):
        #from xml.sax.saxutils import prepare_input_source
        #source = prepare_input_source( source )
        psfxHandler = VeryBasicPsfxHandler()
        try:
            parseXml( filelocator, psfxHandler )
        except StopIteration:
            R = psfxHandler.psfx
        except Exception, e:
            R = Psfx()
        else:
            R = psfxHandler.psfx
        R.filelocator = filelocator
        return R
    forFilelocator = classmethod( f ); del f


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    sep = '#' * 108

    attr = PsfxAttribute( name = 'foo', tableAlias = 'xxx' )
    print attr
    print attr.asXmlObject()

    #   <psfx>
    #       <TableAlias alias='BelegArten'>
    #           <Attribute dbType='unicode'>Aufnahme_ID</Attribute>
    #           <Attribute dbType='unicode'>goo</Attribute>
    #       </TableAlias>
    #       <TableAlias alias='Orte'>
    #           <Attribute dbType='date'>added_when</Attribute>
    #       </TableAlias>
    #   </psfx>

    psfx = Psfx()
    psfx.addAttribute( tableAlias = 'BelegArten', name = 'Aufnahme_ID', )
    psfx.addAttribute( tableAlias = 'BelegArten', name = 'goo', )
    psfx.addAttribute( tableAlias = 'Orte', dbType = 'date', name = 'added_when', )
    print psfx.asXmlObject()
    psfxXmlSource = '%s' % ( psfx.asXmlObject(), )
    #print psfxXmlSource

    filelocator = 'c:/temp/psfx_trial.xml'
    fso = file( filelocator, 'wb' )
    fso.write( psfxXmlSource )
    fso.flush()
    fso.close
    del fso
    import time; time.sleep( 0.25 )
    #filelocator = r'C:\biocase\databases\pontaurus\config\provider_setup_file.xml'
    psfx = getPsfx.forFilelocator( filelocator )
    print psfx.asXmlObject()
    print psfx.attributesByTableAlias

    #   <psfx>
    #       <TableAlias alias="Orte">
    #           <Attribute dbType="date">added_when</Attribute>
    #       </TableAlias>
    #       <TableAlias alias="BelegArten">
    #           <Attribute dbType="unicode">Aufnahme_ID</Attribute>
    #           <Attribute dbType="unicode">goo</Attribute>
    #       </TableAlias>
    #   </psfx>









