#
# -*- coding: utf-8 -*-

"""Assortment of utility functions that are not put elsewhere."""

u = g.configtool.utilities
_ReprMx = u.IOU.ReprMx


############################################################################################################
#
#
#
#===========================================================================================================
def isderivative( x, y ):
    """Return whether ``isinstance(x,y)`` or ``issubclass(x,y)`` is true; never raise an exception with unsuitable
    arguments (as occurs with ``issubclass(x,y)`` when ``x`` is not a class)."""
    try:
        return isinstance( x, y ) or issubclass( x, y )
    except:
        return False


############################################################################################################
#
#
#
#===========================================================================================================
def acquire( *P, **Q ):
    """Given any number of unnamed dictionaries (retrieved as ``P``) and any number of named arguments,
    return the dictionary that results from taking the named arguments as defaults, and all entries in
    ``P`` *in reversed order of appearance* as updates to those defaults. In other words, values with
    same names that come earlier *overwrite* values that come later."""
    for i in range( len( P ) - 1, -1, -1 ):
        Q.update( P[ i ] )
    return Q

#-----------------------------------------------------------------------------------------------------------
def fillin( mapping, *P, **Q ):
    """Set facets where not already present; ``P`` and ``Q`` are handled by the module-level
    function ``acquire()``, which see for an explanantion."""
    return acquire( mapping, **acquire( *P, **Q ) )


#-----------------------------------------------------------------------------------------------------------
def iterSimpleCaselessSort( words ):
    wordpairs = [ ( word.lower(), word ) for word in words ]
    wordpairs.sort()
    for wordpair in wordpairs:
        yield wordpair[ 1 ]

#   def reprOfFacets( **Q ):
#       return '<%s %s>' % ( self.__class__.__name__, ', '.join(
#           [ '%s=%r' % ( name[ 1 ], self.__dict__[ name[ 1 ] ], )
#               for name in words ]
#           ) )



###########################################################################################################
#
#
#
#===========================================================================================================
class MySerializableMx:

    #-------------------------------------------------------------------------------------------------------
    def __getstate__( self ):
        """Method to make instances ``pickle``\ able."""
        R = {}
        dontpickle = getattr( self, '__dontpickle__', () )
        for name, value in self.__dict__.iteritems():
            if name not in dontpickle:
                R[ name ] = value
        return R

#       #-------------------------------------------------------------------------------------------------------
#       def __setstate__( self, getstate ):
#           """Method to make instances ``pickle``\ able."""
#           for name, value in getstate.iteritems():
#               setattr( self, name, value )



############################################################################################################
#
#
#
#===========================================================================================================
class PylonDict( dict ):

    #-------------------------------------------------------------------------------------------------------
    def fillin( self, *P, **Q ):
        """Set facets where not already present; ``P`` and ``Q`` are handled by the module-level
        function ``acquire()``, which see for an explanantion."""
        self.acquire( self, **acquire( *P, **Q ) )

    #-------------------------------------------------------------------------------------------------------
    def acquire( self, *P, **Q ):
        """Update facets with the dictionary that results from applying the module-level function
        ``acquire(*P,**Q)`` to the arguments."""
        self.update( acquire( *P, **Q ) )


############################################################################################################
#
#
#
#===========================================================================================================
class Vars( dict ):

    """Dictionary-like objects created with ``Vars(x)`` turn item lookups into ``getattr()`` calls, which
    makes it possible e.g. to use ``Vars(self)`` in place of ``vars(self)`` for name-based interpolation
    strings, the difference being that the latter only uses the instance dictionary, whereas ``Vars(self)``
    all members in the inheritance chain participate in name resolution."""

    #=======================================================================================================
    def __init__( self, obj ):
        self.obj = obj

    #=======================================================================================================
    def __getitem__( self, name ):
        return getattr( self.obj, name )


############################################################################################################
#
#
#
#===========================================================================================================
class Stack( list ):

    #===========================================================================================================
    push = list.append

    #===========================================================================================================
    def top( self ):
        if self: return self[ -1 ]


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit( 50 )
    sep = '#' * 108

    d0 = PylonDict( d0 = 'd0', a = 'd0A', b = 'd0B' )
    print d0
    q0 = dict( q0 = True, a = 'q0A' )
    d0.acquire( q0, acquired = True, a = 'a0A' )
    print d0
    q1 = dict( q1 = True, a = 'q1A' )
    d0.fillin( q1, fillin = True, a = 'a1A' )
    print d0

    d1 = dict( d1 = 'd1', new = 'd1new' )
    d1 = fillin( d1, fillin = 'fillin', new = 'fillinNew' )
    print d1




