#
# -*- coding: utf-8 -*-


"""Provides services to deal with so-called document reference structures (docrefs). Docref objects provide
a convenient way to reference or keep textual resources; they are expected to provide the following members,
all of which are optional:

*   text

*   filename

*   filenameTemplate

*   file



"""

############################################################################################################
#
#   YELLOW PAGES
#
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
import os.path as                       _p
#-----------------------------------------------------------------------------------------------------------
IO                                      = u.IO
cfg                                     = g.cfg
inter                                   = u.kpyInterpol.inter

############################################################################################################
#
#
#
#===========================================================================================================
class getFilename:

    #=======================================================================================================
    def fromFilenameTemplate( this, filenameTemplate ):
        """***CONSIDER TO MOVE INTERPOLATION INTO CALLER´S NAMESPACE***"""
        return inter( filenameTemplate )
    fromFilenameTemplate = classmethod( fromFilenameTemplate )

    #=======================================================================================================
    def fromRelativeFilename( this, relativeFilename ):
        return _p.abspath( _p.join( g.cfg.homeLocator, relativeFilename, ) )
    fromRelativeFilename = classmethod( fromRelativeFilename )

############################################################################################################
#
#
#
#===========================================================================================================
class getText:
    """A docref-savvy text reader."""

    docrefAttsAndMeths = (
        ( 'text',               'fromText' ),
        ( 'filename',           'fromFilename' ),
        ( 'filenameTemplate',   'fromFilenameTemplate' ),
        ( 'file',               'fromFile', )
        )

    #=======================================================================================================
    def fromFile( this, file ):
        return this.fromText( file.read() )
    fromFile = classmethod( fromFile )

    #=======================================================================================================
    def fromFilename( this, filename, mode = 'rb' ):
        filename = getFilename.fromRelativeFilename( filename )
        try:
            fileobj = file( filename, mode )
            R = this.fromFile( fileobj )
        finally:
            try:    fileobj.close()
            except: pass
        return R
    fromFilename = classmethod( fromFilename )

    #=======================================================================================================
    def fromFilenameTemplate( this, filenameTemplate, mode = 'rb' ):
        return this.fromFilename( getFilename.fromFilenameTemplate( filenameTemplate ), mode )
    fromFilenameTemplate = classmethod( fromFilenameTemplate )

    #=======================================================================================================
    def fromText( this, octets ):
        return IO.toUnicode( octets )
    fromText = classmethod( fromText )

    #=======================================================================================================
    def fromDocref( this, docref ):
        """Retrieve text from document reference ``docref``, set its attribute ``text``, and return that
        value."""
        for name, method in this.docrefAttsAndMeths:
            R = getattr( docref, name, None )
            if R is not None:
                method = getattr( this, method )
                R = method( R )
                docref.text = R
                return R
        raise AttributeError(
            "Unable to retrieve any of these attributes: %s. Is this a valid document reference?"
                % ', '.join( [ `t[ 0 ]` for t in this.docrefAttsAndMeths ] ) )
    fromDocref = classmethod( fromDocref )


############################################################################################################
#
#
#
#===========================================================================================================
class getOctets( getText ):

    #=======================================================================================================
    def fromText( this, octets ):
        return octets
    fromText = classmethod( fromText )










