#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from __future__ import division
#-----------------------------------------------------------------------------------------------------------
import                                  re
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
_feds                                   = u.feds
Stack                                   = u.Stack.Stack
#-----------------------------------------------------------------------------------------------------------
PARENTHESESREGEX                        = re.compile( r'([()])' )
#-----------------------------------------------------------------------------------------------------------
DEFAULTGRAPHNAME                        = 'bion'
DEBUG                                   = True
#-----------------------------------------------------------------------------------------------------------

############################################################################################################
#
#
#
#===========================================================================================================
class Token( object ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, name, literal, precedence ): #, associativeness, unary = False ):
        self.name               = name
        self.literal            = literal
        self.precedence         = precedence
#           self.associativeness    = associativeness
#           self.unary              = unary

    #   #-------------------------------------------------------------------------------------------------------
    #   def __str__( self ):
    #       return '<%s>' % ( self.literal, )

    #-------------------------------------------------------------------------------------------------------
    def __repr__( self ):
        return '<%s>' % ( self.name, )

#===========================================================================================================
class Tokens:

    left        = 'L'
    right       = 'R'
    #-------------------------------------------------------------------------------------------------------
    AND         = Token( 'AND',     'and',  1,      )   #   left,  False   )
    OR          = Token( 'OR',      'or',   1,      )   #   left,  False   )
    NOT         = Token( 'NOT',     'not',  2,      )   #   right, True    )
    NORM        = Token( 'NORM',    'norm', 2,      )   #   right, True    )
    ADD         = Token( 'ADD',     '+',    2,      )   #   left,  False   )
    SUB         = Token( 'SUB',     '-',    2,      )   #   left,  False   )
    MUL         = Token( 'MUL',     '*',    5,      )   #   left,  False   )
    DIV         = Token( 'DIV',     '/',    5,      )   #   left,  False   )
    EXP         = Token( 'EXP',     '^',    7,      )   #   left,  False   )
    LPAR        = Token( 'LPAR',    '(',    0,      )   #   left,  False   )
    RPAR        = Token( 'RPAR',    ')',    0,      )   #   left,  False   )
    ENTER       = Token( 'ENTER',   None,   None,   )   #   left,  False   )
    EMPTY       = Token( 'EMPTY',   None,   None,   )   #   left,  False   )
    STOP        = Token( 'STOP',    None,   0,      )   #   left,  False   )
    #   LIFT        = Token( 'LIFT',    'lift', 2,      )   #   right, True    )
    _literals   = {}

    #-------------------------------------------------------------------------------------------------------
    def isTokenLiteral( self, x ):
        return x in self._literals

    #-------------------------------------------------------------------------------------------------------
    def f( this, x ):
        return isinstance( x, Token )
    isToken = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def tokenForLiteral( self, literal ):
        """If ``literal`` is in fact a ``Token`` instance, return it as-is (this can only happen with the
        ``STOP`` token that is appended to the list of words). If ``literal`` spells out a token literal,
        return the appropriate token. Otherwise, return the literal itself."""
        if self.isToken( literal ): return literal
        return self._literals.get( literal, literal )

    #-------------------------------------------------------------------------------------------------------
    def __init__( self ):
        #self.STOP.literal = self.STOP
        for _name in dir( Tokens ):
            _value = getattr( Tokens, _name )
            if isinstance( _value, Token ):
                r = _value.literal
                if r is None: continue
                Tokens._literals[ r ] = _value

#-----------------------------------------------------------------------------------------------------------
tokens = Tokens()


############################################################################################################
#
#
#
#===========================================================================================================
class Queryparser( object ):

    #-------------------------------------------------------------------------------------------------------
    tokens          = tokens
    _parserCache    = {}

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, graphname = None, normalizeCase = True ):
        if graphname is None: graphname = DEFAULTGRAPHNAME
        self.graphname          = graphname
        self.termnrByLocator    = {}
        self.normalizeCase      = normalizeCase
        self.groveFed           = self._getGroveFed()
        self._fillDictionary()
        self.groveFed.close()
        self.groveFed = None

    #-------------------------------------------------------------------------------------------------------
    def _getGroveFed( self ):
        R = getattr( self, 'groveFed', None )
        if R is None:
            R = _feds.resolve( 'Grove', readonly = True )
        return R

    #-------------------------------------------------------------------------------------------------------
    def _fillDictionary( self ):
        #---------------------------------------------------------------------------------------------------
        dsGrove = self._getGroveFed().dsGrove
        #---------------------------------------------------------------------------------------------------
        termnrByLocator = self.termnrByLocator
        for nodefact in dsGrove.iterNodesForGauges( self.graphname ):
            locator = nodefact.locator
            if self.normalizeCase:
                locator = locator.lower()
            termnrByLocator[ locator ] = nodefact.termnr

    #-------------------------------------------------------------------------------------------------------
    def iterNodefactsForWords( self, word ):
        #---------------------------------------------------------------------------------------------------
        dsGrove = self._getGroveFed().dsGrove
        #---------------------------------------------------------------------------------------------------
        getNodefact = dsGrove.nodefactForOntonym
        gauge       = ':%s:' % ( self.graphname, )
        for termnr in self.iterTermnrsForWord( word ):
            yield getNodefact( ( gauge, termnr, ) )

    #-------------------------------------------------------------------------------------------------------
    def iterTermnrsForWord( self, word ):
        if self.normalizeCase:
            word = word.lower()
        if '*' in word:
            pattern = '%s$' % '.*'.join( [ re.escape( subword ) for subword in word.split( '*' ) ] )
            return self._iterTermnrsForPattern( pattern )
        return self._iterTermnrsForLiteral( word )

    #-------------------------------------------------------------------------------------------------------
    def _iterTermnrsForPattern( self, pattern ):
        regex = re.compile( pattern )
        for locator, termnr in self.termnrByLocator.iteritems():
            mo = regex.search( locator )
            if mo is not None:
                yield termnr

    #-------------------------------------------------------------------------------------------------------
    def _iterTermnrsForLiteral( self, word ):
        for locator, termnr in self.termnrByLocator.iteritems():
            if locator.endswith( word ):
                yield termnr

    #-------------------------------------------------------------------------------------------------------
    def f( this, graphname = None, normalizeCase = True ):
        if graphname is None: graphname = DEFAULTGRAPHNAME
        key = ( graphname, normalizeCase )
        R = this._parserCache.get( key, None )
        if R is None:
            R = this._parserCache[ key ] = this( *key )
        return R
    getParserForGauge = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def _iterWords( self, text ):
        spacewords = text.split()
        for spaceword in spacewords:
            for word in PARENTHESESREGEX.split( spaceword ):
                if not word: continue
                yield word
        yield self.tokens.STOP

    #-------------------------------------------------------------------------------------------------------
    def iterTermsForQuery( self, query ):
        #---------------------------------------------------------------------------------------------------
        try:
            #-----------------------------------------------------------------------------------------------
            tokens          = self.tokens
            stack           = Stack( tokens.STOP )
            #-----------------------------------------------------------------------------------------------
            for word in self._iterWords( query ):
                token = tokens.tokenForLiteral( word )
                #-------------------------------------------------------------------------------------------
                if not tokens.isToken( token ):
                    ########################################################
                    if token in 'ABCDEFGH':
                        yield token; continue
                    nodes = list( self.iterNodefactsForWords( word ) )
                    if not nodes:
                        yield tokens.EMPTY
                        continue
                    yield nodes[ 0 ]
                    for i in range( 1, len( nodes ) ):
                        yield nodes[ i ]
                        yield tokens.OR
                    continue
                #-------------------------------------------------------------------------------------------
                if token is tokens.STOP:
                    while stack: yield stack.pop()
                    break
                #-------------------------------------------------------------------------------------------
                if token is tokens.LPAR:
                    stack.push( token ); continue
                #-------------------------------------------------------------------------------------------
                if token is tokens.RPAR:
                    while True:
                        if stack.top() is tokens.STOP: break
                        pop = stack.pop()
                        if pop is tokens.LPAR: break
                        yield pop
                    continue
                #-------------------------------------------------------------------------------------------
                top = stack.top()
                if top.precedence < token.precedence:
                    stack.push( token ); continue
                #-------------------------------------------------------------------------------------------
                while True:
                    top = stack.top()
                    if top is tokens.STOP or top is tokens.LPAR: break
                    pop = stack.pop()
                    if pop is tokens.LPAR: continue
                    yield pop
                stack.push( token )
        #---------------------------------------------------------------------------------------------------
        except SyntaxError:
            rquery = ' '.join( words[ : -1 ] )
            raise SyntaxError( "Unbalanced parentheses in '%s'." % ( rquery, ) )

    #-------------------------------------------------------------------------------------------------------
    def f( this, query, graphname = None, normalizeCase = True ):
        parser = this.getParserForGauge( graphname, normalizeCase )
        return parser.iterTermsForQuery( query )
    iterTermsForQueryAndGauge = classmethod( f ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, x ):
        return Tokens.isToken( x )
    isToken = classmethod( f ); del f

############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit(50)
    sep0 = '#' * 108
    sep1 = '=' * 108
    sep2 = '-' * 108


    def show( graphname, query ):
        print sep2
        print query
#           for node in parser.iterNodefactsForWords( query ):
#               print node.ontonym
        for term in Queryparser.iterTermsForQueryAndGauge( query, graphname ):
            if not isinstance( term, ( Token, basestring ) ):
                term = term.ontonym
            print term,
        print
        print

    #parser = Queryparser()
#       graphname = 'bion'
#       show( graphname, 'botanical' )
#       show( graphname, 'botanical*' )
#       show( graphname, '*garden' )
#       show( graphname, 'bion/c*/collection-type/b*' )
#       show( graphname, 'bion/c*/c*/b*' )
#       show( graphname, 'state*' )
#       show( graphname, 'botanical* and zoological*' )
#       show( graphname, 'botanical* and zoological* and derivate-preparation' )
#       show( graphname, 'botanical* and zoological* and not derivate-preparation' )
#       show( graphname, 'botanical* and zoological* and not preparation' )
#       show( graphname, 'bot* and not zool* and living or state*' )
#       graphname = 'abcd12'
#       show( graphname, 'botanical*epi*' )
#       #show( graphname, 'gather*site*' )


    graphname = 'bion'
    DEBUG = False
    show( graphname, '( A + B ) / ( C - D )' )
    show( graphname, 'A + B / ( C - D )' )
    show( graphname, 'A + B ^ C' )
    show( graphname, 'A ^ B + C' )
    show( graphname, '( A + B ) ^ C' )
    show( graphname, 'A * ( B + C + D )' )
    show( graphname, 'A * ( B + C * D )' )
    show( graphname, 'A * (B * C + D)' )
    show( graphname, 'A and ( B or C or D )' )
    show( graphname, '( A and B ) or ( C and D )' )
    show( graphname, 'A and B or C and D' )
    show( graphname, 'A or B and C or D' )
    show( graphname, 'A and not B' )
    show( graphname, 'A and not B and C' )
    show( graphname, 'A and not ( B and C )' )
    show( graphname, 'A and ( not B and C )' )
    show( graphname, 'A and not ( B or C or D )' )
    show( graphname, 'A and not E' )
    show( graphname, 'bot* and not nat*' )
    show( graphname, 'herbarium and microbial-or-tissue-culture and identification' )
    show( graphname, 'herbarium and (culture or aquatic or living)' )
    show( graphname, 'norm A and norm B' )
#       show( graphname, 'A and ( not B or C ) and D' )
#       show( graphname, 'A and ( not B or C ) and D' )
    show( graphname, """norm( norm bot* and not norm zoo*)""" )
    show( graphname, """norm( norm bot* and norm not zoo*)""" )
    show( graphname, """norm( norm bot* and not zoo*)""" )
    show( graphname, """norm( norm bot* and not not zoo*)""" )
    show( graphname, """norm( norm bot* and norm zoo*)""" )
    show( graphname, """bot* and not not zoo*""" )
    show( graphname, """not zoo* and not plants and not fungi""" )








