#
# -*- coding: utf-8 -*-

"""

CVS::

    $Revision: 3 $
    $Author: markus $
    $Date: 2005-07-01 17:15:59 +0200 (Fri, 01 Jul 2005) $

"""


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
from __future__ import division
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
Stack                                   = u.Stack.Stack
_feds                                   = u.feds
Queryparser                             = u.tempratingsweb.Queryparser.Queryparser
#-----------------------------------------------------------------------------------------------------------
DEFAULTTOPICGRAPHNAME                   = 'bion'
DEFAULTFOCUSGRAPHNAME                   = 'abcd12'
#-----------------------------------------------------------------------------------------------------------
EM_EMPTYSTACK = """Unable to process query\n'%s'\nbecause of inconsistent stack state. Try to add
parentheses to your query (i.e. write 'norm( not x )' ) instead of 'norm not x')."""

############################################################################################################
#
#
#
#===========================================================================================================
class Ratingscalculator( object ):

    def __init__( self, query, topicGraphname = None, focusGraphname = None ):
        #---------------------------------------------------------------------------------------------------
        if topicGraphname is None: topicGraphname = DEFAULTTOPICGRAPHNAME
        if focusGraphname is None: focusGraphname = DEFAULTFOCUSGRAPHNAME
        #   #   Uncomment these lines to enable searching by namespace URLs:
        #   #---------------------------------------------------------------------------------------------------
        #   groveFed            = _feds.resolve( 'Grove', readonly = True )
        #   dsGrove             = groveFed.dsGrove
        #   topicGraphname      = dsGrove.getGraphnameForGraphnameOrNamespace( topicGraphname )
        #   focusGraphname      = dsGrove.getGraphnameForGraphnameOrNamespace( focusGraphname )
        #---------------------------------------------------------------------------------------------------
        self.topicGraphname = topicGraphname
        self.focusGraphname = focusGraphname
        #---------------------------------------------------------------------------------------------------
        self.query      = ' '.join( query.split() )
        self.terms      = []
        self.report     = ''


    #-------------------------------------------------------------------------------------------------------
    def getIndifferentRatingsForEntireGraph( self, groveFed ):
        return [ ( 0, termnr ) for termnr in groveFed.dsGrove.iterTermnrs( self.focusGraphname ) ]

    #-------------------------------------------------------------------------------------------------------
    def getRatings( self ):
        #---------------------------------------------------------------------------------------------------
        topicGraphname          = self.topicGraphname
        focusGraphname          = self.focusGraphname
        query                   = self.query
        groveFed                = _feds.resolve( 'Grove', readonly = True )
        #---------------------------------------------------------------------------------------------------
        if not query: return self.getIndifferentRatingsForEntireGraph( groveFed )
        #---------------------------------------------------------------------------------------------------
        dsGrove                 = groveFed.dsGrove
        stack                   = Stack()
        tokens                  = Queryparser.tokens
#           R                       = {}
#           traces                  = {}
#           ratingsDb               = RatingsVectorsByTopicTermnrDb( topicGraphname, focusGraphname )
#           extremum                = ratingsDb[ '__meta__' ][ 'extremum' ]
#           usedNodes               = []
        #---------------------------------------------------------------------------------------------------
        queryterms = Queryparser.iterTermsForQueryAndGauge( query, topicGraphname )
        #---------------------------------------------------------------------------------------------------
        for term in queryterms:
            self.terms.append( term )
            #print sep2
            #-----------------------------------------------------------------------------------------------
            #   Next term is a token:
            #   deal with it
            if Queryparser.isToken( term ):
                token = term
                self.report += '%s\n' % ( repr( token ), )
                #-------------------------------------------------------------------------------------------
                #   Token is STOP:
                #   terminate processing
                if token is tokens.STOP:
                    #print 'STOP'
                    break
                #-------------------------------------------------------------------------------------------
                #   Token is AND:
                #   pop two v0, v1, push v0 fuzzy-AND v1
                if term is tokens.AND:
                    assert len( stack ) > 1, EM_EMPTYSTACK % query
                    vector0 = stack.pop()
                    vector1 = stack.pop()
                    assert not Queryparser.isToken( vector0 )
                    assert not Queryparser.isToken( vector1 )
                    vector0.fuzzy2andE( vector1 )
                    stack.push( vector0 )
                    continue
                #-------------------------------------------------------------------------------------------
                #   Token is OR:
                #   pop two v0, v1, push v0 fuzzy-OR v1
                if term is tokens.OR:
                    assert len( stack ) > 1, EM_EMPTYSTACK % query
                    vector0 = stack.pop()
                    vector1 = stack.pop()
                    assert not Queryparser.isToken( vector0 )
                    assert not Queryparser.isToken( vector1 )
                    vector0.fuzzy2orE( vector1 )
                    stack.push( vector0 )
                    continue
                #-------------------------------------------------------------------------------------------
                #   Token is NOT:
                #   pop v0, push -v0
                if term is tokens.NOT:
                    assert len( stack ) > 0
                    vector0 = stack.pop()
                    assert not Queryparser.isToken( vector0 )
                    vector0.mulS( -1 )
                    stack.push( vector0 )
                    continue
                #-------------------------------------------------------------------------------------------
                #   Token is NORM:
                #   pop v0, push v0.squeeze() (i.e. all values squeezed into [+1...-1])
                if term is tokens.NORM:
                    assert len( stack ) > 0
                    vector0 = stack.pop()
                    assert not Queryparser.isToken( vector0 )
                    vector0.squeeze()
                    stack.push( vector0 )
                    continue
                #-------------------------------------------------------------------------------------------
                print "Unhandled operator: %s" % ( token, )
                continue
            #-----------------------------------------------------------------------------------------------
            #   Next term is a node:
            #   push copy of that node's ratings vector to stack
            node = term
            self.report += '    %s\n' % ( node.ontonym, )
            ratingsVector = dsGrove.getRatingsvector(
                topicGraphname  = topicGraphname,
                topicTermnr     = node.termnr,
                focusGraphname  = focusGraphname,
                )
            stack.push( ratingsVector.copy() )
        if len( stack ) != 1:
            if   len( stack ) == 0: msg = "resultstack empty"
            else:                   msg = "resultstack contained more than one element"
            raise SyntaxError( ''
                +   "Unable to evaluate query '%s': %s." % ( query, msg, )
                )
        ratingsVector = stack.pop()
        ratings = [ ( -rating, focusTermnr ) for focusTermnr, rating in ratingsVector.iterAllItems() ]
        ratings.sort()
        #ratings.reverse()
        #self.report = '\n'.join( report )
        ratings = [ ( -rating, focusTermnr, ) for rating, focusTermnr in ratings ]
        return ratings





#   class handle tracings
#
#       for topicNr, queryterm in enumerate( queryterms ):
#           print queryterm
#           ratingsVector = None
#           for node in queryterm.nodes:
#               usedNodes.append( node )
#               topicTermnr = node.termnr
#               ratingsSubvector = ratingsDb[ topicTermnr ]
#               if ratingsVector is None:
#                   ratingsVector = ratingsSubvector
#                   continue
#               ratingsVector.fuzzy2orE( ratingsSubvector )
#           topicTermref        = ( topicGraphname, topicTermnr )
#           #---------------------------------------------------------------------------------------------------
#           operator = queryterm.operator
#           negation = queryterm.negation
#           #---------------------------------------------------------------------------------------------------
#           if resultingRatingsVector is None:
#               resultingRatingsVector = ratingsVector
#               if negation == NOT:
#                   resultingRatingsVector.mulS( -1 )
#           else:
#               if negation == NOT:
#                   ratingsVector.mulS( -1 )
#               if operator == AND:
#                   resultingRatingsVector.fuzzy2andE( ratingsVector )
#               elif operator == OR:
#                   resultingRatingsVector.fuzzy2orE( ratingsVector )
#               else:
#                   raise NotImplementedError( "This operator is not implemented: %s." % ( operator, ) )
#               #print resultingRatingsVector
#           #---------------------------------------------------------------------------------------------------
#           for focusTermnr, nrating in resultingRatingsVector.iterAllItems():
#               focusTermref = ( focusGraphname, focusTermnr, )
#               try:
#                   try:
#                       traceskey0  = ( topicTermref, focusTermref )
#                       tracesvalue = tracesDb[ traceskey0 ]
#                   except KeyError:
#                       traceskey1  = ( focusTermref, topicTermref )
#                       tracesvalue = tracesDb[ traceskey1 ]
#               except KeyError:
#                   tracesvalue = []
#               target = traces.setdefault( focusTermref, {} )
#               target.setdefault( topicNr, [] ).extend( tracesvalue )
#
#       #-------------------------------------------------------------------------------------------------------
#       for focusTermnr, nrating in resultingRatingsVector.iterAllItems():
#           focusTermref = ( focusGraphname, focusTermnr, )
#           uppertarget = R.setdefault( nrating, {} )
#           completetrace_ = traces[ focusTermref ]
#           completetrace = tuple( [
#               tuple( completetrace_[ topicNr ] )
#                   for topicNr in range( topicCount )
#               ] )
#           lowertarget = uppertarget.setdefault( completetrace, [] )
#           lowertarget.append( focusTermref )
#
#       #-------------------------------------------------------------------------------------------------------
#       nratings = R.keys()
#       nratings.sort()
#       nratings.reverse()
#       for nrating in nratings:
#           print sep0
#           print nrating
#           uppertarget = R[ nrating ]
#           for completetrace, focusTermrefs in uppertarget.iteritems():
#               alltopicratings = {}
#               allfocusratings = {}
#               topicratings = [ 0 ] * topicCount
#               for topicNr, completetraceoftopic in enumerate( completetrace ):
#                   print sep2
#                   for trace in completetraceoftopic:
#                       topicTermref, focusTermref, assessment = trace
#                       pnrating = assessment / extremum
#                       topicratings[ topicNr ] += pnrating
#                       alltopicratings[ topicTermref ] = alltopicratings.get( topicTermref, 0 ) + pnrating
#                       allfocusratings[ focusTermref ] = allfocusratings.get( focusTermref, 0 ) + pnrating
#                       print trace, pnrating
#               #print alltopicratings
#               #print allfocusratings
#               #print completetrace
#               print sep1
#               print topicratings
#               print sep1
#               for focusTermref in focusTermrefs:
#                   focusNode = Elohim.getNode( focusTermref )
#                   print focusNode.locator
#       sys.exit()
#       #-------------------------------------------------------------------------------------------------------
#
#
#       lasttracesvalue = None
#       for rating, node in ratingsAndNodes:
#           if rating == 0: continue
#           print '%.3f %s' % ( rating, node.ontonym, )
#
#           focusTermref = focusGraphname, focusTermnr = node.gauge, node.termnr
#           try:
#               target = traces[ focusTermref ]
#           except KeyError:
#               print '(no trace for %s)' % ( focusTermref, )
#               lasttracesvalue = None
#           else:
#               tracesvalue = target[ 0 ]
#               if tracesvalue != lasttracesvalue:
#                   lasttracesvalue = tracesvalue
#                   tracescontributions = target[ 1 ]
#                   tracescontributions = [ x / extremum for x in tracescontributions ]
#                   tracescontribution = fuzzy2And( *tracescontributions )
#                   print '# contribution %.3f (by terms: %s)' % (
#                       tracescontribution,
#                       ', '. join( [ '%.3f' % x for x in tracescontributions ] ),
#                       )
#                   print tracesvalue
#                   for originTopicTermref, originFocusTermref, originAssessment in tracesvalue:
#                       #print originTopicTermref, originFocusTermref, assessment
#                       originTopicNode = Elohim.getNode( originTopicTermref )
#                       originFocusNode = Elohim.getNode( originFocusTermref )
#                       print '\t%s' % originTopicNode.ontonym
#                       print '\t%s' % originFocusNode.ontonym
#                       print '\t\t%s' % originAssessment




############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    import sys; sys.setrecursionlimit(50)
    sep0 = '#' * 108
    sep1 = '=' * 108
    sep2 = '-' * 108


    groveFed                = _feds.resolve( 'Grove', readonly = True )
    dsGrove                 = groveFed.dsGrove

    #=======================================================================================================
    def rate( query ):
        #print ratings
        topicGraphname  = 'bion'
        focusGraphname  = 'abcd12'
        linenr      = 0
        rc          = Ratingscalculator( query, topicGraphname, focusGraphname )
        ratings     = rc.getRatings()
        #---------------------------------------------------------------------------------------------------
        print sep0
        print rc.query
        print sep2
        print rc.report
        print sep1
        #---------------------------------------------------------------------------------------------------
        for rating, focusTermnr in ratings:
            linenr += 1
            if linenr > 50: break
            numeronym   = ':%s:%s' % ( focusGraphname, focusTermnr, )
            nodefact    = dsGrove.nodefactForOntonym( numeronym )
            nomonym     = nodefact.ontonym
            print '%3d %6.3f %s' % ( linenr, rating, nomonym, )


    rate( """digital """ )
    rate( """norm digital """ )
    rate( """norm( norm digital and norm image )""" )
    rate( """norm( fungi and plants )""" )
    rate( """norm( norm bot* and not ( norm zoo* ) )""" )
    #rate( """norm( norm bot* and not norm zoo*)""" )
