#
# -*- coding: utf-8 -*-


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
BACKSLASH       = BSLASH            = intern(  '\\'     )
SPACE           = SPC               = intern(  ' '      )
UNDERSCORE      = USC               = intern(  '_'      )
NEWLINE         = NL                = intern(  '\n'     )
HASH                                = intern(  '#'      )
SLASH                               = intern(  '/'      )
STAR                                = intern(  '*'      )
ZERO                                = intern(  '\x00'   )
#-----------------------------------------------------------------------------------------------------------
ESC_SPC                             = intern( r'\ '     )
ESC_HASH                            = intern( r'\#'     )
ESC_USC                             = intern( r'\_'     )
ESC_SLASH                           = intern( r'\/'     )
#-----------------------------------------------------------------------------------------------------------
_ESCAPE_SEQUENCES                   = [
        ( ESC_SPC, SPC ), ( ESC_USC, SPC ), ( ESC_HASH, HASH ), ( ESC_SLASH, SLASH ) ]

#   ############################################################################################################
#   #
#   #   ASCII-ART
#   #
#   #===========================================================================================================
#   class ascii:

def h( text, below = '=', above = None ):
    R = []
    w = R.append
    if above is None: above = below
    if above: w( ( above * len( text ) )[ : len( text ) ] )
    w( text )
    if below: w( ( below * len( text ) )[ : len( text ) ] )
    return NL.join( R )
#       h = classmethod( h )




############################################################################################################
#
#
#
#===========================================================================================================
def findall( text, query, start = 0, stop = None ):
    """Return a (possibly empty) list with indices pointing to all occurrences of ``query`` in ``text``."""
    R = []
    if stop is None:
        stop = len( text )
    if start < stop:
        index = text.find( query, start, stop )
        if index != -1:
            R.append( index )
            if query:
                R.extend( findall( text, query, index + len( query ), stop ) )
    return R

#===========================================================================================================
def wrap( text, width = 75 ):
    """A word-wrap function that preserves existing line breaks and most spaces in the text. Expects that
    existing line breaks are posix newlines (\\n).

    Thx to Mike Brown; see http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061."""
    return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                         + len(word.split('\n',1)[0]
                              ) >= width)],
                   word),
                  text.split()
                 )

#===========================================================================================================
def wraplines( text, width = 75 ):
    return wrap( text, width ).splitlines()

#===========================================================================================================
def splitAtLast( x, splitter = '.', allowAtPeriphery = False ):
    index = x.rfind( splitter ) + 1
    if index == 0:
        return x, ''
    if allowAtPeriphery and ( index == 1 or index == len( x ) ):
        raise TypeError( "Unable to split names that start or end with %r." % ( splitter, ) )
    return x[ : index - 1 ], x[ index : ]

#===========================================================================================================
def splitdot( x ):
    return splitAtLast( x, '.', False )

#       index = x.rfind( '.' ) + 1
#       if index == 0:
#           return x, ''
#       if index == 1 or index == len( x ):
#           raise TypeError( "Unable to split names that start or end with a dot." )
#       return x[ : index - 1 ], x[ index : ]

#===========================================================================================================
def unslash( x ):
    #if x.endswith( '/' ): return x[ : -1 ]
    return x.rstrip( '/' )


#===========================================================================================================
def normalizeWhitespace( text ):
    """Given a ``text``, return a copy that has each contiguous stretch of arbitrary whitespace replaced
    by a single space (equalized) and is stripped of leading and trailing whitespace (trimmed)."""
    return ( ' '.join( text.split() ) ).strip()

#========================================================================================================
def clip( text, currentPosition = 0, stopPosition = 108, ellipsis = '...' ):
    targetLength = stopPosition - currentPosition
    ellipsisLength = len( ellipsis )
    if targetLength < ellipsisLength:
        return ellipsis [ : targetLength ]
    if len( text ) > targetLength:
        fs = '%%s%s' % ellipsis
        text = fs % text[ : targetLength - ellipsisLength ]
    R = text.ljust( targetLength )
    return R


############################################################################################################
#
#   LINE TUPLES ITERATOR
#
#===========================================================================================================
class IterLinetuples( object ):

    r"""to be written.


    Special characters and escape sequences::

        #           (hash): comment to the end of the line
        \           (backslash, space): space (especially at the beginning of a line)
        \_          (backslash, underscore): space (useful at the end of a line)
        \#          (backslash, hash): hash (i.e., not a comment)

    Extensions:
        /xyz/i      will compile to regex with flag i
        $a := foo
        a  =: $foo  interprets variables and expressions in the caller's namespace

    """

    #=======================================================================================================
    splitter        = None
    hiddenSplitter  = '\x00'
    hiddenHash      = '\x01'
    #-------------------------------------------------------------------------------------------------------
    def __init__( self, splitter = None ):
        #   self.new()
        if splitter is not None:
            self.splitter = splitter
    #   #-------------------------------------------------------------------------------------------------------
    #   def new( self ):
    #       self.clear()
    #   #-------------------------------------------------------------------------------------------------------
    #   def clear( self ):
    #       pass

    #=======================================================================================================
    def _unescape( this, text ):
        R = text
        for old, new in _ESCAPE_SEQUENCES:
            R = R.replace( old, new )
        return R
    _unescape = classmethod( _unescape )

    #=======================================================================================================
    def __call__( self, text, splitter = None ):
        #---------------------------------------------------------------------------------------------------
        if splitter is None:
            splitter = self.splitter
        #---------------------------------------------------------------------------------------------------
        if splitter is None:
            splitterchar = ' '
        else:
            splitterchar = splitter
        #---------------------------------------------------------------------------------------------------
        text = text.replace( BACKSLASH + splitterchar, self.hiddenSplitter )
        text = text.replace( ESC_HASH, self.hiddenHash )
        #---------------------------------------------------------------------------------------------------
        for line in text.splitlines():
            line = line.strip()
            line = line.split( HASH )[ 0 ]
            if line:
                lhs, rhs = line.split( splitter )
                lhs = lhs.strip()
                if lhs.startswith( BACKSLASH ) or not lhs.startswith( HASH ):
                    rhs = rhs.strip()
                    #---------------------------------------------------------------------------------------
                    lhs = self._unescape( lhs )
                    lhs = lhs.replace( self.hiddenSplitter, splitterchar )
                    lhs = lhs.replace( self.hiddenHash, HASH )
                    #---------------------------------------------------------------------------------------
                    rhs = self._unescape( rhs )
                    rhs = rhs.replace( self.hiddenSplitter, splitterchar )
                    rhs = rhs.replace( self.hiddenHash, HASH )
                    #---------------------------------------------------------------------------------------
                    yield lhs, rhs


############################################################################################################
#
#
#
#===========================================================================================================
def intfsFromfloatfs( floatfs ):
    """Given a formatstring of the form ``'%9.4f'`` or similar, try to return a format string for integers
    that has a compatible alignment of digits."""
    test = floatfs % 0
    dotposition = test.find( '.' )
    if dotposition < 0:
        dotposition = test.find( '0' ) + 1
    R = '%%%dd%s' % ( dotposition, ' ' * ( len( test ) - dotposition ) )
    return R

def _testIntfsFromfloatfs():
    for floatfs in """%6.f %3f %5.3f %10.2f""".split():
        intfs = intfsFromfloatfs( floatfs )
        print '#' * 108
        print floatfs
        print intfs
        print floatfs % 0, '#'
        print intfs % 6, '#'


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    print `IterLinetuples._unescape( 'abc\#\ def\_' )`
    print `IterLinetuples._unescape("""\r\n\t\j\ \#\_
    abc""")`

    for x, y in IterLinetuples()( """
    hello,world
    how,are you\,today
    """, splitter = ',' ):
        print `x`, `y`

    linetuples = IterLinetuples( splitter = '=>' )
    for x, y in linetuples( """
    (WMSURL)                    => WmsUrl\_ # ends in a space
    (thelocation)               => the\ #location
    (?<=\b)(Long)(?!\bname)     => longitude \=> whatever
    \/hello*/                    => hello
        """ ):
        print `x`, `y`

#       text = ' foo bar '
#       for i in range( -1, 9 ):
#           wsi = wordstart( text, i )
#           print i, wsi, `text[ wsi : ]`, `completeword( text, wsi )`

    _testIntfsFromfloatfs()


