#
# -*- coding: utf-8 -*-

#   veryBasicSaxParser


############################################################################################################
#
#   YELLOW PAGES
#
#===========================================================================================================
#   from __future__ import division
#-----------------------------------------------------------------------------------------------------------
#import os
import                                  xml.sax.handler
from xml.sax import parse as            parseXml
from xml.sax._exceptions import         SAXParseException
#-----------------------------------------------------------------------------------------------------------
u                                       = g.configtool.utilities
#-----------------------------------------------------------------------------------------------------------
#   E                                       = u.IOU.E
fsi                                     = u.filesystemIdioms
XmlMaker                                = u.XmlMaker
SchemaDescription                       = u.SchemaDescription.SchemaDescription
#-----------------------------------------------------------------------------------------------------------
cmf14Prefix = 'x-cmf-1.4'
cmf20Prefix = 'x-cmf-2.0'


class BadBadError( Exception ): pass
class ParseError( BadBadError, SAXParseException ): pass

############################################################################################################
#
#
#
#===========================================================================================================
class VeryBasicHandler( xml.sax.handler.ContentHandler, object ):

    """Base class for custom SAX handlers. The intention is to provide a slightly enhanced SAX event handler
    that makes parsing of typical XML files a bit easier. The handler intercepts all calls to
    ``~.startElement()`` and ``~.endElements()`` and dispatches them to methods named like
    ``~.start_grand_opening()`` etc., where each method name and the prefix ``start`` are separated by an
    underscore, and underscores with the name are translations of hyphens in the XML (so this would be the
    method for the XML tags ``<grand_opening>`` and ``<grand-opening>`` alike). In order to facilitate
    recognition of elements with non-unique names, the property ``~.locator`` always gives the (unmangled)
    locator of the current element (e.g. ``'/root/employees/employee/name'``). In addition, attributes
    prefixed with ``within_`` hold a Boolean indicating whether parsing is currently within the element
    indicated (when the current locator is ``'/root/employees/employee/name'``, then attributes
    ``~.within_root``, ``~.within_employees``, ``~.within_employee``, ``~.within_name``, are all true, while
    all other attributes ``~.within_*`` (even ones completely unknown to the specific XML file) all return
    false). -- Characters are collected in a buffer called ``~._characters`` (but only if
    ``~.bufferCharacters`` is true, which it is not by default), and there are a few methods to get the
    resulting string and to clear the buffer."""

    _prefix_within      = 'within_'
    _prefix_start       = 'start_'
    _prefix_end         = 'end_'

    #-------------------------------------------------------------------------------------------------------
    def __init__( self ):
        xml.sax.handler.ContentHandler.__init__(self)
        self.R = []
        self.bufferCharacters = False
        self._characters = []
        self._namestack = []

    ########################################################################################################
    #   CHARACTER BUFFER HANDLING
    #-------------------------------------------------------------------------------------------------------
    def pushCharacters( self, characters ):
        self._characters.append( characters.strip() )

    #-------------------------------------------------------------------------------------------------------
    def clearCharacters( self ):
        del self._characters[:]

    #-------------------------------------------------------------------------------------------------------
    def getAndClearCharacters( self ):
        R = self.getCharacters()
        self.clearCharacters()
        return R

    #-------------------------------------------------------------------------------------------------------
    def getCharacters( self ):
        return ''.join( self._characters )

    #-------------------------------------------------------------------------------------------------------
    def characters( self, characters ):
        if self.bufferCharacters:
            self.pushCharacters( characters )

    ########################################################################################################
    #   GENERIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def _methodForElementtypename( self, prefix, name ):
        methodname = '%s%s' % ( prefix, name, )
        R = getattr( self, methodname, None )
        if R is None:
            methodname = '%sElementDefault' % ( prefix, )
            R = getattr( self, methodname, None )
        return R

    #-------------------------------------------------------------------------------------------------------
    def _xmlNameToPythonName( self, name ):
        return XmlMaker.unmangle.rules.allHyphensToUnderscores(
            XmlMaker.unmangle.rules.ColonToDoubleUnderscore(
                name
                )
            )

    #-------------------------------------------------------------------------------------------------------
    def startElement( self, name, A ):
        #   A : _attrs copy get getLength getNameByQName getNames getQNameByName
        #   getQNames getType getValue getValueByQName has_key items keys values
        self.startElementEach( name, A )
        pythonesqueName = self._xmlNameToPythonName( name )
        self.flagWithin( pythonesqueName, True )
        self._namestack.append( name )
        method = self._methodForElementtypename( self._prefix_start, pythonesqueName )
        if method is not None:
            method( name, A )

    #-------------------------------------------------------------------------------------------------------
    def startElementEach( self, name, A ):
        pass

    #-------------------------------------------------------------------------------------------------------
    def endElement( self, name ):
        method = self._methodForElementtypename( self._prefix_end, name )
        if method is not None:
            method( name )
        self.flagWithin( name, False )
        self._namestack.pop()

    #-------------------------------------------------------------------------------------------------------
    def flagWithin( self, name, value ):
        attributename = '%s%s' % ( self._prefix_within, name )
        setattr( self, attributename, value )

    #-------------------------------------------------------------------------------------------------------
    def __getattr__( self, name ):
        if not name.startswith( self._prefix_within ):
            raise AttributeError( "Unknown attribute: %r." % ( name, ) )
        return False

    #-------------------------------------------------------------------------------------------------------
    def f():
        def fget( self ):
            return '/%s' % '/'.join( self._namestack )
        return locals()
    locator = property( **f() ); del f

    #-------------------------------------------------------------------------------------------------------
    def f( this, filelocator, *P, **Q ):
        handler = this( *P, **Q )
        parseXml( filelocator, handler )
    parse = classmethod( f ); del f

############################################################################################################
#
#
#
#===========================================================================================================
class VeryBasicPsfHandler( VeryBasicHandler ):

    #-------------------------------------------------------------------------------------------------------
    def __init__( self ):
        super( VeryBasicPsfHandler, self ).__init__()
        self.schemaDescriptions = []

    ########################################################################################################
    #   GENERIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def startElementDefault( self, name, A ):
        pass # print '???', name, A.items()

    ########################################################################################################
    #   SPECIFIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def start_Provider( self, name, A ):
        pass # print name, A.items()

    #-------------------------------------------------------------------------------------------------------
    def start_Schema( self, name, A ):
        if self.locator == '/Provider/SupportedSchemas/Schema':
            self.bufferCharacters = True
            #print name, A.items()
            schemaDescription = SchemaDescription( isTemplate = False )
            self.schemaDescriptions.append( schemaDescription )
            namespaceUri    = A.get( 'namespace', None )
            namespacePrefix = None
            schemaDescription.namespaceUri = namespaceUri

    #-------------------------------------------------------------------------------------------------------
    def start_SupportedSchemas( self, name, A ):
        pass

    #-------------------------------------------------------------------------------------------------------
    def end_SupportedSchemas( self, name ):
        raise StopIteration

    #-------------------------------------------------------------------------------------------------------
    def end_Schema( self, name ):
        if self.locator == '/Provider/SupportedSchemas/Schema':
            self.bufferCharacters = False
            schemaDescription = self.schemaDescriptions[ -1 ]
            schemaDescription.filename = self.getAndClearCharacters()

############################################################################################################
#
#
#
#===========================================================================================================
class CmfHandler( VeryBasicHandler ):

    #-------------------------------------------------------------------------------------------------------
    class CmfError( Exception ): pass
    class NotAValidCmfError( CmfError, StopIteration, IOError, ValueError ): pass

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, cmflocator, isTemplate = True ):
        VeryBasicHandler.__init__( self )
        self.schemaDescription = SchemaDescription( filelocator = cmflocator, isTemplate = isTemplate )

    ########################################################################################################
    #   GENERIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def startElementEach( self, name, A ):
        print name, A
#           if len( self._namestack ) != 1: return
#           if self._namestack[ 0 ] != 'conceptMapping':
#               raise self.NotAValidCmfError(
#                   "This is not a valid CMF XML file, since its root element is named %r." % ( name, ) )


    #-------------------------------------------------------------------------------------------------------
    def f( this, filelocator, isTemplate = True ):
        handler = this( filelocator, isTemplate )
        parseXml( filelocator, handler )
    parse = classmethod( f ); del f



############################################################################################################
#
#
#
#===========================================================================================================
class CmfNamespaceDescriptionHandler( VeryBasicHandler ):

    #-------------------------------------------------------------------------------------------------------
    class CmfError( Exception ): pass
    class NotAValidCmfError( CmfError, StopIteration, IOError, ValueError ): pass

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, cmflocator, isTemplate ):
        VeryBasicHandler.__init__( self )
        self.schemaDescription = SchemaDescription( filelocator = cmflocator, isTemplate = isTemplate )

    ########################################################################################################
    #   GENERIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def startElementEach( self, name, A ):
        if len( self._namestack ) != 1: return
        if self._namestack[ 0 ] != 'conceptMapping':
            raise self.NotAValidCmfError(
                "This is not a valid CMF XML file, since its root element is named %r." % ( name, ) )

    #-------------------------------------------------------------------------------------------------------
    def start_conceptMapping( self, name, A ):
        if not self.locator == '/conceptMapping': return
        xmlns = A.get( 'xmlns', None )
        if xmlns is None:
            raise self.NotAValidCmfError( ''
                +   "This is not a valid CMF XML file, "
                +   "since its root element does not have an xmlns attribute."
                )
        if ' ' in xmlns: xmlns = xmlns.split()[ 0 ]
        #print '++++++', xmlns ###DEBUG###
        try:
            self.schemaDescription.cmfNamespacePrefix = xmlns # implicit conversion
        except '#':#Exception:
            raise self.NotAValidCmfError( ''
                +   "This is not a valid CMF XML file, "
                +   "since the xmlns attribute of the root element could "
                +   "not be recognized: %s." % ( xmlns, )
                )

    ########################################################################################################
    #   SPECIFIC START AND END ELEMENT HANDLING
    #-------------------------------------------------------------------------------------------------------
    def start_dataElement( self, name, A ):
        #print self.locator, name, A.items()
        #print 'namespace' in A.keys()
        if not self.locator == '/conceptMapping/dataElement': return
        #if self.schemaDescription.schemaPrefix == cmf20Prefix:
        namespaceUri = A.get( 'namespace', None )
        if namespaceUri is not None:
            self.schemaDescription.namespaceUri = namespaceUri
        raise StopIteration()

    #-------------------------------------------------------------------------------------------------------
    def start_schemaNamespace( self, name, A ):
        if not self.locator == '/conceptMapping/schemaNamespace': return
        self.bufferCharacters = True

    #-------------------------------------------------------------------------------------------------------
    def end_schemaNamespace( self, name ):
        #if not self.locator == '/conceptMapping/schemaNamespace': return
        self.bufferCharacters = False
        namespaceUri = self.getAndClearCharacters()
        self.schemaDescription.namespaceUri = namespaceUri # uri implicitly converted to prefix


#===========================================================================================================
def getPsfSchemaDescriptionsForPsflocator( psflocator ):
    psfHandler = VeryBasicPsfHandler()
    try:
        parseXml( psflocator, psfHandler )
    except StopIteration:
        pass
    except Exception, e:
        return None
    return psfHandler.schemaDescriptions

#===========================================================================================================
def getCmfSchemaDescriptionForCmflocatorPlus( cmflocator, isTemplate ):
    messages = []
    problems = []
    cmfHandler = CmfNamespaceDescriptionHandler( cmflocator, isTemplate )
    try:
        parseXml( cmflocator, cmfHandler )
    except StopIteration:
        pass
    except '#':#Exception, e:
        return None, messages, problems
    return cmfHandler.schemaDescription, messages, problems

#===========================================================================================================
def getCmfSchemaDescriptionForCmflocator( cmflocator, isTemplate = False ):
    cmfHandler = CmfNamespaceDescriptionHandler( cmflocator, isTemplate )
    try:
        parseXml( cmflocator, cmfHandler )
    except StopIteration:
        pass
    except Exception, e:
        raise ParseError( ''
            +   "An exception was encountered\n"
            +   "when trying to parse %s:\n" % ( cmflocator, )
            +   "(%s)\n%s" % ( e.__class__.__name__, e, )
            )
    return cmfHandler.schemaDescription


############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':

    sep  = '#' * 108
    sep1 = '=' * 108
    sep2 = '-' * 108
    import sys; sys.setrecursionlimit(50)

    psfi = g.configtool.utilities.PsfCmfIdioms.PsfIdioms
    cmfi = g.configtool.utilities.PsfCmfIdioms.CmfIdioms

    #   def iternodes( filelocator ):
    #       handler = MyHandler()
    #       parseXml( filelocator, handler )


    cmfsByPrefix = {}
    print sep
    print 'Templates:'
    for cmflocator in cmfi.iterAllLikelyCmfLocators( cmfi.getTemplatesRepositoryLocator() ):
        schemaDescription, messages, problems = \
            getCmfSchemaDescriptionForCmflocatorPlus( cmflocator, isTemplate = True )
        if schemaDescription is not None:
            #print schemaDescription.namespacePrefix, schemaDescription
            target = cmfsByPrefix.setdefault( schemaDescription.namespacePrefix, [] )
            target.append( schemaDescription )
        else:
            print '# ????', cmflocator, problems

    for prefix, sds in cmfsByPrefix.iteritems():
        print sep1
        print prefix
        for sd in sds:
            print sep2
            sd.__printoverview__()

    for dsa, dsalocator in psfi.getDsaliasLocatorPairs():
        psflocator = psfi.getPsfLocator( dsa )
        schemaDescriptions = getPsfSchemaDescriptionsForPsflocator( psflocator )
        cmfsByPrefix = cmfsByPrefix.copy()
        print
        print sep
        print dsa
        print sep
        print psflocator
        if schemaDescriptions is not None:
            for schemaDescription in schemaDescriptions:
                print schemaDescription
        print sep1
        for cmflocator in cmfi.iterAllLikelyCmfLocatorsForDsa( dsa ):
            schemaDescription, messages, problems = \
                getCmfSchemaDescriptionForCmflocatorPlus( cmflocator, isTemplate = False )
            if schemaDescription is not None:
                #print schemaDescription.namespacePrefix, schemaDescription
                target = cmfsByPrefix.setdefault( schemaDescription.namespacePrefix, [] )
                target.append( schemaDescription )
            else:
                print '# ????', cmflocator, problems
        for prefix, sds in cmfsByPrefix.iteritems():
            print sep1
            print prefix
            for sd in sds:
                print sep2
                sd.__printoverview__()


#       for x in parseXml( filelocator, handler ):
#           print x

#       f = open( r'c:\biocase\configtoolWorkshop\omega\stylishXml\xmlCss\psf_cvrtd.xml', 'wb' )
#       f.write( '<?xml version="1.0"?>' )
#       f.write( '<?xml-stylesheet type="text/css" href="psf.css"?>' )
#       f.write( '\n' )
#       f.write( ''.join( handler.R ) )
#       f.close()



