#
# -*- coding: utf-8 -*-
'''
$RCSfile: configuration.py,v $
$Revision: 699 $
$Author: markus $
$Date: 2006-04-07 17:33:11 +0200 (Fri, 07 Apr 2006) $
'''

############################################################################################################
#
#   IMPORTS AND CONSTANTS
#
#===========================================================================================================
import inspect as           _i
import os.path as           _p
import os as                _os
from cgi import escape as   _esc
from types import *
import logging, logging.config
# get a global logger instance
log = logging.getLogger("configuration")


############################################################################################################
#
#   CONFIGURATION SECTION CLASS
#
#===========================================================================================================
class ConfigurationSection( object ):

    #-------------------------------------------------------------------------------------------------------
    pass


############################################################################################################
#
#   GLOBAL CONFIGURATION BASE CLASS. might be replaced by another class when using getCfg()
#
#===========================================================================================================
def setCfgClassLocators(rootPath=None):
    if rootPath is None:
        rootPath = _p.join(_p.dirname( _i.getfile( lambda:None ) ), _os.pardir, _os.pardir)
    # set paths
    Cfg.homeLocator                 = _p.abspath( rootPath )
    Cfg.libLocator                  = _p.join( Cfg.homeLocator,   'lib'          )
    Cfg.libbiocaseLocator           = _p.join( Cfg.libLocator,    'biocase'      )
    Cfg.wwwLocator                  = _p.join( Cfg.homeLocator,             'www'                       )
    Cfg.xslLocator                  = _p.join( Cfg.homeLocator,             'www','resources','xsl'     )
    Cfg.configurationLocator        = _p.join( Cfg.homeLocator,             'configuration'             )
    Cfg.datasourcesLocator          = _p.join( Cfg.configurationLocator,    'datasources'               )
    Cfg.skinLocator                 = _p.join( Cfg.configurationLocator,    'querytool','skins'         )
    Cfg.persistenceLocator          = _p.join( Cfg.configurationLocator,    'persistence'               )
    Cfg.rawCmfTemplateLocator       = _p.join( Cfg.configurationLocator,    'templates','cmf'           )
    Cfg.prefilledTemplateLocator    = _p.join( Cfg.configurationLocator,    'templates','prefilled'     )
    Cfg.logLocator                  = _p.join( Cfg.homeLocator,             'log'                       )
    Cfg.assessmentsHomelocator      = _p.join( Cfg.configurationLocator,    'assessments'               )
    

class Cfg( object ):
    # default http header
    http_header = '''Content-Type: text/html; charset=utf-8

'''
    
    #-------------------------------------------------------------------------------------------------------
    _configurationSectionClass  = ConfigurationSection
    #-------------------------------------------------------------------------------------------------------
    def _dictAsTable( self, d, indent=0 ):
        R = [ ]
        names = d.keys()
        names.sort()
        for name in names:
            value = d[ name ]
            if type(value) == DictType:
                value = '\n' + self._dictAsTable(d=value, indent=indent+1)
            indentString = '  '*indent
            R.append( '%s%s:\t%s' % ( indentString, name, value, ) )
        R = '\n'.join( R )
        return R

    #-------------------------------------------------------------------------------------------------------
    def _dictAsHtmlTable( self, d ):
        R = [ '<table border="1" cellspacing="0" cellpadding="1" style="font-size:85%">' ]
        names = d.keys()
        names.sort()
        for name in names:
            value = d[ name ]
            if type(value) == DictType:
                value = '\n' + self._dictAsHtmlTable(d[ name ])
            else:
                value = _esc( unicode(d[ name ]) )
            name_ = _esc( name )
            R.append( '<tr><td><b>%s</b></td><td>%s</td></tr>' % ( name_, value, ) )
        R.append( '</table>' )
        R = '\n'.join( R )
        return R

    #-------------------------------------------------------------------------------------------------------
    def _environmentAsTable( self, html = False ):
        if html:
            return self._dictAsHtmlTable( _os.environ )
        else:
            return self._dictAsTable( _os.environ )

    #-------------------------------------------------------------------------------------------------------
    def _getRecursiveDict( self, obj=None ):
        if obj == None:
            obj = self
        R = {}
        for name in dir( obj ):
            if name.startswith( '_' ): continue
            if isinstance(getattr( obj, name ), ConfigurationSection):
                R[ name ] = self._getRecursiveDict( obj=getattr( obj, name ) )
            else:
                R[ name ] = getattr( obj, name )
        return R
        
    #-------------------------------------------------------------------------------------------------------
    def _asTable( self, html = False, showEnvironment = False ):
        R = self._getRecursiveDict()
        if html:
            return self._dictAsHtmlTable( R )
        else:
            return self._dictAsTable( R )

    #-------------------------------------------------------------------------------------------------------
    def __repr__( self ):
        return self._asTable()

    #-------------------------------------------------------------------------------------------------------
    def __init__( self, *includes ):
        #---------------------------------------------------------------------------------------------------
        # read dicts (not meant to be edited or written)
        readConfiguration(_p.join( self.configurationLocator, 'configtool', 'namespaces.ini' ), section=self)
        # read main .configuration file
        readConfiguration(_p.join( self.configurationLocator, '.configuration' ), section=self)
        
        #---------------------------------------------------------------------------------------------------
        self._postprocessingMethods = []        
        postprocessingMethodname = '_postprocess'
        postprocessingMethod = getattr( self, postprocessingMethodname, None )
        if postprocessingMethod is not None:
            self._postprocessingMethods.append( postprocessingMethod )
        #---------------------------------------------------------------------------------------------------
        for include in includes:
            sectionConfigurationLocator = _p.join( self.configurationLocator, include, '.configuration' )
            section = readConfiguration(sectionConfigurationLocator)
            setattr( self, include, section )

    #-------------------------------------------------------------------------------------------------------
    def _write(self):
        # write main
        params = self._getRecursiveDict()
        for pname, pval in params.items():
            if type(pval) == DictType:
                section = getattr( self, pname )
                if isinstance(section, ConfigurationSection):
                    writeConfiguration(fn=section.__filename__, **pval)
                del params[pname]
            elif pname.startswith('_') or hasattr(self.__class__, pname) :
                # dont write class attributes or attributes atarting with _
                del params[pname]
        writeConfiguration(fn=self.__filename__, **params)

    #-------------------------------------------------------------------------------------------------------
    def _postprocess( self ):
        """General postprocessing method. The method is registered in ``~._postprocessingMethods`` and
        executed by ``fundamentals.py`` when objects like ``g.cfg`` are available."""
        _inter = g.configtool.utilities.kpyInterpol.inter
        additionalGraphsourcesByPrefix = g.cfg.additionalGraphsourcesByPrefix
        #print '%%%%%%%%%%%%%%%%%%%%%', dir( g.cfg.configtool )
        for prefix, locatortemplate in additionalGraphsourcesByPrefix.iteritems():
            additionalGraphsourcesByPrefix[ prefix ] = _inter( locatortemplate )

    #-------------------------------------------------------------------------------------------------------
    def _updateLogger(self, logname):
        '''Update the logger by setting handlers with the proper log dir'''
        # read logger config        
        logging.config.fileConfig(_p.join(self.configurationLocator, "logconf.ini"))
        # formatter
        f_simple = logging.Formatter('%(asctime)s - %(levelname)s from %(name)s: %(message)s')
        f_apache = logging.Formatter('[%(asctime)s] [%(levelname)s] [%(name)s, file "%(filename)s", line %(lineno)d]  %(message)s', '%a %b %d %H:%M:%S %Y')
        l = logging.getLogger()
        # create debug file handler        
        hdl_debug = logging.handlers.RotatingFileHandler(filename=_p.join(self.logLocator, "%s-debug.log"%logname), maxBytes=100000, backupCount=3)
        hdl_debug.setFormatter(f_simple)
        hdl_debug.setLevel(logging.DEBUG)
        l.addHandler(hdl_debug)
        # create error file handler
        hdl_error = logging.handlers.RotatingFileHandler(filename=_p.join(self.logLocator, "%s-error.log"%logname), maxBytes=100000, backupCount=3)
        hdl_error.setFormatter(f_simple)
        hdl_error.setLevel(logging.ERROR)
        l.addHandler(hdl_error)

setCfgClassLocators()


# ------------------------------------------------------------------------------------
def readConfiguration(fn, section=None):
    if section is None:
        section = ConfigurationSection()
    execfile( fn, globals(), section.__dict__ )
    section.__filename__ = fn 
    return section

def escapeVal(x):
    return x.replace('\n',r'\n').replace("'",r"\'")
def createLine(param, val):
    if val is None:
        line = "%s     = None\n" % param
    elif type(val) in (DictType, ListType, TupleType):
        line = "%s     = %s\n" % (param, str(val))
    else:
        line = "%s     = r'%s'\n" % (param, val)
    return line
def writeConfiguration(fn, **params):
    #domain='http://localhost', baseURL='/biocase', adminPasswd='ACDC', graphviz_dot=None):
    f = file(fn, 'r')
    text = ""
    log.debug("Write configuration to %s" % (fn))
    log.debug("Cfg params: %s" % str(params))
    # write existing entries and update them with new values
    for line in f.readlines():
        existed = False
        for p,val in params.items():
            if p == 'http_header': continue  #FIXME: dont write 'http_header'. Hack to ignore multiple lines
            if line.startswith(p):
                text += createLine(p,val)
                existed = True
                del params[p]
                break
        if not existed:
            text += line
    # write entries that did not exist before
    for p,val in params.items():
        text += createLine(p,val)
        
    f.close()
    # write back
    fw = file(fn, 'w')
    fw.write(text)
    fw.close()
    
def getBPSversion(BpsPath):
    '''Return the version string of an BPS installation. Pass the root install dir.'''
    try:
        sect = readConfiguration(_os.path.join(BpsPath,'lib','biocase','__init__.py'))
    except IOError:
        return None
    return sect.__version__
    


        
############################################################################################################
#
#
#
#===========================================================================================================
if __name__ == '__main__':
    a = "Hi\n Du'n Tier?"
    print '"%s"' % a
    print '"%s"' % escapeVal(a)
    import sys

    sys.setrecursionlimit( 20 )

    cfg = Cfg('pywrapper','configtool')
    print cfg

    print "-"*40
    print "BPS install dir=%s" % cfg.homeLocator
    print "BPS version=%s" % getBPSversion(cfg.homeLocator)
    cfg._write()

