#
# -*- coding: utf-8 -*-

''' TBD '''


############################################################################################################
#
#   IMPORTS
#
#===========================================================================================================
from biocase.querytool import __version__
from mx.DateTime import now
import logging
# debugging to file
log = logging.getLogger("webapp.querytool")




############################################################################################################
#
#   CLASSES
#
#===========================================================================================================
class QueryGenerator:
    '''A class to generate biocase protocol xml strings.'''
    
    NS_DiGIR   = "http://digir.net/schema/protocol/2003/1.0"
    NS_BioCASe = "http://www.biocase.org/schemas/protocol/1.3"
    NS_TAPIR   = ''
    
    # ------------------------------------------------------------------------------------
    def __init__(self, protocolNS=None):
        if protocolNS is None:
            protocolNS = QueryGenerator.NS_BioCASe
        elif protocolNS.lower() == 'biocase':
            protocolNS = QueryGenerator.NS_BioCASe
        elif protocolNS.lower() == 'digir':
            protocolNS = QueryGenerator.NS_DiGIR            
        elif protocolNS.lower() == 'tapir':
            protocolNS = QueryGenerator.NS_TAPIR
        self._type = protocolNS
        
    # ------------------------------------------------------------------------------------
    def getCapabilitiesProtocol(self):
        return u'''<?xml version="1.0" encoding="UTF-8"?>
    <!-- XML generated by biocase.querytool.querygenerator. -->
    <request 
        xmlns="%s" 
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
        xsi:schemaLocation="%s http://www.bgbm.org/biodivinf/Schema/protocol_1_3.xsd">
        <header>
            <version software="querytool">%s</version>
            <sendTime>%s</sendTime>
            <source/>
            <destination/>
            <type>capabilities</type>
        </header>
    </request>''' %(QueryGenerator.NS_BioCASe, QueryGenerator.NS_BioCASe, __version__, str( now() ))
    
    # ------------------------------------------------------------------------------------
    def getScanProtocol(self, concept, NS="http://www.tdwg.org/schemas/abcd/1.2", filterObj=None):
        if filterObj is not None:
            filter = "\n<filter>%s\n</filter>\n"%filterObj.getProtocolXML()
        else:
            filter = ''
        return u'''<?xml version="1.0" encoding="UTF-8"?>
    <request 
        xmlns="%s" 
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
        xsi:schemaLocation="%s http://www.bgbm.org/biodivinf/schema/protocol.xsd">
        <header>
            <version software="querytool">%s</version>
        <sendTime>%s</sendTime>
            <type>scan</type>
        </header>
        <scan>
            <requestFormat>%s</requestFormat>
            <concept>%s</concept>
            %s
        </scan>
    </request>''' %(self._type, self._type, __version__, str(now()), NS, concept, filter)
    

    # ------------------------------------------------------------------------------------
    def getSearchProtocol(self, NS="http://www.tdwg.org/schemas/abcd/1.2", respNS="http://www.tdwg.org/schemas/abcd/1.2", start=0, limit=10, count=False, filterObj=None, destination=None):
        '''Parameter destination should be the access point for the service.
        In the case of DiGIR resources should be provided in the form of RESOURCE@ACCESSPOINT as the destination string.'''
        # if digir destination parse resource of form RESOURCE@ACCESSPOINT
        if destination is not None:
            if destination.count("@") == 1:
                digirResource, destination = destination.split("@")
            else:
                digirResource = ''
        else:
            digirResource = ''
            destination   = ''
        # prepare filter string
        if filterObj is not None:
            if self._type == QueryGenerator.NS_DiGIR:
                filter = filterObj.getProtocolXML(digir=True)
            else:
                filter = filterObj.getProtocolXML()
        else:
            filter = ''
        if self._type == QueryGenerator.NS_BioCASe:
            return u'''<?xml version="1.0" encoding="UTF-8"?>
        <!--XML generated by BioCASE query tool for a single database, written by Markus Doering and Javier de la Torre, Botanic Garden and Botanical Museum, Berlin.-->
        <request 
            xmlns="%s" 
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
            xsi:schemaLocation="%s http://www.bgbm.org/biodivinf/Schema/protocol_1_3.xsd">
            <header>
                <version software="querytool">%s</version>
                <sendTime>%s</sendTime>
                <source/>
                <destination>%s</destination>
                <type>search</type>
            </header>
            <search>
                <requestFormat>%s</requestFormat>
                <responseFormat start="%i" limit="%i">%s</responseFormat>
                <filter>
                  %s
                </filter>
                <count>%s</count>
            </search>
        </request>
        ''' %(self._type, self._type, __version__, str(now()), destination, NS, start, limit, respNS, filter, str(count))
    
        elif self._type == QueryGenerator.NS_DiGIR:
            return u'''<?xml version="1.0" encoding="UTF-8"?>
<request 
    xmlns="%s" 
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
    xmlns:darwin="%s" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" >
    <header>
      <version>1.0</version>
      <sendTime>%s</sendTime>
      <source/>
      <destination resource="%s">%s</destination>
      <type>search</type>
    </header>
    <search>
      <filter>
          %s
      </filter>
      <records start="%i" limit="%i">
        <structure>
          <xsd:element name="record">
            <xsd:complexType>
              <xsd:sequence>
                <xsd:element ref="darwin:DateLastModified"/>
                <xsd:element ref="darwin:InstitutionCode"/>
                <xsd:element ref="darwin:CollectionCode"/>
                <xsd:element ref="darwin:CatalogNumber"/>
                <xsd:element ref="darwin:ScientificName"/>
                <xsd:element ref="darwin:BasisOfRecord"/>
                <xsd:element ref="darwin:Kingdom"/>
                <xsd:element ref="darwin:Phylum"/>
                <xsd:element ref="darwin:Class"/>
                <xsd:element ref="darwin:Order"/>
                <xsd:element ref="darwin:Family"/>
                <xsd:element ref="darwin:Genus"/>
                <xsd:element ref="darwin:Species"/>
                <xsd:element ref="darwin:Subspecies"/>
                <xsd:element ref="darwin:ScientificNameAuthor"/>
                <xsd:element ref="darwin:IdentifiedBy"/>
                <xsd:element ref="darwin:YearIdentified"/>
                <xsd:element ref="darwin:MonthIdentified"/>
                <xsd:element ref="darwin:DayIdentified"/>
                <xsd:element ref="darwin:TypeStatus"/>
                <xsd:element ref="darwin:CollectorNumber"/>
                <xsd:element ref="darwin:FieldNumber"/>
                <xsd:element ref="darwin:Collector"/>
                <xsd:element ref="darwin:YearCollected"/>
                <xsd:element ref="darwin:MonthCollected"/>
                <xsd:element ref="darwin:DayCollected"/>
                <xsd:element ref="darwin:JulianDay"/>
                <xsd:element ref="darwin:TimeOfDay"/>
                <xsd:element ref="darwin:ContinentOcean"/>
                <xsd:element ref="darwin:Country"/>
                <xsd:element ref="darwin:StateProvince"/>
                <xsd:element ref="darwin:County"/>
                <xsd:element ref="darwin:Locality"/>
                <xsd:element ref="darwin:Longitude"/>
                <xsd:element ref="darwin:Latitude"/>
                <xsd:element ref="darwin:CoordinatePrecision"/>
                <xsd:element ref="darwin:BoundingBox"/>
                <xsd:element ref="darwin:MinimumElevation"/>
                <xsd:element ref="darwin:MaximumElevation"/>
                <xsd:element ref="darwin:MinimumDepth"/>
                <xsd:element ref="darwin:MaximumDepth"/>
                <xsd:element ref="darwin:Sex"/>
                <xsd:element ref="darwin:PreparationType"/>
                <xsd:element ref="darwin:IndividualCount"/>
                <xsd:element ref="darwin:PreviousCatalogNumber"/>
                <xsd:element ref="darwin:RelationshipType"/>
                <xsd:element ref="darwin:RelatedCatalogItem"/>
                <xsd:element ref="darwin:Notes"/>
              </xsd:sequence>
            </xsd:complexType>
          </xsd:element>
        </structure>
      </records>
      <count>false</count>
    </search>        
    </request>''' %(self._type, NS, str(now()), digirResource, destination, filter, start, limit)
    
    
    def NOTHING(self):
        return '''
        xsi:schemaLocation="%s 
      http://digir.sourceforge.net/schema/protocol/2003/1.0/digir.xsd 
      http://digir.net/schema/conceptual/darwin/2003/1.0 
      http://digir.sourceforge.net/schema/conceptual/darwin/2003/1.0/darwin2.xsd">
      '''
    
