# -*- coding: utf8 -*-
'''
CMF FULLBASE CLASS
*************************
***  CMF FULLBASE CLASS
*************************
$RCSfile: cmf_converter.py,v $
$Revision: 216 $
$Author: markus $
$Date: 2005-08-25 16:13:45 +0200 (Thu, 25 Aug 2005) $

Classes defining a full CMF and how to read/write it in different versions.
'''

import xml.sax
from xml.dom.minidom 				import Document
import xml.dom.ext

from biocase.wrapper.logging		import loggingClass
from biocase.tools.cmf_fullhandler 	import CMFhandler_1_4full, CMFhandler_2_0full
# inherited classes
from biocase.wrapper.cmf_base	 	import CMFClass as CMFBaseClass
from biocase.wrapper.cmf_element 	import CMFMappingClass, CMFAttributeClass as CMFAttributeBaseClass, CMFElementClass as CMFElementBaseClass




# ------------------------------------------------------------------------------------
class CMFAttributeClass(CMFAttributeBaseClass):
	'''Attribute class to represent all information of a CMF.
	Initialized with a dictionary.'''
	
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, CMFBaseObj, path2CMFElemObj={}, initDict={}, logObj=loggingClass(), parentObj=None):
		# init the inherited class
		CMFAttributeBaseClass.__init__(self, CMFBaseObj, path2CMFElemObj, initDict, logObj, parentObj)
		# the attributes fullname for displaying
		self.Annotation	= initDict.get('doc', None)

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def createCMFDOM_1_4(self, XMLwriter):
		'''Returns the DOM nodes for this CMF attribute and according to CMF version 1.4'''
		datanode = CMFAttributeBaseClass.createCMFDOM_1_4(self, XMLwriter)
		if self.Annotation <> None:
			docNode = XMLwriter.createNode('annot', self.Annotation)
			beforeDocNode = datanode.lastChild
			if beforeDocNode.tagName == 'dbAttribute':
				while beforeDocNode.tagName == 'dbAttribute':
					beforeDocNode = beforeDocNode.previousSibling
				beforeDocNode = beforeDocNode.nextSibling
				datanode.insertBefore(docNode, beforeDocNode)
			else:
				datanode.appendChild(docNode)
		return datanode

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def createCMFDOM_2_0(self, XMLwriter):
		'''Returns the DOM nodes for this CMF attribute and according to CMF version 1.4'''
		datanode = CMFAttributeBaseClass.createCMFDOM_2_0(self, XMLwriter)
		if self.Annotation <> None:
			XMLwriter.addNode(datanode, 'annot', self.Annotation)
		return datanode

# ------------------------------------------------------------------------------------
class CMFElementClass(CMFElementBaseClass, CMFAttributeClass):
	'''New dummy class inheriting the new attribute class.'''
	CMFAttributeClass = CMFAttributeClass
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, CMFBaseObj, path2CMFElemObj={}, initDict={}, logObj=loggingClass(), parentObj=None):
		CMFElementBaseClass.__init__(self, CMFBaseObj, path2CMFElemObj, initDict, logObj, parentObj)
		CMFAttributeClass.__init__(self, CMFBaseObj, path2CMFElemObj, initDict, logObj, parentObj)

# ------------------------------------------------------------------------------------
class CMFClass(CMFBaseClass):
	'''The base class for a full CMF object.'''
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, logObj=loggingClass()):
		CMFBaseClass.__init__(self, logObj)
		
