# -*- coding: utf8 -*-
'''
XML WRITER MODULE
*****************************
***  XML WRITER MODULE
*****************************
$RCSfile: xmlwriter.py,v $
$Revision: 9 $
$Author: markus $
$Date: 2005-07-04 11:47:00 +0200 (Mon, 04 Jul 2005) $

Module to ease the creation of XML minidom objects
'''


# ------------------------------------------------------------------------------------
class XMLwriterClass:
	'''Class to ease the creation of minidom objects'''
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, ns=None):
		from xml.dom.minidom import Document
		self.doc = Document()
		self.ns  = ns
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __del__(self):
		self.doc = None

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def setNamespace(self, ns):
		self.ns = ns

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def createNode(self, name, text=None, attributes={}):
		'''Creates a minidom node with optional text and attributes.'''
		node = self.doc.createElementNS(self.ns, name)
		if text <> None:
			text = unicode(text)
			textNode = self.doc.createTextNode(text)
			node.appendChild(textNode)
		if len(attributes) > 0:
			self.addOptionalAttributes(node, attributes)
		return node
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def addNode(self, parent, name, text=None, attributes={}):
		'''Creates a node and adds it to a parental node.'''
		node = self.createNode(name, text, attributes)
		parent.appendChild(node)
		return node
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def addOptionalNode(self, parent, name, text=None, attributes={}):
		'''Creates a node if a text value exists and adds it to a parental node.'''
		node = None
		if text <> None:
			node = self.createNode(name, text)
		for k,v in attributes.items():
			if k <> None and v <> None and len(unicode(v)) > 0:
				if node == None:
					node = self.createNode(name, None)
				node.setAttribute(k, unicode(v))
		if node <> None:
			parent.appendChild(node)
		return node

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def addOptionalAttributes(self, node, attributes={}):
		'''Adds optional attributes to a node. If the value is None the attribute will not be created.'''
		if len(attributes) > 0:
			for k,v in attributes.items():
				if k <> None and v <> None and len(unicode(v)) > 0:
					node.setAttribute(k, unicode(v))



# ------------------------------------------------------------------------------------
if __name__ ==  "__main__":
	import xml.dom.ext

	writer = XMLwriter("http://www.biocase.org/schemas/conceptmapping/1.4")
	conmap_node = writer.createNode("conceptMapping")
	writer.addNode(conmap_node, "schemaNamespace", "http://www.biocase.org")
	writer.addNode(conmap_node, "rootTableAlias", 'ROOT')
	for t in ['specimen','geo','persons']:
		writer.addNode(conmap_node, "staticTableAlias", t)
	for r in ['/doc/paragraph', '/doc/title']:
		writer.addNode(conmap_node, "recordIdentifier", r)
	
	print xml.dom.ext.PrettyPrint(conmap_node)