# -*- coding: utf8 -*-
'''
DB MODULE - DB2
*******************************
***  DB MODULE - DB2
*******************************
$RCSfile: dbmod_db2.py,v $
$Revision: 9 $
$Author: markus $
$Date: 2005-07-04 11:47:00 +0200 (Mon, 04 Jul 2005) $

The biocase.wrapper database module to access generic ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

import os
from string                             import join
from biocase.wrapper.errorclasses       import *
from biocase.wrapper.static_functions   import unique
from biocase.wrapper.typeconverter      import DBtransformerClass
from biocase.wrapper.sql.operators      import getWhereObjTables
from biocase.wrapper.sql.base           import sqlBase
from biocase.wrapper.sql.select         import sqlSelect


# uses inherited ANSI SQL
        

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(sqlSelect):
    __version__ = '0.7 using pyDB2 ?'
    __dbmod__   = u'DB2 DB module'
    
    def __repr__(self):
        return ("%s v%s" %(self.__class__.__dbmod__, self.__class__.__version__) )

    def __init__(self, psfObj, logObj):
        #
        # init the inherited class 
        # setting the logObj and psfObj references to self.log and self.psfObj
        # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
        sqlSelect.__init__(self, psfObj, logObj)
        #
        # connect to DB
        self.conn = self.createDBconnection()
        #
        # read DB 8bit encoding
        #
        logObj+(2,"8bit DB Encoding: %s"%(psfObj.encoding))
        self.conn.encoding = psfObj.encoding

    def __del__(self):
        # release db connection
        try:
            self.conn.close()
            self.log + "successfully closed DB2 db connection."
        except:
            self.log + "error when trying to close the DB2 db connection."
            pass

    def createDBconnection(self):
        try:
            import DB2
        except:
            self.log + "The pyDB2 module is not properly installed on your system. You need this module to connect to DB2 sources."
            raise
        self.__class__.__version__ = self.__class__.__version__[:-1]+DB2.VERSION
        #
        # connect
        #
        # import OS specific DB2 version:
        # 'posix', 'nt', 'dos', 'mac', 'os2', 'ce', 'java', 'riscos'
        if os.name in ("nt","dos"):
            import DB2  as DB2
        elif os.name in ("unix","posix","java"):
            import DB2  as DB2
        else:
            raise ConnectionError(os=os.name, log=self.log)
        loginParas = "dsn='%s',uid='%s',pwd='%s'" % (self.psfObj.datasource, self.psfObj.user ,self.psfObj.password )
        #conn = DB2.connect( loginParas )
        conn = DB2.connect(dsn='training',uid='db2admin',pwd='vodkolita5')
        #conn.stringformat = odbc.NATIVE_UNICODE_STRINGFORMAT
        return conn
            
    def execSelect(self, selectList, whereObj=None, recID=None):
        '''execute a select statement. the selectList gives the wanted attribute/alias tuples and the optional whereObj the WHERE filter.'''
        # get SQL string
        (sql, paras) = self.select(selectList, whereObj)
        # execute statement
        return self.execSQL(sql, paras)

    def execSQL(self, sql, paras):
        # close existing connection and create a new one!
        if self.conn is not None:
            self.conn.close()
        self.conn = self.createDBconnection()
        self.c = self.conn.cursor()
        # execute SQL statement and return result
        self.log.logPassedTime("start DB query")
        try:
            sql = sql.encode(self.psfObj.encoding)
            self.log+(1,"Executing SQL: '%s' with parameters: %s" %(sql, "'"+join([unicode(p) for p in paras], "', '")+"'"))
        except:
            self.c.close()
            raise SQLunicodeGenerationError()
        self.c.execute(sql, paras)
        tmp = self.c.fetchall()
        self.log.logPassedTime("end DB query")
        self.log+(1,"Hits: "+str(len(tmp)))
        if self.log.debuglevel > 1 or self.log.logstate > 1:
            try:
                if type(tmp) == type('a'):
                    rsString = unicode(tmp, self.psfObj.encoding)
                else:
                    rsString = unicode(tmp)
                self.log+(2,"Resulting Recordset:  "+rsString )
            except:
                self.log+(2,"The Recordset Result could not be converted to a string")
        # close cursor and connection!
        self.c.close()
        self.conn.close()
        return tmp
    
