# -*- coding: utf8 -*-
'''
DB MODULE - mySQL
*******************************
***  DB MODULE - mySQL
*******************************
$RCSfile: dbmod_mysql.py,v $
$Revision: 354 $
$Author: markus $
$Date: 2005-10-12 14:54:21 +0200 (Wed, 12 Oct 2005) $

The biocase.wrapper database module to access mySQL databases.
You will need the mxODBC module to connect to a MS SQL Server DB via ODBC.
'''

from string								import join
from biocase.wrapper.errorclasses 		import *
from biocase.wrapper.sql.base 			import sqlBase
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule

# uses inherited ANSI SQL.

		

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
	__version__ = '0.51 using MySQLdb ?'
	__dbmod__	= u'mySQL DB module'

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, psfObj, logObj):
		# init the inherited class 
		# setting the logObj and psfObj references to self.logObj and self.psfObj
		# call createDBconnection method
		DBBasemodule.__init__(self, psfObj, logObj)
		# dont use parameters but integrate all parameters into SQL strings:
		self.trans.integrateParametersInSQLString()
		# read DB 8bit encoding
		self.logObj+(2,"8bit DB Encoding: %s"%(psfObj.encoding))
		self.conn.encoding = psfObj.encoding
	

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def createDBconnection(self):
		try:
			import MySQLdb
		except:
			raise DbModuleMissingError('MySQLdb', log=self.logObj)
		self.__class__.__version__ = self.__class__.__version__[:-1] +MySQLdb.__version__
	
		try:
			loginParas = "Host=%s, DB=%s, User=%s, Passwd=%s" % (self.psfObj.dbIP, self.psfObj.datasource, self.psfObj.user ,self.psfObj.password )
			return MySQLdb.connect( host=self.psfObj.dbIP, db=self.psfObj.datasource, user=self.psfObj.user, passwd=self.psfObj.password )
		except:
			raise ConnectionError(log=self.logObj)
			

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def execSQL(self, sql, paras=None):
		# create cursor:
		self.c = self.conn.cursor()
		# execute SQL statement and return result
		self.logObj.logPassedTime("start DB query")
		try:
			sql = sql.encode(self.psfObj.encoding)
			self.logObj+(1,"Executing SQL: '%s'" %(sql))
		except:
			self.c.close()
			raise SQLunicodeGenerationError()
		# execute this SQL always before executing SQL statements:
		constantSQL = 'SET SQL_BIG_SELECTS = 1'
		self.c.execute(constantSQL)
		# execute dynamic SQL:
		self.c.execute(sql)
		tmp = self.c.fetchall()
		self.logObj.logPassedTime("end DB query")
		self.logObj+(1,"Hits: "+str(len(tmp)))
		if self.logObj.debuglevel > 1 or self.logObj.logstate > 1:
			try:
				if type(tmp) == type('a'):
					rsString = unicode(tmp, self.psfObj.encoding)
				else:
					rsString = unicode(tmp)
				self.logObj+(2,"Resulting Recordset:  "+rsString )
			except:
				self.logObj+(2,"The Recordset Result could not be converted to a string")
		self.c.close()
		return tmp
	
