# -*- coding: utf8 -*-
'''
DB MODULE - generic ODBC
*******************************
***  DB MODULE - generic ODBC
*******************************
$RCSfile: dbmod_odbc.py,v $
$Revision: 465 $
$Author: markus $
$Date: 2005-11-21 13:00:56 +0100 (Mon, 21 Nov 2005) $

The biocase.wrapper database module to access generic ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

import os
from string								import join
from biocase.wrapper.errorclasses 		import *
from biocase.tools.various_functions 	import unique, flatten, scalarpNoList
from biocase.wrapper.typeconverter		import DBtransformerClass
from biocase.wrapper.sql.operators		import getWhereObjTables, logicalOperator, compressOperator
from biocase.wrapper.sql.base 			import sqlBase
from biocase.wrapper.sql.select 		import sqlSelect
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule


# uses inherited ANSI SQL
sqlBase.templates['attribute'] = '"#1"."#2"'
sqlBase.templates['table'] = '"#1"'
sqlBase.templates['alias'] = '#1 #2'
sqlBase.templates['join'] = '#1 JOIN #2 ON #3'
sqlBase.templates['select-limit'] = None
		

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
	__version__ = '0.2 using mxODBC ?'
	__dbmod__	= u'4D ODBC DB module'
	

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, psfObj, logObj):
		# init the inherited class 
		# setting the logObj and psfObj references to self.logObj and self.psfObj
		# call createDBconnection method
		DBBasemodule.__init__(self, psfObj, logObj)
		#
		# read DB 8bit encoding
		#
		logObj+(2,"8bit DB Encoding: %s"%(psfObj.encoding))
		self.conn.encoding = psfObj.encoding

	# ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
	# --------------------------------------------------------------------------------
	def createDBconnection(self):
		try:
			import mx.ODBC
		except:
			raise DbModuleMissingError('mx_ODBC', log=self.logObj)			
		self.__class__.__version__ = self.__class__.__version__[:-1]+mx.ODBC.__version__
		#
		# connect
		#
		# import OS specific ODBC version:
		# 'posix', 'nt', 'dos', 'mac', 'os2', 'ce', 'java', 'riscos'
		if os.name in ("nt","dos"):
			import mx.ODBC.Windows	as odbc
		elif os.name in ("unix","posix","java"):
			import mx.ODBC.unixODBC	as odbc
		else:
			raise ConnectionError(os=os.name, log=self.logObj)
		loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.datasource, self.psfObj.user ,self.psfObj.password )
		conn = odbc.DriverConnect( loginParas )
		conn.stringformat = odbc.NATIVE_UNICODE_STRINGFORMAT
		return conn


	def fromClause(self, tableList):
		'''Returns the SQL string of the FROM clause for a given table(alias) list.
		Makes use of the tablegraph to produce an ordered list of tables
		and their relation - the relObj (table-alias names, whereObj).'''
		if len(tableList) > 1:
			TreeJoinLists = self.CMFObject.tableTreeObj.getJoinTableListOfLists(tableList)
			AdditionalWhereObjs = []
			fromClause = None
			for t, relObj in flatten(TreeJoinLists, scalarpNoList):
				if relObj is not None:
					#relObj.type = 'ora_left_join'
					AdditionalWhereObjs.append(relObj)
				if fromClause is None:
					fromClause = self.getTableSyntax(t)
				else:
					fromClause += ', '+ self.getTableSyntax(t)
			addWhereObj = compressOperator(logicalOperator('and', AdditionalWhereObjs))
			return (fromClause, addWhereObj)
		else:
			return (self.getTableSyntax(tableList[0]), None)	
