# -*- coding: utf8 -*-
'''
DB MODULE - generic ODBC
*******************************
***  DB MODULE - generic ODBC
*******************************
$RCSfile: dbmod_odbc_db2.py,v $
$Revision: 9 $
$Author: j.torre $
$Date: 2005-07-04 11:47:00 +0200 (Mon, 04 Jul 2005) $

The biocase.wrapper database module to access generic ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

import os
from string                             import join
from biocase.wrapper.errorclasses       import *
from biocase.tools.various_functions    import unique
from biocase.wrapper.typeconverter      import DBtransformerClass
from biocase.wrapper.sql.operators      import getWhereObjTables
from biocase.wrapper.sql.base           import sqlBase
from biocase.wrapper.sql.select         import sqlSelect
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule


# uses inherited ANSI SQL

#The only difference in DB2 with ANSI SQL is the way to limit the number
#of records, you must use the FETCH FIRST #3 ROWS ONLY clause
sqlBase.templates['select-limit'] = 'SELECT DISTINCT #1 FROM #2 FETCH FIRST #3 ROWS ONLY'
        

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
    __version__ = '0.81 using mxODBC ?'
    __dbmod__   = u'generic ODBC DB module'
    

    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def __init__(self, psfObj, logObj):
        # init the inherited class 
        # setting the logObj and psfObj references to self.logObj and self.psfObj
        # call createDBconnection method
        DBBasemodule.__init__(self, psfObj, logObj)
        #
        # read DB 8bit encoding
        #
        logObj+(2,"8bit DB Encoding: %s"%(psfObj.encoding))
        self.conn.encoding = psfObj.encoding

    # ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
    # --------------------------------------------------------------------------------
    def createDBconnection(self):
        try:
            import mx.ODBC
        except:
            raise DbModuleMissingError('mx_ODBC', log=self.logObj)          
        self.__class__.__version__ = self.__class__.__version__[:-1]+mx.ODBC.__version__
        #
        # connect
        #
        # import OS specific ODBC version:
        # 'posix', 'nt', 'dos', 'mac', 'os2', 'ce', 'java', 'riscos'
        if os.name in ("nt","dos"):
            import mx.ODBC.Windows  as odbc
        elif os.name in ("unix","posix","java"):
            import mx.ODBC.unixODBC as odbc
        else:
            raise ConnectionError(os=os.name, log=self.logObj)
        loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.datasource, self.psfObj.user ,self.psfObj.password )
        conn = odbc.DriverConnect( loginParas )
        conn.stringformat = odbc.NATIVE_UNICODE_STRINGFORMAT
        return conn

