# -*- coding: utf8 -*-
'''
DB MODULE - ODBC for TSQL
*******************************
***  DB MODULE - ODBC for TSQL
*******************************
$RCSfile: dbmod_odbc_tsql.py,v $
$Revision: 354 $
$Author: markus $
$Date: 2005-10-12 14:54:21 +0200 (Wed, 12 Oct 2005) $

The biocase.wrapper database module to access MS SQL Server databases.
You will need the mxODBC module to connect to a MS SQL Server DB via ODBC.
'''


import os
from string								import join
from biocase.wrapper.errorclasses 		import *
from biocase.wrapper.typeconverter		import DBtransformerClass
from biocase.wrapper.sql.base 			import sqlBase
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule

from mx.DateTime import now, ISO
_DateTime = now()



#############################################
#
# CUSTOMIZATION for SQL Server
#
# OVERLOAD BASE CLASSES AND OTHER METHODS
#
#############################################

# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'
sqlBase.templates['select-limit'] = 'SELECT DISTINCT TOP #3 #1 FROM #2'

# override standard type-transformation from module dbtypes:
def makeDate(self, x):
	# convert from date-string to python timestamp
	# http://www.w3.org/TR/xmlschema-2/#date
	# Gregorian calendar dates as defined in Â§ 5.2.1 of [ISO 8601].
	# YYYY-MM-DD
	# also allows date input with .,: instead of -
	if type(x) == type(_DateTime):
		# mxDateTime object
		x = x.ticks()
	elif type(x) in (type('k'), type(u'Ã¤')):
		x = x.strip()
		tmp = x
		# ignore time
		if len(x) > 10:
			x = x[:10]
		# parse string
		try:
			d = ISO.ParseDate(x)
			x = d.ticks()
			self.logObj + (1, "Converted date-string %s into timestamp %s" %(tmp ,str(x)))
		except:
			raise wrongTypeForDBError(x, "date")
			x = None
	return x
DBtransformerClass.makeDate = makeDate

		
		

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
	__version__ = '1.01 using mxODBC ?'
	__dbmod__	= u'MS SQL Server module'
	
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, psfObj, logObj):
		# init the inherited class 
		# setting the logObj and psfObj references to self.logObj and self.psfObj
		# call createDBconnection method
		DBBasemodule.__init__(self, psfObj, logObj)
		#
		# read DB 8bit encoding
		#
		logObj+(2,"8bit DB Encoding: %s"%(psfObj.encoding))
		self.conn.encoding = psfObj.encoding

	# ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
	# --------------------------------------------------------------------------------
	def createDBconnection(self):
		try:
			import mx.ODBC
		except:
			raise DbModuleMissingError('mx_ODBC', log=self.logObj)
		self.__class__.__version__ = self.__class__.__version__[:-1]+mx.ODBC.__version__
		#
		# connect
		#
		# import OS specific ODBC version:
		# 'posix', 'nt', 'dos', 'mac', 'os2', 'ce', 'java', 'riscos'
		if os.name in ("nt","dos"):
			import mx.ODBC.Windows	as odbc
		elif os.name in ("unix","posix","java"):
			import mx.ODBC.unixODBC as odbc
		else:
			raise ConnectionError(os=os.name, log=self.logObj)
		loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.datasource, self.psfObj.user ,self.psfObj.password )
		conn = odbc.DriverConnect( loginParas )
		conn.stringformat = odbc.NATIVE_UNICODE_STRINGFORMAT
		return conn 

