# -*- coding: utf8 -*-
'''
DB MODULE - SYBASE
*******************************
***  DB MODULE - SYBASE
*******************************
$RCSfile: dbmod_sybase.py,v $
$Revision: 354 $
$Author: markus $
$Date: 2005-10-12 14:54:21 +0200 (Wed, 12 Oct 2005) $

The biocase.wrapper database module to access Sybase databases.
You will need the Sybase module from http://www.object-craft.com.au/projects/sybase/ to connect.
'''

from string								import join
from biocase.wrapper.errorclasses 		import *
from biocase.wrapper.typeconverter		import DBtransformerClass
from biocase.wrapper.sql.base 			import sqlBase
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule
from types 								import *

from mx.DateTime import now, ISO
_DateTime = now()



#############################################
#
# CUSTOMIZATION for Sybase T-SQL
#
# OVERLOAD BASE CLASSES AND OTHER METHODS
#
#############################################


# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'

# override standard type-transformation from module dbtypes:
def makeDate(self, x):
	# convert from date-string to python timestamp
	# http://www.w3.org/TR/xmlschema-2/#date
	# Gregorian calendar dates as defined in Ã‚Â§ 5.2.1 of [ISO 8601].
	# YYYY-MM-DD
	# also allows date input with .,: instead of -
	if type(x) == type(_DateTime):
		# mxDateTime object
		x = x.ticks()
	elif type(x) in (StringType, UnicodeType):
		x = x.strip()
		tmp = x
		# ignore time
		if len(x) > 10:
			x = x[:10]
		# parse string
		try:
			d = ISO.ParseDate(x)
			x = d.ticks()
			self.logObj + (1, "Converted date-string %s into timestamp %s" %(tmp ,str(x)))
		except:
			raise wrongTypeForDBError(x, "date")
			x = None
	return x
DBtransformerClass.makeDate = makeDate

			
		

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
	__version__ = '0.31 using Sybase ?'
	__dbmod__	= u'Sybase module'
	
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, psfObj, logObj):
		# init the inherited class 
		# setting the logObj and psfObj references to self.logObj and self.psfObj
		# call createDBconnection method
		DBBasemodule.__init__(self, psfObj, logObj)

		# dont use parameters but integrate all parameters into SQL strings:
		self.trans.integrateParametersInSQLString()


	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def createDBconnection(self):
		try:
			import Sybase
		except:
			raise DbModuleMissingError('Sybase', log=self.logObj)
		self.__class__.__version__ = self.__class__.__version__[:-1] +Sybase.__version__
		#
		# connect
		#
		conn = Sybase.connect(self.psfObj.dbIP, self.psfObj.user, self.psfObj.password, self.psfObj.datasource)
		return conn 
		
	
