# -*- coding: utf8 -*-
'''
ERROR CLASSES
**********************
***  ERROR CLASSES
**********************
$RCSfile: errorclasses.py,v $
$Revision: 569 $
$Author: markus $
$Date: 2006-03-09 16:10:04 +0100 (Thu, 09 Mar 2006) $

Custom errors and their logging message
'''

# --------------------------------------------------------------------------------
# SIMPLE LOG CLASS
# --------------------------------------------------------------------------------
class loggingClass:
	'''This is a simple dummy logging class.
	It serves as a demonstration how to implement your own class.
	'''
	LOG_TYPES = ['debug','info','warn','error','fatal']
	LOG_LEVEL = {0:'critical', 1:'performance', 2:'simple debug', 3:'detailed debug'}
	
	def __init__(self):
		# logging list.
		self.logObj = [] 

	def __add__(self, logEntry):
		'''operator overload to add a logging message to the instance.'''
		self.logObj.append(logEntry)
	
	def __repr__(self):
		'''returns logs according to the debugging level as a string'''
		logging = ''
		for entry in self.logObj:
			logging += "\n"
			logging += unicode(entry)
		return logging
		



# --------------------------------------------------------------------------------
# SPECIFIC ERROR CLASSES WITH LOGGING ENABLED
# --------------------------------------------------------------------------------

# --------------------------------------------------------------------------------
# EXAMPLE
# --------------------------------------------------------------------------------
#	LOG_LEVEL = {0:'critical', 1:'performance', 2:'simple debug', 3:'detailed debug'}
#   ERROR_CODE see http://ww3.bgbm.org/protocolwiki/ProtocolFeatures_2fReturnedErrorCodes

class SampleError(Exception):
	"""Explanation of error."""
	def __init__(self, parameter1, parameterN, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'LOG_LEVEL, ERROR_CODE', 'msg':'Text description of error with para1=%s and para2=%s' %(parameter1, parameterN)}
# in the code it looks like this:
#  -> raise SampleError(para1, para2)


# EXAMPLE FROM REAL CODE:
# -> raise "Unknown protocol: "+protocol
# should become:
# -> raise UnknownProtocolError(protocol, log=self.logObj)

# in this errorclasses file we then need:
# class UnknownProtocolError(Exception):
#	"""Error raised because of an unknown protocol."""
#	def __init__(self, protocol, log=loggingClass()):
#		log + (0, REQ_UNKNOWN_PROTOCOL, 'The requested protocol "%s" is unknown to the wrapper' %(protocol))

#-----------------------CODE ADDED BY JAVI--------------------------
class UnknownProtocolError(Exception):
	"""Error raised because of an unknown protocol."""
	def __init__(self, protocol, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_UNKNOWN_PROTOCOL', 'msg':'The requested protocol "%s" is unknown to the wrapper' %(protocol)}
		

class NotPossibleToCreateDatasourceError(Exception):
	"""Can't set datasource name. No PSF object created yet!"""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_PSF_NOT_POSSIBLE_SET_DATASOURCE', 'msg':'Cant set datasource name. No PSF object created yet!'}

class PSFmissingError(Exception):
	"""No PSF object created yet!"""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_PSF_MISSING', 'msg':'No PSF object created yet!'}

class UnknownCMFVersionError(Exception):
	"""Unknown CMF version. Unable to parse it. No CMF object loaded."""
	def __init__(self, cmf_version, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_CMF_UNKNOWN_VERSION', 'msg':'Unknown CMF version. Unable to parse it. No CMF object loaded. Version:%s ' %(cmf_version) }

class MappingObjectTypeNotSupportError(Exception):
	"""A mapping object can only contain strings or dbattribute objects as items. Item should be a string or DBAttribute object."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_CMF_OBJ_NOT_SUPPORTED', 'msg':'Error in the CMF. Mapping object can only contain strings or dbattribute objects as items.' }	

class DbModuleNotSpecifiedError(Exception):
	"""Database module not specified"""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_DB_MODULE_NOT_SPECIFIED', 'msg':'No database module has been specified.'}

class DbModuleMissingError(Exception):
	"""Database module missing"""
	def __init__(self, db_module, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_DB_MODULE_MISSING', 'msg':'The database module %s is missing.' %(db_module)}

class DbOperationNotSupportedError(Exception):
	"""An operation, like alias or outer joins, are not supported by the database"""
	def __init__(self, operation, db_module, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_OPERATION_NOT_SUPPORTED', 'msg':'The module/database %s does not support the next operation: %s' %(operation, db_module)}		

class CMFwrongDefinition(Exception):
	"""Something is wrong defined in the CMF file"""
	def __init__(self, element, value, error_desc, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_CMF_WRONG_DEF', 'msg':'There is a wrong definition a the %s element with value %s. Error: %s' %(element, value, error_desc)}	
			
#-----------------------------------------------------------------------

class DBComplexError(Exception):
	"""temp error for too complex DBs not yet supported."""
	def __init__(self, name, table, attr, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_TOO_COMPLEX_DB', 'msg':'Too complex DBs. Multiple tables linked to one repNode: %s.%s of element %s' %(attr,table,name)}

class CGIError(Exception):
	"""Exception raised for errors encountered while retrieving the CGI request XML document."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_NOT_REQUEST_SPECIFIED', 'msg':'No request found. Search aborted.'}

class CMFError(Exception):
	"""Exception raised for errors trying to parse a CMF file."""
	def __init__(self, cmf='?', log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_CMF_FILE_CORRUPTED', 'msg':'The CMF DB mapping file %s was corrupt or could not be interpreted.'%cmf}

class CMFVersionError(Exception):
	"""Exception raised for errors trying to deal with unknown versions of CMF."""
	def __init__(self, version, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_CMF_VERSION_UNKNOWN', 'msg':('The requested CMF version "%s" is unknown.'%version)}

class CMFMaxAttributeError(Exception):
	"""Exception raised for wrong CMF files, that have more DB attributes mapped to a non or too little repeatable element."""
	def __init__(self, xpath, no_is, no_max, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_CMF_MAX_ATTRIBUTE', 'msg':('In your CMF file the element %s was assigned %s DB attributes, although its maximum repeatability is %s.'%(xpath, str(no_is), str(no_max)))}

class ConnectionError(Exception):
	"""Exception raised for errors while establishing a DB connection."""
	def __init__(self, log=loggingClass(), os=None, dbms=None):
		log + ("No DB connection could be established. Please verify DB state and connection parameters in your provider_setup_file.xml")			
		if os!=None:
			log + {'type':3, 'level':0, 'code':' PRG_WRONG_OS', 'msg':("The Providers Operating System " +os +" is not yet supported.")}
		if dbms!=None:
			log + {'type':3, 'level':0, 'code':'PRG_WRONG_DBMS', 'msg':("The Providers DBMS " +dbms +" is not yet supported or it's database drivers are missing.")}			

class CouldNotPickleError(Exception):
	"""Could not pickle a file."""
	def __init__(self):
		pass

class MultipleCMFsWithSameNamespaceError(Exception):
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_MULTIPLE_CMF_NS', 'msg':'Multiple CMFs with the same namespace are found in your datasource.'}

class DBAttrObjError(Exception):
	"""An expected DB-tuple was not found."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_MISSING_DB_TUPLE', 'msg':'An expected DB-tuple was not found.'}

class MissingRootTableError(Exception):
	"""Exception raised for errors trying to parse a CMF file."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'PRG_CMF_MISSING_ROOT_TABLE', 'msg':'The CMF DB mapping file does not contain a root table.'}

class NoFilterError(Exception):
	"""Exception raised for empty request filters."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_NO_FILTER', 'msg':"No valid filter was given or all filter xpath expressions are unknown to this provider."}

class ProviderSetupParseError(Exception):
	"""Exception raised for errors encountered while parsing the ProviderSetup.xml file."""
	def __init__(self, filename, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'PRG_PSF_PARSING_ERROR', 'msg':('The Provider Configuration could not be parsed. Please check if "%s" is accesible and a valid XML document.' %(filename))}

class RequestParseError(Exception):
	"""Exception raised for errors encountered while parsing the CGI request."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_PARSING_ERROR', 'msg':'The request could not be parsed. Please check if it is a valid XML document.'}

class RequestedSchemaError(Exception):
	"""Exception raised for requested searches against schemas not supported by the provider."""
	def __init__(self, type, schema, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_UNKOWN_SCHEMA', 'msg':('The requested ' +type +' schema "' +schema +'" is not supported by this provider! Please do a capabilities request to see all supported schemas')}

class RequestUnknownTypeError(Exception):
	"""Exception raised when encountering an unknown request type."""
	def __init__(self, type='unknown', log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_UNKNOWN_TYPE', 'msg':("The request type '%s' is unknown. Fatal error." %type)}

class RequestUnknownConceptError(Exception):
	"""Exception raised for requested searches referencing concepts not supported by the provider."""
	def __init__(self, concept, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_UNKNOWN_SEARCH_CONCEPT', 'msg':('The requested concept %s is unknown to this provider! Please do a capabilities request to see all supported concepts' %(concept))}

class RequestNotSearchableConceptError(Exception):
	"""Exception raised for requested searches referencing concepts that are not searchable."""
	def __init__(self, concept, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'REQ_NOT_SEARCHABLE_CONCEPT', 'msg':('The requested concept %s is not searchable for this provider! Please do a capabilities request to see all searchable concepts' %(concept))}

class ResponseError(Exception):
	"""Exception raised for errors encountered while creating the response object."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'RSP_NOT_DEFINED_ERROR_WHILE_PROCESSING', 'msg':'The response object could not be properly established.'}

class SQLError(Exception):
	"""Exception raised for DB errors trying executing a SQL statement."""
	def __init__(self, sql, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_SQL_EXEC', 'msg':'The last SQL statement produced a DB error.'}

class SQLgenerationError(Exception):
	"""Exception raised for errors encountered while generating a SQL string."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_SQL_GENERATING', 'msg':'The SQL string to query the DB could not be generated because an error occured.'}

class SQLunicodeGenerationError(Exception):
	"""Exception raised for non ANSI characters used in building a SQL string."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'DBS_NON_ASCII_CHAR_IN_COLUMNS', 'msg':'The request, some tables or columns in your database contain non 7bit ASCII characters. Your DB driver cannot handle this !'}

class StopSAX(Exception):
	"""Exception raised to stop a SAX parser from parsing any further. No error but intention."""
	def __init__(self):
		pass

class TableConfigError(Exception):
	"""Exception raised for any table configuration errors."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_PSF_TABLEALIAS_ERROR', 'msg':'The PSF file contains tablealias declaration errors.'}

class TableConfigAliasError(Exception):
	"""Exception raised for Table configurations that have a non unique alias name."""
	def __init__(self, alias, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_PSF_NON_UNIQUE_TABLEALIAS', 'msg': ('The PSF configuration contains the non unique table-alias "'+alias +'". This is not allowed.')}

class TableConfigCircleError(Exception):
	"""Exception raised for Table configurations that have a circular structure."""
	def __init__(self,table, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_PSF_CIRCULAR_TABLES', 'msg': ('The PSF table configuration contains circular structures with table '+table+'. This is not allowed.')}

class UnicodeError(Exception):
	"""Exception raised because of unsupported Unicode funcionality."""
	def __init__(self, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'PRG_UNICODE_NOT_SUPPORTED', 'msg': 'The wrapper can not handle Unicode. Please wait for an improved version.'}

class unknownEncodingError(Exception):
	"""Exception raised for errors of unknown encoding entries in the providerConfig file."""
	def __init__(self, encoding, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'CFG_PSF_UNKNOWN_ENCODING', 'msg': ('Your Python installation does not your configured text-encoding "%s". Please check your providersetup.xml file.' %(encoding) )}

class unknownTypeError(Exception):
	"""Exception raised for errors of unknown type parameters in SQL strings or DB results."""
	def __init__(self, value, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'PRG_UNKNOWN_TYPE_PARAMS', 'msg': ("The wrapper cannot deal with parameter of type %s." %(type(value)))}

class wrongXMLTypeError(Exception):
	"""Exception raised for values of wrong type as specified in the XML schema."""
	def __init__(self, value, XMLtype, log=loggingClass()):
		log + {'type':3, 'level':0, 'code':'RSP_DB_XML_TYPE_CONVERSION_ERROR', 'msg': ("Unable to convert the DB value '%s' into XML type %s" %(value, XMLtype))}

class wrongTypeForDBError(Exception):

	"""Exception raised for values that cannot be converted to a specified database type."""
	def __init__(self, value, DBtype, log=loggingClass()):
		try:
			log + {'type':3, 'level':0, 'code':'RSP_XML_DB_TYPE_CONVERSION_ERROR', 'msg': ("Unable to convert the value '%s' into DB type %s" %(value, DBtype))}
		except:
			log + {'type':3, 'level':0, 'code':'RSP_XML_DB_TYPE_CONVERSION_ERROR', 'msg': ("Unable to convert a value into DB type %s" %(DBtype))}
