# -*- coding: utf8 -*-
'''
**********************************************
*	   P R O T O C O L - C L A S S
*	   for the biocase protocol
**********************************************
$RCSfile: biocase_protocol.py,v $
$Revision: 218 $
$Author: markus $
$Date: 2005-08-25 18:24:18 +0200 (Thu, 25 Aug 2005) $

Request and Response classes to support the biocase protocol.
'''

import types
from biocase.wrapper.errorclasses		 import *
from biocase.wrapper.dbmod.base_dbmod    import DBBasemodule
from biocase.wrapper.protocol.base_protocol import ProtocolBaseClass
from biocase.wrapper.protocol.BioCASe.biocase_request  import RequestClass
from biocase.wrapper.protocol.BioCASe.biocase_response import ResponseClass
from biocase.wrapper.psf import ProviderSetup



# ------------------------------------------------------------------------------------
class ProtocolClass(ProtocolBaseClass):
	'''The main protocol class accepting a standard API and managing requests and responses.'''
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, logObj):
		logObj + (1, "BioCASe protocol used.")
		ProtocolBaseClass.__init__(self, logObj)
		self.responseObj = ResponseClass(logObj)

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def setRequest(self, args):
		'''Get request args from parameter or CGI and create & process a request object.'''
		ProtocolBaseClass.setRequest(self, RequestClass, args)
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def doRequest(self):
		'''Starts to process the request and prepares output object.'''
		# test if all objects are existing
		if self.psfObj is None:
			self.logObj + 'PyWrapper instance is missing its configurations!'
		elif self.requestObj is None:
			self.logObj + 'PyWrapper instance is missing a request!'
		else:
			# ***************
			# process request
			# ***************
			try:
				# capabilities
				if self.requestObj.type == "capabilities":
					self.responseObj.setCapabilitiesContent()
					# log time to check for performance bottlenecks
					self.logObj.logPassedTime("get capabilities")
				# purely static CMFs
				elif self.requestObj.type == "static":
					# create CMF object of the response-CMF for output of XML
					try:
						# build new CMF object
						CMFObject = CMFClass(self.logObj)
						CMFObject.loadCMFdata(self.psfObj.getCMFfilename(self.requestObj.resSchema), psfObj=self.psfObj)
					except CouldNotPickleError:
						pass
					except:
						raise CMFError(log=self.logObj, cmf=self.requestObj.resSchema)
					xmldom = CMFObject.createXMLResultDOM( [[]] )
					self.responseObj.setSearchContent(xmldom, self.resultStatus)
				# search or scan
				elif self.requestObj.type in ("search", "scan"):
		
					# chose DBmodule, connect to DB.
					from biocase.wrapper.operations import getOperationsObjectForDBMod
					operationObj = getOperationsObjectForDBMod(self.psfObj, self.logObj, self.responseObj)
					# log time to check for performance bottlenecks
					self.logObj.logPassedTime("init operations")
					
					# remember dbmod in response object for protocol header generation later on
					self.responseObj.dbmodVersion = DBBasemodule.__repr__(operationObj)
		
					# search or scan
					if self.requestObj.type == "scan":
						#scan
						scanList = operationObj.scan(self.requestObj)
						# log time to check for performance bottlenecks
						self.logObj.logPassedTime("do scan")
		
						# copy list to response object
						self.responseObj.setScanContent(scanList)
						# log number of results
						self.logObj.setResponseStatus(len(scanList))
					else:
						# SEARCH
						# check if requested schemas are supported for output
						self.requestObj.areSchemasSupported()
		
						# check whether full result or hit-count is requested
						if self.requestObj.count:
							# count hits (integer)
							x = operationObj.count(self.requestObj)
							# log time to check for performance bottlenecks
							self.logObj.logPassedTime("do count")
							# copy count to the response object
							self.responseObj.setCountContent(x)
							# log number of results
							self.logObj.setResponseStatus(0,0,x)
						else:
							self.logObj.logPassedTime("start of search request incl DOM creation")
							(self.resultDOM, self.resultStatus) = operationObj.search(self.requestObj)
							# log time to check for performance bottlenecks
							self.logObj.logPassedTime("end of search request incl DOM creation")
							# copy result to the response object
							self.responseObj.setSearchContent(self.resultDOM, self.resultStatus)
							# log number of results
							self.logObj.setResponseStatus(self.resultStatus.count, self.resultStatus.drop, self.resultStatus.total)
				else:
					self.logObj + ("Unknown Request-type : " + str(self.requestObj.type))
			except:
				self.logObj + {'type':4, 'msg':'Request processing aborted due to fatal error !'}
	

		
		