# -*- coding: utf8 -*-
'''
**********************************************
*	   P R O T O C O L - B A S E - C L A S S
*	   for any protocol
**********************************************
$RCSfile: base_protocol.py,v $
$Revision: 9 $
$Author: markus $
$Date: 2005-07-04 11:47:00 +0200 (Mon, 04 Jul 2005) $

'''

import types
from biocase.wrapper.errorclasses		 import *
from biocase.wrapper.psf import ProviderSetup



# ------------------------------------------------------------------------------------
class ProtocolBaseClass:
	'''The main protocol class accepting a standard API and managing requests and responses.'''
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, logObj):
		self.logObj = logObj
		self.psfObj = None
		self.requestObj  = None
		self.responseObj = None
	
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def setPsfObj(self, psfObj):
		'''Update self with psf object reference.'''
		self.psfObj     = psfObj
		self.responseObj.setPsfObj(psfObj)
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def setRequest(self, RequestClass, args):
		'''Get request args from parameter or CGI and create & process a request object.
		Pass the specific request class to be used with the method.'''
		self.requestObj = RequestClass(self.logObj, self.psfObj)
		self.requestObj.setRequest(args)
		self.responseObj.setRequestObj(self.requestObj)
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def printResponseString(self, stream, encoding, httpHeader):
		'''Process request and print out the response into the given stream.'''
		self.doRequest()
		self.responseObj.printResponseString(stream, encoding, httpHeader)

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def doRequest(self):
		'''Override this to implement the operational logic to process a request.'''
		raise "No real protocol class has been used. This is only the empty base class which cannot process real requests."
	
	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def writeLogs(self):
		'''write error log and request log'''
		self.responseObj.writeLogs()

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def _isPsfObjExisting(self):
		'''Checks if the PSF configuration has been read and returns 1 for true.'''
		if self.psfObj is None:
			return 0
		elif self.psfObj.__class__ == ProviderSetup:
			return 1
		else:
			return 0
		
		
		