# -*- coding: utf8 -*-
'''
**********************************************
*	   P R O T O C O L - C L A S S
*	   for the biocase protocol
**********************************************
$RCSfile: base_response.py,v $
$Revision: 218 $
$Author: markus $
$Date: 2005-08-25 18:24:18 +0200 (Thu, 25 Aug 2005) $

Request and Response classes to support the biocase protocol.
'''

import types, sys, os
import xml.dom.minidom, xml.dom.ext
from mx.DateTime 			import now
from xml.sax							import parse, parseString
from cgi								 import FieldStorage
from os								 import environ
from os.path							 import isfile

import biocase.wrapper
from biocase.tools.various_functions 		import isTrue
from biocase.wrapper.typeconverter	import convertDateTime2ISOString
from biocase.wrapper.operations     import recordStatus
from biocase.wrapper.errorclasses		 import *
from biocase.wrapper.protocol.BioCASe.biocase_handler	 import requestHandler
from biocase.wrapper.protocol.base_request	 import RequestBaseClass
from biocase.wrapper.sql.operators		import compressOperator

	


# ------------------------------------------------------------------------------------
class ResponseBaseClass:
	'''Response class implementing the essential methods used by the base protocol class'''

	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def __init__(self, logObj):
		# store logging object
		self.logObj = logObj
		# the protocol xml document
		self.doc = xml.dom.minidom.Document()   # complete response xml document.
		# copyright comment
		comment = self.doc.createComment("XML generated by BioCASE PyWrapper software version " +biocase.wrapper.__version__ +" written by Markus Doering, Botanic Garden and Botanical Museum, Berlin.")
		self.doc.appendChild(comment)
		#
		# properties being set later by their set methods
		#
		# the PSF config object
		self.psfObj = None
		# store reference to request object
		self.requestObj = None
		# content sub-tree
		self.content = None
		# the used dbmod version
		self.dbmodVersion = None
		#
		# initialize response doc
		#
		self.init()
		
	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def init(self):
		raise "This is the response base class. The init method should be overriden by the protocol specific response class."

	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def setPsfObj(self, psfObj):
		'''Adds a psf object to the protocol object.'''
		# store reference to psf object
		self.psfObj = psfObj

	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def setRequestObj(self, requestObj):
		'''Adds a request object to the protocol object.'''
		# store reference to request object
		self.requestObj = requestObj

	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def getProtocolDocument(self):
		'''Method to override that returns the full protocol document as a DOM object.
		Used by method "printResponseStructure" to return a string.'''
		# return protocol document as DOM
		return self.doc
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def printResponseString(self, stream=sys.stdout, encoding='UTF-8', httpHeader=1):
		'''Returns a response as a string inserted into a stream.
		If parameter "stream" is None, the DOM object is returned. Otherwise the response is printed into the stream.
		The parameter "httpHeader" is used to include a http header.
		'''
		if httpHeader > 0:
			stream.write('Content-Type: text/xml\n\n') # Blank line marking end of HTTP headers
		# print / return document
		xml.dom.ext.PrettyPrint(self.getProtocolDocument(), stream=stream, encoding=encoding)
		self.writeLogs()		

	# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	def writeLogs(self):
		'''write error log and request log'''
		# write logfile
		try:
			self.logObj.writeLogFile()
		except:
			self.logObj + "Error writing log file!"
	




