# -*- coding: utf8 -*-
'''
**********************************************
*	   P R O T O C O L - C L A S S
*	   for the species2000 protocol
**********************************************
$RCSfile: sp2k_protocol.py,v $
$Revision: 33 $
$Author: markus $
$Date: 2005-07-05 16:17:54 +0200 (Tue, 05 Jul 2005) $

The species2000 protocol uses no XML for querying but CGI parameters.
The response is a fixed schema knwon as the Common Data Model (CDM).
This is built into the pywrapper via its namespaces:
http://www.sp2000.org/CDMSchema1.3
'''

import types
from biocase.wrapper.errorclasses		 import *
from biocase.wrapper.protocol.base_protocol import ProtocolBaseClass
from biocase.wrapper.protocol.species2000.sp2k_request  import RequestClass
from biocase.wrapper.protocol.species2000.sp2k_response import ResponseClass
from biocase.wrapper.psf import ProviderSetup
from biocase.wrapper.cmf_base		import CMFClass

	
# ------------------------------------------------------------------------------------
class ProtocolClass(ProtocolBaseClass):
	'''The main protocol class accepting a standard API and managing requests and responses.'''
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, logObj):
		logObj + (1, "Species2000 SPICE protocol used.")
		ProtocolBaseClass.__init__(self, logObj)
		self.responseObj = ResponseClass(logObj)
	
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def setRequest(self, args):
		'''Get request args from parameter or CGI and create & process a request object.'''
		ProtocolBaseClass.setRequest(self, RequestClass, args)
		
	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def doRequest(self):
		'''Starts to process the request and prepares output object.'''
		# test if all objects are existing
		if self.psfObj is None:
			self.logObj + 'PyWrapper instance is missing its configurations!'
		elif self.requestObj is None:
			self.logObj + 'PyWrapper instance is missing a request!'
		else:
			# ***************
			# process request
			# ***************
			try:
				# purely static CMFs
				if self.requestObj.type == "static":
					# create CMF object of the response-CMF for output of XML
					try:
						# build new CMF object
						CMFObject = CMFClass(self.logObj)
						CMFObject.loadCMFdata(self.psfObj.getCMFfilename(self.requestObj.resSchema), psfObj=self.psfObj)
						# cut off the unused parts:
						for path in self.requestObj.removeCMFSubtreePaths:
							CMFObject.removeSubtree(path, self.psfObj)
					except CouldNotPickleError:
						pass
					except:
						raise CMFError(log=self.logObj, cmf=self.requestObj.resSchema)
					# self.logObj + unicode(CMFObject.fullRepr() )
					xmldom = CMFObject.createXMLResultDOM( [[]] )
					self.responseObj.setSearchContent(xmldom)
				# search
				elif self.requestObj.type == "search":
					# chose DBmodule, connect to DB.
					from biocase.wrapper.operations import getOperationsObjectForDBMod
					operationObj = getOperationsObjectForDBMod(self.psfObj, self.logObj, self.responseObj, self.requestObj.removeCMFSubtreePaths, self.requestObj.rtype)
					# log time to check for performance bottlenecks
					self.logObj.logPassedTime("init operations")
					# SEARCH
					# check if requested schemas are supported for output
					self.requestObj.areSchemasSupported()
					self.logObj.logPassedTime("start of search request incl DOM creation")
					(self.resultDOM, self.resultStatus) = operationObj.search(self.requestObj)
					# log time to check for performance bottlenecks
					self.logObj.logPassedTime("end of search request incl DOM creation")
					# copy result to the response object
					self.responseObj.setSearchContent(self.resultDOM, self.resultStatus)
					# log number of results
					self.logObj.setResponseStatus(self.resultStatus.count, self.resultStatus.drop, self.resultStatus.total)
				else:
					self.logObj + ("Unknown Request-type : " + str(self.requestObj.type))
			except:
				self.logObj + {'type':4, 'msg':'Request processing aborted due to fatal error !'}
	

		
		