#!/usr/bin/env python

# ****************************************
# +++  BioCASE Provider Software v2
# +++  Installer script
# $Id: setup.py 665 2006-04-07 13:37:43Z markus $
# ****************************************


import sys, os, string, shutil, exceptions
from types import *

# ***** include the biocase.lib directory in the python sys path for importing *****
execfile( os.path.abspath( os.path.join( os.path.dirname( __file__ ), 'lib', 'biocase', 'adjustpath.py' ) ))

from biocase.configuration  import Cfg, setCfgClassLocators, writeConfiguration, readConfiguration, getBPSversion

# suppress annoying future warnings for python 2.4
import warnings
warnings.filterwarnings('ignore', category=FutureWarning)
 
class DevNull:
    '''/dev/null file object discarding all write data.'''
    def __init__(self):
        pass
    def write(self, x):
        pass
    
############################################################################################################
#
# FUNCTIONS
#
#===========================================================================================================
def askForParameter(question, boolean=False, selectList=None, default=None, isFile=False, isDir=False):
    '''Ask user for a parameter.'''
    answer = default
    if boolean:
        # only ask for yes or no
        defaultstr = ""
        if default is True: 
            defaultstr = " [yes]"
        elif default is False:
            default = False
            defaultstr = " [no]"
        answer = raw_input("%s%s " %(question, defaultstr))
        if answer is None or len(answer) == 0:
            answer = default
        while not answer in (True,False,"yes","y","no","n"):
            print " Please choose yes or no."
            answer = raw_input("%s%s " %(question, defaultstr))
            if answer is None or len(answer) == 0:
                answer = default
        if answer in ('yes','y',True):
            answer = True
        else:
            answer= False
    elif type(selectList) == ListType and len(selectList) > 0:
        i = 0
        selectDict = {}
        for option in selectList:
            selectDict[i] = option
            if default == option:
                default = i
            question += "\n  (%i) - %s" %(i,option)
        # ask question
        defaultstr = ""
        if default is not None:
            answer = raw_input("%s\nPlease select your option: [%s] " %(question, default))
            if answer is None or len(answer) == 0:
                answer = default
            while not selectDict.has_key(answer):
                print " Please select a valid option."
                answer = raw_input("%s\nPlease select your option: [%s] " %(question, default))
                if answer is None or len(answer) == 0:
                    answer = default
        else:
            answer = raw_input("%s\nPlease select your option: " %(question))
            while answer is None or not selectDict.has_key(answer):
                print " Please select a valid option."
                answer = raw_input("%s\nPlease select your option: " %(question))
        answer = selectDict[answer]
    else:
        defaultstr = ""
        if default is not None:
            answer = raw_input("%s [%s] " %(question, default))
            if answer is None or len(answer) == 0:
                answer = default
        else:
            answer = raw_input("%s " %(question))
            while answer is None or len(answer) == 0:
                print "Please enter something."
                answer = raw_input("%s " %(question))
        if isFile:
            # ensure its a file
            while not os.path.isfile(answer):
                print " The file does not exist. Please enter a valid filepath."
                answer = raw_input("%s%s " %(question, defaultstr))
        elif isDir:
            # ensure its a directory
            while not os.path.isdir(answer):
                print " The directory does not exist. Please enter a valid dirpath."
                answer = raw_input("%s%s " %(question, defaultstr))
    print " --> %s" % str(answer)
    return answer


def getMetakitLibDir(BpsPath):
    return os.path.join(BpsPath,"lib","biocase","configtool","utilities","metakit")
        

def copytree(src, dst, symlinks=False, removeSuffix=[]):
    """Recursively copy a directory tree using copy2().

    Modified version of the shutils function.
    Does not raise erros when dirs already exist.
    
    The destination directory must not already exist.
    If exception(s) occur, an Error is raised with a list of reasons.

    If the optional symlinks flag is true, symbolic links in the
    source tree result in symbolic links in the destination tree; if
    it is false, the contents of the files pointed to by symbolic
    links are copied.
    """
    names = os.listdir(src)
    if not os.path.isdir(dst):
        os.mkdir(dst)
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        if symlinks and os.path.islink(srcname):
            linkto = os.readlink(srcname)
            os.symlink(linkto, dstname)
        elif os.path.isdir(srcname):
            copytree(srcname, dstname, symlinks, removeSuffix)
        else:
            shutil.copyfile(srcname, dstname)
    # remove files which end as suffix XXX in destination
    for suffix in removeSuffix:
        names = os.listdir(dst)
        for name in names:
            if name.endswith(suffix):
                dstname = os.path.join(dst, name)
                try:
                    os.remove(dstname)
                except OSError:
                    print "Could not remove file %s"%dstname
        

def upgradeInstallation(ThisBpsPath):
    searching=True
    while searching:
        BpsPath = askForParameter("Where is your existing BPS installation ?", isDir=True, default=DefaultBpsRootPath)
        oldVersion = getBPSversion(BpsPath)
        if oldVersion is not None:
            print "Found existing installation with version %s" % oldVersion
            searching = False
        else:
            print "The directory does not seem to contain a BPS2 installation. Try again."
    # verify that version of old installation >= v2
    if not oldVersion.startswith('2.'):
        print "Sorry, but your existing version %s is not supported by this upgrade script. Please do the upgrade manually." %(oldVersion)
        return
    # read existing .configuration
    setCfgClassLocators(BpsPath)
    cfg = Cfg('configtool','pywrapper')
    # backup existing installation
    if askForParameter("Do you want to backup your existing installation ?", boolean=True, default=True):
        # TODO: backup old installation: copy all files to biocaseBACKUP or biocaseBACKUP123)
        BpsPathBackup = BpsPath+"BACKUP"
        i = 1
        while os.path.isdir(BpsPathBackup):
            BpsPathBackup = BpsPath+"BACKUP%i"%i
            i += 1
        print "Backing up your entire existing installation to %s" % BpsPathBackup
        copytree(BpsPath, BpsPathBackup)
    # run adapt2 tool to make sure permissions are fine for the updating. suppress messages
    global OS
    os.chdir(os.path.join(BpsPath,'tools')) # change current working directory to adapt script
    # discard output
    sys.stdout = DevNull()
    if OS == "win":
        execfile( os.path.join(BpsPath,'tools','adapt2win.py' ))
    else:
        execfile( os.path.join(BpsPath,'tools','adapt2unix.py' ))
    sys.stdout = sys.__stdout__
    # remove unwanted files from existing installation
    pass
    # save the existing metakit lib in case its a special one (e.g. solaris non sparc)
    metalib = os.path.join( getMetakitLibDir(BpsPath),"Mk4py.so")
    if os.path.isfile(metalib):
        os.rename(metalib, metalib+".previous")
    # copy new files to existing installation. Overwrite esisting files but keep additional ones
    if not askForParameter("Do you want to update your existing installation? Additional files you have created will remain, but modified ones (like querytool skins) will be overwritten.", boolean=True, default=True):
        print "Upgrading canceled!"
        sys.exit()
    copyDirs = ['lib','www','examples','tools','configuration/configtool','configuration/persistence','configuration/pywrapper','configuration/querytool','configuration/schemas','configuration/templates']
    for d in copyDirs:
        print "Copying %s ..." % d
        copytree(os.path.join(ThisBpsPath, d), os.path.join(BpsPath, d), removeSuffix=['.pyc'])
    # reuse existing metakit lib if exists
    if os.path.isfile(metalib+".previous"):
        os.rename(metalib+".previous", metalib)
    # write .configuration files
    print "Writing back your old global configuration"
    cfg._write()    
    # run the makeGrove script to update db
    if askForParameter("It is recommended to update your configtool CMF concept index now, if you have your own custom CMFs defined.\nIf you haven't created your own new CMF templates you won't need to run this procedure.\nIt is used to find new concepts to map to but not essential for the wrapper to work.\nThis may take up to 20 minutes. You may skip this step and do it at a later stage manually by executing the makeGrove.py script inside the tools folder.\nDo you want to create the index now ?", boolean=True, default=False):
        os.chdir(os.path.join(BpsPath,'tools')) # change current working directory to adapt script
        execfile( os.path.join(BpsPath,'lib','biocase','adjustpath.py' ), globals())
        execfile( os.path.join(BpsPath,'tools','makeGrove.py' ), globals()) # , globals()
    else:
        print "Please remember to execute the makeGrove.py tools script at a later stage manually."
    print 
    print "The upgrade of your existing installation successfully is finished!"
    return BpsPath

############################################################################################################
#
# MAIN
#
#===========================================================================================================

if __name__ ==  "__main__":
    # gather system information
    OS                  = sys.platform # win32, linux1, linux2, sunos5, darwin1, darwin6
    while OS[-1].isdigit():
        OS = OS[:-1]
    Interpreter         = sys.executable
    BpsPath             = os.path.abspath('.')
    Version             = getBPSversion(BpsPath)
    DefaultBpsRootPath  = r"/usr/local/biocase"
    if OS == "win":
        DefaultBpsRootPath  = r"C:\biocase"
    
    # Intro text
    print '''
 +++  BioCASE Provider Software v2
 +++  Version %s
 +++  Installer script
 
 This setup tool adapts the BioCASE Provider Software (BPS) to your system environment. Upgrades of existing BPS installations are supported for version 2 or higher.'''%Version

    # check for python version.
    # only allow 2.2, 2.3 and 2.4
    if not (sys.version.startswith('2.2') or sys.version.startswith('2.3') or sys.version.startswith('2.4')):
        print '''Sorry. You will need Python version 2.2, 2.3 or 2.4 to run the software.
        Your current version %s of Python is not supported.
        
        Please install Python 2.4 and start the setup again.
        Thanks.
        ''' % string.split(sys.version, ' ')[0]
    
    # ask for upgrade
    print
    if askForParameter("Do you want to upgrade an existing installation ? \nYour existing configurations (e.g. datasources, querytool skins, new templates) will be kept, but modifications to default templates (querytool default skin) will be lost. \nIf desired the script will perform a backup of your entire installation to save this information before it gets updated.", boolean=True, default=False):
        BpsPath = upgradeInstallation(BpsPath)
        setCfgClassLocators(BpsPath)
        cfg = Cfg('configtool','pywrapper')
    else:
        cfg = Cfg('configtool','pywrapper')
        cfg.host    = askForParameter("Please indicate the domain of your webserver ?", default='http://localhost')
        cfg.webroot = askForParameter("Please indicate the base URL you are planning to use for the software ?", default='/biocase')        
        cfg._write()

    # run adapt2 tool
    os.chdir(os.path.join(BpsPath,'tools')) # change current working directory to adapt script
    if OS == "win":
        print "Adapting files to the Windows operating system."
        execfile( os.path.join(BpsPath,'tools','adapt2win.py' ))
    else:
        print "Adapting files to the Unix operating system."
        execfile( os.path.join(BpsPath,'tools','adapt2unix.py' ))
        
    # FINISHED
    print
    print
    print "Setup done."
    print "Please test your installation at %s%s" %(cfg.host, cfg.webroot)
        
