#!/usr/bin/python

# ****************************************
# +++  BioCASE 
# +++  Adapt files to Unix systems
#
# chmod, change shebang to "/usr/bin/env python.
#
# ****************************************


import sys, os, shutil

############################################################################################################
#
# FUNCTIONS
#
#===========================================================================================================
def updateShebang(fn):
	f = file(fn, 'r')
	text = f.readline()
	if text[:2] == "#!":
		#text = "#!%s\n" %"/usr/bin/env python"
		text = "#!%s\n" %(sys.executable)
	text += f.read()
	text = text.replace(chr(13), '')
	f.close()
	# write back
	fw = file(fn, 'w')
	fw.write(text)
	fw.close()
	

def adaptMetakitLib(metakitLibPath):
    '''Rename metakit shared objects to fit the current OS and python version.
    Needs the path to the metakit libraries.'''
    OS = sys.platform # win32, linux1, linux2, sunos5, darwin1, darwin6
    # rename metakit lib according to OS
    soLib = os.path.join(metakitLibPath,"Mk4py.so")
    # delete old one?
    if os.path.isfile(soLib):
        os.unlink(soLib)
    # copy new one
    if OS.startswith("sunos") and os.path.isfile(os.path.join(metakitLibPath,"Mk4py_solaris8Sparc.so")):
        soSrcLib = os.path.join(metakitLibPath,"Mk4py_solaris8Sparc.so")
        print "Using the metakit library for Solaris SPARC."
        shutil.copy(soSrcLib, soLib)
    elif OS.startswith("darwin") and os.path.isfile(os.path.join(metakitLibPath,"Mk4py_macosx.so")):
        soSrcLib = os.path.join(metakitLibPath,"Mk4py_macosx.so")
        print "Using the metakit library for Mac OS X."
        shutil.copy(soSrcLib, soLib)
    elif OS.startswith("linux") and os.path.isfile(os.path.join(metakitLibPath,"Mk4py_macosx.so")):
        soSrcLib = os.path.join(metakitLibPath,"Mk4py_linux.so")
        print "Using the metakit library for Linux."
        shutil.copy(soSrcLib, soLib)
    else:
        print "Unknown operating system %s found." % sys.platform
        print "Aborted."
        sys.exit()                

############################################################################################################
#
# MAIN
#
#===========================================================================================================

if __name__ ==  "__main__":

	print '''
 +++  BioCASE 
 +++  Adapt files to Unix systems

chmod, change shebang to %s.
''' %(sys.executable)
	
	adaptMetakitLib(os.path.abspath('../lib/biocase/configtool/utilities/metakit'))	
	scriptDirs 	= ['../www/', '.']
	suffices   	= ['.cgi', '.py']
	writeDirs  	= ['../configuration/', '../log/']
	execDirs   	= ['../lib/']
	writeExecDirs	= ['../www/configtool/']
	for d in writeDirs:
		d = os.path.abspath(d)
		print "setting write permissions for all users in directory %s" %d
		for root, dirs, files in os.walk(d):
			for name in files:
				f = os.path.join(root, name)
				print "  update %s" %f
				os.chmod(f, 0666)
			print "  -- update directories --"
			for name in dirs:
				f = os.path.join(root, name)
				print "  update %s" %f
				os.chmod(f, 0777)
			print "  update %s" %root
			os.chmod(root, 0777)

	for d in execDirs:
		d = os.path.abspath(d)
		print "setting execution permission for all users in directory %s" %d
		for root, dirs, files in os.walk(d):
			for name in files:
				for suff in suffices:
					if name[-len(suff):] == suff:
						f = os.path.join(root, name)
						print "  update %s" %f
						os.chmod(f, 0755)
		
	for d in writeExecDirs:
		d = os.path.abspath(d)
		print "setting execution & write permission for all users in directory %s" %d
		for root, dirs, files in os.walk(d):
			for name in files:
				for suff in suffices:
					if name[-len(suff):] == suff:
						f = os.path.join(root, name)
						print "  update %s" %f
						os.chmod(f, 0777)
			for name in dirs:
				f = os.path.join(root, name)
				print "  update %s" %f
				os.chmod(f, 0777)
			print "  update %s" %root
			os.chmod(root, 0777)

	for d in scriptDirs:
		d = os.path.abspath(d)
		print "updating scripts in directory %s" %d
		for root, dirs, files in os.walk(d):
			for name in files:
				for suff in suffices:
					if name[-len(suff):] == suff:
						# change this script
						f = os.path.join(root, name)
						print "  update %s" %f
						os.chmod(f, 0755)
						updateShebang(f)
		
